@@ -1,11 +1,15 @@
 /* gGraph Implemntation
  *
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
  * for more details. */
 
+#define TEST_MACROS_ENABLEDoff
+#include "test_macros.h"
+
 #include "Graphs/gGraph.h"
 
 #include <QLabel>
@@ -20,6 +24,64 @@
 
 extern MainWindow *mainwin;
 
+#if 0
+/*
+from qt 4.8
+int QGraphicsSceneWheelEvent::delta() const
+Returns the distance that the wheel is rotated, in eighths (1/8s) of a degree. A positive value indicates that the wheel was rotated forwards away from the user; a negative value indicates that the wheel was rotated backwards toward the user.
+
+int QWheelEvent::delta () const
+Returns the distance that the wheel is rotated, in eighths of a degree. A positive value indicates that the wheel was rotated forwards away from the user; a negative value indicates that the wheel was rotated backwards toward the user.
+
+Most mouse types work in steps of 15 degrees, in which case the delta value is a multiple of 120; i.e., 120 units * 1/8 = 15 degrees.
+
+However, some mice have finer-resolution wheels and send delta values that are less than 120 units (less than 15 degrees). To support this possibility, you can either cumulatively add the delta values from events until the value of 120 is reached, then scroll the widget, or you can partially scroll the widget in response to each wheel event.
+
+Example:
+
+ void MyWidget::wheelEvent(QWheelEvent *event)
+ {
+     int numDegrees = event->delta() / 8;
+     int numSteps = numDegrees / 15;
+
+     if ( isWheelEventHorizontal(event) )  {
+         scrollHorizontally(numSteps);
+     } else {
+         scrollVertically(numSteps);
+     }
+     event->accept();
+ }
+
+
+ from qt 5.15
+Returns the relative amount that the wheel was rotated, in eighths of a degree. A positive value indicates that the wheel was rotated forwards away from the user; a negative value indicates that the wheel was rotated backwards toward the user. angleDelta().y() provides the angle through which the common vertical mouse wheel was rotated since the previous event. angleDelta().x() provides the angle through which the horizontal mouse wheel was rotated, if the mouse has a horizontal wheel; otherwise it stays at zero. Some mice allow the user to tilt the wheel to perform horizontal scrolling, and some touchpads support a horizontal scrolling gesture; that will also appear in angleDelta().x().
+
+Most mouse types work in steps of 15 degrees, in which case the delta value is a multiple of 120; i.e., 120 units * 1/8 = 15 degrees.
+
+However, some mice have finer-resolution wheels and send delta values that are less than 120 units (less than 15 degrees). To support this possibility, you can either cumulatively add the delta values from events until the value of 120 is reached, then scroll the widget, or you can partially scroll the widget in response to each wheel event. But to provide a more native feel, you should prefer pixelDelta() on platforms where it's available.
+*/
+#endif
+
+// The qt5.15 obsolescence of hex requires this change.
+// this solution to QT's obsolescence is only used in debug statements
+#if QT_VERSION >= QT_VERSION_CHECK(5,15,0)
+    #define wheelEventPos( id )   id ->position()
+    #define wheelEventX( id )     id ->position().x()
+    #define wheelEventY( id )     id ->position().y()
+    #define wheelEventDelta( id ) id ->angleDelta().y()
+
+    #define isWheelEventVertical( id )  id ->angleDelta().x()==0
+    #define isWheelEventHorizontal( id )  id ->angleDelta().y()==0
+#else
+    #define wheelEventPos( id )   id ->pos()
+    #define wheelEventX( id )     id ->x()
+    #define wheelEventY( id )     id ->y()
+    #define wheelEventDelta( id ) id ->delta()
+
+    #define isWheelEventVertical( id )  id ->orientation() == Qt::Vertical
+    #define isWheelEventHorizontal( id )  id ->orientation() == Qt::Horizontal
+#endif
+
 // Graph globals.
 QFont *defaultfont = nullptr;
 QFont *mediumfont = nullptr;
@@ -130,6 +192,7 @@ gGraph::gGraph(QString name, gGraphView *graphview, QString title, QString units
       m_units(units),
       m_visible(true)
 {
+    // DEBUGF Q(name) Q(title) QQ("UNITS",units) Q(height) Q(group);
     if (height == 0) {
         height = AppSetting->graphHeight();
         Q_UNUSED(height)
@@ -139,7 +202,8 @@ gGraph::gGraph(QString name, gGraphView *graphview, QString title, QString units
         qDebug() << "Trying to duplicate " << name << " when a graph with the same name already exists";
         name+="-1";
     }
-    m_min_height = 60;
+    m_min_height = 60;      // this is the graphs minimum height.- does not consider the graphs layer height requirements. 
+    m_defaultLayerMinHeight = 25;   // this is the minimum requirements for the layer height. can be chnaged by a layer. // not changable
     m_width = 0;
 
     m_layers.clear();
@@ -152,7 +216,7 @@ gGraph::gGraph(QString name, gGraphView *graphview, QString title, QString units
     min_x = min_y = 0;
     rec_miny = rec_maxy = 0;
     rphysmax_y = rphysmin_y = 0;
-    m_zoomY = 0;
+    m_zoomY = ZS_AUTO_FIT;
     m_selectedDuration = 0;
 
     if (graphview) {
@@ -174,7 +238,6 @@ gGraph::gGraph(QString name, gGraphView *graphview, QString title, QString units
     m_pinned = false;
     m_lastx23 = 0;
 
-    invalidate_yAxisImage = true;
     invalidate_xAxisImage = true;
 
     m_block_select = false;
@@ -275,12 +338,28 @@ void gGraph::setDay(Day *day)
     }
 
     rmin_y = rmax_y = 0;
-    ResetBounds();
+    // This resets weight and bmi overview graphs to full date range when they are changed.
+    // is it required ever?
+    // ResetBounds();
 }
 
-void gGraph::setZoomY(short zoom)
-{
-    m_zoomY = zoom;
+void gGraph::setZoomY(ZoomyScaling zoomy) {
+    m_zoomY = zoomy;
+    dynamicScalingOn =false;
+    timedRedraw(0);
+}
+
+void gGraph::mouseDoubleClickYAxis(QMouseEvent * ) {
+    if ( isDynamicScalingEnabled() ) {
+        dynamicScalingOn = !dynamicScalingOn;
+    } else {
+        dynamicScalingOn =false;
+        if (m_zoomY == ZS_AUTO_FIT ) {
+            m_zoomY = ZS_DEFAULT;
+        } else if (m_zoomY == ZS_DEFAULT) {
+            m_zoomY = ZS_AUTO_FIT ;
+        }
+    }
     timedRedraw(0);
 }
 
@@ -561,22 +640,67 @@ void gGraph::ToolTip(QString text, int x, int y, ToolTipAlignment align, int tim
     m_graphview->m_tooltip->display(text, x, y, align, timeout);
 }
 
+bool gGraph::isDynamicScalingEnabled() {
+    return ((m_lineChart_layer!=nullptr) && AppSetting->allowYAxisScaling() );
+}
+
+QString gGraph::unitsTooltip() {
+    if (isDynamicScalingEnabled()) {
+        if(dynamicScalingOn) {
+            if (zoomY() == ZS_AUTO_FIT ) {
+                return QString("%1%2%3").arg(m_units).arg("\n").arg(tr("Double click Y-axis: Return to AUTO-FIT Scaling"));
+            } else if (zoomY() == ZS_DEFAULT ) {
+                return QString("%1%2%3").arg(m_units).arg("\n").arg(tr("Double click Y-axis: Return to DEFAULT Scaling"));
+            } else {
+                return QString("%1%2%3").arg(m_units).arg("\n").arg(tr("Double click Y-axis: Return to OVERRIDE Scaling"));
+            }
+        } else {
+            return QString("%1%2%3").arg(m_units).arg("\n").arg(tr("Double click Y-axis: For Dynamic Scaling"));
+        }
+    } else {
+        if (zoomY() == ZS_AUTO_FIT ) {
+            return QString("%1%2%3").arg(m_units).arg("\n").arg(tr("Double click Y-axis: Select DEFAULT Scaling"));
+        } else if (zoomY() == ZS_DEFAULT ) {
+            return QString("%1%2%3").arg(m_units).arg("\n").arg(tr("Double click Y-axis: Select AUTO-FIT Scaling"));
+        }
+    }
+    return m_units;
+}
+
+void gGraph::dynamicScaling(EventDataType &miny, EventDataType &maxy) {
+    // Have new Dynamic mode;
+    miny = m_lineChart_layer->actualMinY();
+    maxy = m_lineChart_layer->actualMaxY();
+    EventDataType diff= (maxy-miny);
+    maxy += diff*0.08;      // more space at top for event ticks.
+    miny -= diff*0.04;
+    if (m_saved_minY!=m_lineChart_layer->actualMinY() || m_saved_maxY!=m_lineChart_layer->actualMaxY()  ) {
+        // DEBUGF O(m_name) Q(m_saved_minY) QQ("==>",m_lineChart_layer->actualMinY() ) QQ("==>",miny) Q(m_saved_maxY) QQ("==>",m_lineChart_layer->actualMaxY()  )  QQ("==>",maxy);
+        m_saved_minY=m_lineChart_layer->actualMinY();
+        m_saved_maxY=m_lineChart_layer->actualMaxY();
+    }
+}
+
 // YAxis Autoscaling code
 void gGraph::roundY(EventDataType &miny, EventDataType &maxy)
 {
-
-    if (zoomY() == 2) {
+    if (dynamicScalingOn) {
+        dynamicScaling(miny, maxy) ;
+        if (maxy > miny) return;
+    };
+    if (zoomY() == ZS_OVERRIDE) {    // Have override mode
+        // set min and max to override values.
         miny = rec_miny;
         maxy = rec_maxy;
-        if (maxy > miny) return;
-    } else if (zoomY() ==1) {
+        if (maxy > miny) return; // Not Autoscaling
+    } else if (zoomY() ==ZS_DEFAULT) {  // Have Default mode
+        // set min and max to physical Min / max values.
         miny = physMinY();
         maxy = physMaxY();
-        if (maxy > miny) return;
+        if (maxy > miny) return; // Not Autoscaling
     }
     miny = MinY();
     maxy = MaxY();
-
     int m, t;
     bool ymin_good = false, ymax_good = false;
 
@@ -682,6 +806,11 @@ void gGraph::AddLayer(Layer *l, LayerPosition position, short width, short heigh
     l->setPos(x, y);
     l->addref();
     m_layers.push_back(l);
+
+    if (l->layerType()==LT_LineChart) {
+        m_lineChart_layer = dynamic_cast<gLineChart *>(l);
+    }
+
 }
 
 void gGraph::dataChanged()
@@ -704,6 +833,7 @@ double gGraph::currentTime() const
 {
     return m_graphview->currentTime();
 }
+
 double gGraph::screenToTime(int xpos)
 {
     double w = m_rect.width() - left - right;
@@ -792,8 +922,7 @@ void gGraph::mouseMoveEvent(QMouseEvent *event)
             if (d > 1) {
                 m_selDurString = tr("%1 days").arg(floor(d));
             } else {
-
-                m_selDurString.sprintf("%02i:%02i:%02i:%03i", h, m, s, ms);
+                m_selDurString=QString::asprintf("%02i:%02i:%02i:%03i", h, m, s, ms);
             }
 
             ToolTipAlignment align = x >= x2 ? TT_AlignLeft : TT_AlignRight;
@@ -882,18 +1011,6 @@ void gGraph::mousePressEvent(QMouseEvent *event)
                 return;
             }
     }
-
-    /*
-    int w=m_lastbounds.width()-(right+m_marginright);
-    //int h=m_lastbounds.height()-(bottom+m_marginbottom);
-    //int x2,y2;
-    double xx=max_x-min_x;
-    //double xmult=xx/w;
-    if (x>left+m_marginleft && x<m_lastbounds.width()-(right+m_marginright) && y>top+m_margintop && y<m_lastbounds.height()-(bottom+m_marginbottom)) { // main area
-        x-=left+m_marginleft;
-        y-=top+m_margintop;
-    }*/
-    //qDebug() << m_title << "Clicked" << x << y << left << right << top << bottom << m_width << m_height;
 }
 
 void gGraph::mouseReleaseEvent(QMouseEvent *event)
@@ -997,6 +1114,15 @@ void gGraph::mouseReleaseEvent(QMouseEvent *event)
 
     if ((m_graphview->horizTravel() < mouse_movement_threshold) && (x > left && x < w + left
             && y > top && y < h)) {
+        if ((event->modifiers() & Qt::ShiftModifier) != 0) {
+            qint64 time = (qint64)screenToTime(x);
+            qint64 period =qint64(p_profile->general->eventWindowSize())*60000L;  // eventwindowsize units  minutes
+            qint64 small  =period/10;
+            qint64 start = time - period;
+            qint64 end   = time + small;
+            m_graphview->SetXBounds(start, end);
+            return;
+        }
         // normal click in main area
         if (!m_blockzoom) {
             double zoom;
@@ -1071,23 +1197,23 @@ void gGraph::mouseReleaseEvent(QMouseEvent *event)
 
 void gGraph::wheelEvent(QWheelEvent *event)
 {
-    qDebug() << m_title << "Wheel" << event->x() << event->y() << event->delta();
+    //qDebug() << m_title << "Wheel" << wheelEventX(event) << wheelEventY(event) << wheelEventDelta(event);
     //int y=event->pos().y();
-    if (event->orientation() == Qt::Horizontal) {
+    if ( isWheelEventHorizontal(event) ) {
 
         return;
     }
 
-    int x = event->pos().x() - m_graphview->titleWidth; //(left+m_marginleft);
+    int x = wheelEventPos( event).x() - m_graphview->titleWidth; //(left+m_marginleft);
 
-    if (event->delta() > 0) {
+    if (wheelEventDelta(event) > 0) {
         ZoomX(0.75, x);
     } else {
         ZoomX(1.5, x);
     }
 
-    int y = event->pos().y();
-    x = event->pos().x();
+    int y = wheelEventPos(event).y();
+    x = wheelEventPos(event).x();
 
     for (const auto & layer : m_layers) {
         if (layer->m_rect.contains(x, y)) {
@@ -1108,24 +1234,6 @@ void gGraph::mouseDoubleClickEvent(QMouseEvent *event)
             layer->mouseDoubleClickEvent(event, this);
         }
     }
-
-    //int w=m_lastbounds.width()-(m_marginleft+left+right+m_marginright);
-    //int h=m_lastbounds.height()-(bottom+m_marginbottom);
-    //int x2=m_graphview->pointClicked().x(),y2=m_graphview->pointClicked().y();
-    //    if ((m_graphview->horizTravel()<mouse_movement_threshold) && (x>left+m_marginleft && x<w+m_marginleft+left && y>top+m_margintop && y<h)) { // normal click in main area
-    //        if (event->button() & Qt::RightButton) {
-    //            ZoomX(1.66,x);  // Zoon out
-    //            return;
-    //        } else if (event->button() & Qt::LeftButton) {
-    //            ZoomX(0.75/2.0,x); // zoom in.
-    //            return;
-    //        }
-    //    } else {
-    // Propagate the events to graph Layers
-    //    }
-    //mousePressEvent(event);
-    //mouseReleaseEvent(event);
-    //qDebug() << m_title << "Double Clicked" << event->x() << event->y();
 }
 void gGraph::keyPressEvent(QKeyEvent *event)
 {
@@ -1212,7 +1320,6 @@ void gGraph::DrawTextQue(QPainter &painter)
 void gGraph::resize(int width, int height)
 {
     invalidate_xAxisImage = true;
-    invalidate_yAxisImage = true;
 
     Q_UNUSED(width);
     Q_UNUSED(height);
@@ -1298,6 +1405,7 @@ EventDataType gGraph::MinY()
     }
 
     return rmin_y = val;
+//    return rmin_y = val * 0.9;
 }
 EventDataType gGraph::MaxY()
 {
@@ -1412,30 +1520,24 @@ void gGraph::SetMaxY(EventDataType v)
 
 Layer *gGraph::getLineChart()
 {
-    gLineChart *lc;
-
+    if (m_lineChart_layer) return m_lineChart_layer;
     for (auto & layer : m_layers) {
-        lc = dynamic_cast<gLineChart *>(layer);
-
-        if (lc) { return lc; }
+        Layer *tmp = dynamic_cast<gLineChart *>(layer);
+        if (tmp) m_lineChart_layer = tmp;
     }
-
     return nullptr;
 }
+
 int gGraph::minHeight()
 {
-    int minheight = m_min_height;
-
-//    int top = 0;
-//    int center = 0;
-//    int bottom = 0;
+    // adjust graph height for centerLayer (ploting area) required height.
+    int adjustment  = top + bottom;    //adjust graph minimun to layer minimum
+    int minlayerheight = m_min_height  - adjustment;
     for (const auto & layer : m_layers) {
-        int mh = layer->minimumHeight();
-        mh += m_margintop + m_marginbottom;
-        if (mh > minheight) minheight = mh;
+        if (layer->position() != LayerCenter) continue;
+        minlayerheight = max(max (m_defaultLayerMinHeight,layer->minimumHeight()),minlayerheight);
     }
-    // layers need to set their own too..
-    return minheight;
+    return minlayerheight + adjustment; // adjust layer min to graph minimum
 }
 
 int GetXHeight(QFont *font)
