@@ -1,6 +1,7 @@
 ﻿/* gGraphView Header
  *
- * Copyright (c) 2011-2015 Mark Watkins <jedimark@users.sourceforge.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2015 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
@@ -25,6 +26,7 @@
 #include <QDoubleSpinBox>
 #include <QToolButton>
 #include <QTimer>
+#include <QElapsedTimer>
 #include <QGestureEvent>
 #include <QPinchGesture>
 
@@ -102,7 +104,7 @@ public:
     QFont m_font;
     QString m_text;
     Qt::Alignment m_alignment;
-    QTime time;
+    QElapsedTimer time;
 protected slots:
     void doRedraw();
 
@@ -233,16 +235,18 @@ class gToolTip : public QObject
     /*! \fn virtual void display(QString text, int x, int y, int timeout=2000);
         \brief Set the tooltips display message, position, and timeout value
         */
-    virtual void display(QString text, int x, int y, ToolTipAlignment align = TT_AlignCenter, int timeout = 0);
+    virtual void display(QString text, int x, int y, ToolTipAlignment align = TT_AlignCenter, int timeout = 0,bool alwaysShow=false);
 
     //! \brief Draw the tooltip
     virtual void paint(QPainter &paint); //actually paints it.
 
     //! \brief Close the tooltip early.
-    void cancel();
+    virtual void cancel();
 
     //! \brief Returns true if the tooltip is currently visible
-    bool visible() { return m_visible; }
+    virtual bool visible() { return m_visible; }
+
+    virtual QRect calculateRect(QPainter &painter);
 
   protected:
     gGraphView *m_graphview;
@@ -253,13 +257,37 @@ class gToolTip : public QObject
     bool m_visible;
     int m_spacer;
     QImage m_image;
-    bool m_invalidate;
     ToolTipAlignment m_alignment;
+    QFont* m_font=defaultfont;
 
   protected slots:
 
     //! \brief Timeout to hide tooltip, and redraw without it.
-    void timerDone();
+    virtual void timerDone();
+};
+
+class gParentToolTip : public gToolTip
+{
+    Q_OBJECT
+  public:
+    gParentToolTip(gGraphView *graphview);
+    ~gParentToolTip();
+    using gToolTip::display;
+    virtual void display(gGraphView* gv,QString text,int verticalOffset, int alignOffset , ToolTipAlignment align = TT_AlignCenter, int timeout = 0,QFont *font = defaultfont) ;
+    virtual void cancel();
+    virtual bool visible();
+    virtual QRect calculateRect(QPainter &painter);
+    using gToolTip::paint;
+    virtual void paint(QPainter &paint,int width,int height) ;
+  private:
+    int m_verticalOffset;
+    int m_alignOffset;
+    int m_width;
+    int m_height;
+    bool m_parent_visible;
+    int m_timeout;
+  protected slots:
+    virtual void timerDone();
 };
 
 struct SelectionHistoryItem {
@@ -369,11 +397,13 @@ class gGraphView
     //! \brief Supplies time range to all graph objects in linked group, refreshing if requested
     void SetXBounds(qint64 minx, qint64 maxx, short group = 0, bool refresh = true);
 
+    QString settingsFilename (QString title,QString folderName="" ,QString ext=".shg");
+
     //! \brief Saves the current graph order, heights, min & Max Y values to disk
-    void SaveSettings(QString title);
+    void SaveSettings(QString title,QString folderName="");
 
     //! \brief Loads the current graph order, heights, min & max Y values from disk
-    bool LoadSettings(QString title);
+    bool LoadSettings(QString title,QString folderName="");
 
     //! \brief Saves the current (initial) graph order, heights, min & Max Y values for future recovery
     void SaveDefaultSettings();
@@ -424,6 +454,7 @@ class gGraphView
 
     gGraph *m_selected_graph;
     gToolTip *m_tooltip;
+    gParentToolTip *m_parent_tooltip;
     QTimer *timer;
 
     //! \brief Add the Text information to the Text Drawing Queue (called by gGraphs renderText method)
@@ -687,7 +718,7 @@ class gGraphView
 
     QPixmapCache pixmapcache;
 
-    QTime horizScrollTime, vertScrollTime;
+    QElapsedTimer horizScrollTime, vertScrollTime;
     QMenu * context_menu;
     QAction * pin_action;
     QAction * popout_action;
@@ -706,6 +737,7 @@ class gGraphView
     void updateCurrentTime(double);
     void updateRange(double,double);
     void GraphsChanged();
+    void XBoundsChanged(qint64 ,qint64);
 
   public slots:
     //! \brief Callback from the ScrollBar, to change scroll position
