@@ -1,6 +1,7 @@
 ﻿/* gLineOverlayBar Implementation
  *
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
@@ -28,7 +29,6 @@ void gLineOverlayBar::paint(QPainter &painter, gGraph &w, const QRegion &region)
     if (!schema::channel[m_code].enabled())
         return;
 
-
     int left = region.boundingRect().left();
     int topp = region.boundingRect().top(); // FIXME: Misspelling intentional.
     double width = region.boundingRect().width();
@@ -42,10 +42,12 @@ void gLineOverlayBar::paint(QPainter &painter, gGraph &w, const QRegion &region)
 
     double xx = w.max_x - w.min_x;
     //double yy = w.max_y - w.min_y;
+
+    if (xx <= 0) { return; }
+
     double jj = width / xx;
 
 
-    if (xx <= 0) { return; }
 
     double x1, x2;
 
@@ -138,12 +140,20 @@ void gLineOverlayBar::paint(QPainter &painter, gGraph &w, const QRegion &region)
                     x1 = jj * double(X - w.min_x);
                     x2 = jj * double(Y - w.min_x);
 
-                    x2 += (int(x1)==int(x2)) ? 1 : 0;
-
                     x2 = qMax(0.0, x2)+left;
                     x1 = qMin(width, x1)+left;
 
-                    painter.fillRect(QRect(x2, start_py, x1-x2, height), brush);
+                    // x2 represents the begining of a span in pixels
+                    // x1 represent the end of the span in pixels
+                    // BUG HERE
+                    //x2 += (int(x1)==int(x2)) ? 1 : 0;
+                    // Fixed BY 
+                    int duration = x1-x2;
+                    if (duration<2) duration=2;     // display minial span with 2 pixels.
+                    x2 =x1-duration;
+
+                    painter.fillRect(QRect(x2, start_py, duration, height), brush);
+
                 }
             }/* else if (m_flt == FT_Dot) {
                 ////////////////////////////////////////////////////////////////////////////
@@ -367,7 +377,7 @@ void gLineOverlaySummary::paint(QPainter &painter, gGraph &w, const QRegion &reg
         a = QObject::tr("Duration")+": "+w.selDurString();
     } else {
         a = QObject::tr("Events") + ": " + QString::number(cnt) + ", " +
-            QObject::tr("Duration") + " " + QString().sprintf("%02i:%02i:%02i", h, m, s) + ", " +
+            QObject::tr("Duration") + " " + QString::asprintf("%02i:%02i:%02i", h, m, s) + ", " +
             m_text + ": " + QString::number(val, 'f', 2);
     }
     if (isSpan) {
