@@ -3,7 +3,7 @@
  * This isolates the initialization and its dependencies from the header file,
  * which is widely included.
  *
- * Copyright (c) 2020 The OSCAR Team
+ * Copyright (c) 2020-2024 The OSCAR Team
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the sourcecode. */
@@ -27,6 +27,7 @@ AppWideSetting::AppWideSetting(Preferences *pref) : PrefSettings(pref)
 //    initPref(STR_AS_GraphSnapshots, true);
     initPref(STR_AS_IncludeSerial, false);
     initPref(STR_AS_MonochromePrinting, false);
+    //initPref(STR_AS_EventFlagSessionBar, false);
     initPref(STR_AS_ShowPieChart, false);
     m_animations = initPref(STR_AS_Animations, true).toBool();
     m_squareWavePlots = initPref(STR_AS_SquareWave, false).toBool();
@@ -34,7 +35,11 @@ AppWideSetting::AppWideSetting(Preferences *pref) : PrefSettings(pref)
     m_graphTooltips = initPref(STR_AS_GraphTooltips, true).toBool();
     m_usePixmapCaching = initPref(STR_AS_UsePixmapCaching, false).toBool();
     m_odt = (OverlayDisplayType)initPref(STR_AS_OverlayType, (int)ODT_Bars).toInt();
+    initPref(STR_AS_GraphTooltips, 0);
+    m_alternatingColorsCombo = initPref(STR_AS_setAlternatingColorsCombo, 0).toInt(); 
+#ifndef REMOVE_FITNESS
     m_olm = (OverviewLinechartModes)initPref(STR_AS_OverviewLinechartMode, (int)OLC_Bartop).toInt();
+#endif
     m_lineThickness=initPref(STR_AS_LineThickness, 1.0).toFloat();
     m_lineCursorMode = initPref(STR_AS_LineCursorMode, true).toBool();
     initPref(STR_AS_RightSidebarVisible, false);
