@@ -2,8 +2,8 @@
  *
  * This file for all settings related stuff to clean up Preferences & Profiles.
  *
- * Copyright (c) 2020 The OSCAR Team
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the sourcecode. */
@@ -18,8 +18,14 @@
 
 class Preferences;
 
-enum OverviewLinechartModes { OLC_Bartop, OLC_Lines };
+#define REMOVE_FITNESS_OFF
+/* valid values are REMOVE_FITNESS or REMOVE_FITNESS_OFF */
 
+#ifndef REMOVE_FITNESS
+enum OverviewLinechartModes { OLC_Bartop, OLC_Lines };
+#endif
+
+#define REMSTAR_M_SUPPORTdisabled
 
 // ApplicationWideSettings Strings
 const QString STR_CS_UserEventPieChart = "UserEventPieChart";
@@ -33,11 +39,15 @@ const QString STR_AS_ShowPieChart = "EnablePieChart";
 const QString STR_AS_Animations = "AnimationsAndTransitions";
 const QString STR_AS_SquareWave = "SquareWavePlots";
 const QString STR_AS_OverlayType = "OverlayType";
+const QString STR_AS_setAlternatingColorsCombo = "AlternatingColorsCombo";
+#ifndef REMOVE_FITNESS
 const QString STR_AS_OverviewLinechartMode = "OverviewLinechartMode";
+#endif
 const QString STR_AS_UsePixmapCaching = "UsePixmapCaching";
 const QString STR_AS_AllowYAxisScaling = "AllowYAxisScaling";
 const QString STR_AS_IncludeSerial = "IncludeSerial";
 const QString STR_AS_MonochromePrinting = "PrintBW";
+//const QString STR_AS_EventFlagSessionBar = "EventFlagSessionBar";
 const QString STR_AS_GraphTooltips = "GraphTooltips";
 const QString STR_AS_LineThickness = "LineThickness";
 const QString STR_AS_LineCursorMode = "LineCursorMode";
@@ -78,11 +88,14 @@ public:
   bool m_usePixmapCaching, m_antiAliasing, m_squareWavePlots,m_graphTooltips, m_lineCursorMode, m_animations;
   bool m_showPerformance, m_showDebug;
   int m_tooltipTimeout, m_graphHeight, m_scrollDampening;
+  int m_alternatingColorsCombo;
   bool m_multithreading, m_cacheSessions;
   float m_lineThickness;
 
   OverlayDisplayType m_odt;
+#ifndef REMOVE_FITNESS
   OverviewLinechartModes m_olm;
+#endif
   QString m_profileName, m_language;
 
   QString versionString() const { return getPref(STR_PREF_VersionString).toString(); }
@@ -118,17 +131,20 @@ public:
   //! \brief Returns true if Graphical animations & Transitions will be drawn
   bool animations() const { return m_animations; }
   //! \brief Returns true if PixmapCaching acceleration will be used
-  inline bool usePixmapCaching() const { return m_usePixmapCaching; }
+  inline bool usePixmapCaching() const { return /*m_usePixmapCaching*/false; } //disables use pixmap caching without modifing any code.
   //! \brief Returns true if Square Wave plots are preferred (where possible)
   inline bool squareWavePlots() const { return m_squareWavePlots; }
   //! \brief Whether to allow double clicking on Y-Axis labels to change vertical scaling mode
   bool allowYAxisScaling() const { return getPref(STR_AS_AllowYAxisScaling).toBool(); }
-  //! \brief Whether to include serial number in machine settings changes report
+  //! \brief Whether to include serial number in device settings changes report
   bool includeSerial() const { return getPref(STR_AS_IncludeSerial).toBool(); }
   //! \brief Whether to print reports in black and white, which can be more legible on non-color printers
   bool monochromePrinting() const { return getPref(STR_AS_MonochromePrinting).toBool(); }
+  //bool eventFlagSessionBar() const { return getPref(STR_AS_EventFlagSessionBar).toBool(); }
+  //! \Allow disabling of sessions
   //! \brief Whether to show graph tooltips
   inline bool graphTooltips() const { return m_graphTooltips; }
+  inline int  alternatingColorsCombo() { return m_alternatingColorsCombo;}
   //! \brief Pen width of line plots
   inline float lineThickness() const { return m_lineThickness; }
   //! \brief Whether to show line cursor
@@ -137,8 +153,10 @@ public:
   bool rightSidebarVisible() const { return getPref(STR_AS_RightSidebarVisible).toBool(); }
   //! \brief Returns the type of overlay flags (which are displayed over the Flow Waveform)
   inline OverlayDisplayType overlayType() const { return m_odt; }
+#ifndef REMOVE_FITNESS
   //! \brief Returns the display type of Overview pages linechart
   inline OverviewLinechartModes overviewLinechartMode() const { return m_olm; }
+#endif
   bool userEventPieChart() const { return getPref(STR_CS_UserEventPieChart).toBool(); }
   bool showSerialNumbers() const { return getPref(STR_US_ShowSerialNumbers).toBool(); }
   int openTabAtStart() const { return getPref(STR_US_OpenTabAtStart).toInt(); }
@@ -180,14 +198,18 @@ public:
   void setOverlayType(OverlayDisplayType odt) { setPref(STR_AS_OverlayType, (int)(m_odt=odt)); }
   //! \brief Sets whether to allow double clicking on Y-Axis labels to change vertical scaling mode
   void setAllowYAxisScaling(bool b) { setPref(STR_AS_AllowYAxisScaling, b); }
-  //! \brief Sets whether to include machine serial number on machine settings report
+  //! \brief Sets whether to include device serial number on device settings report
   void setIncludeSerial(bool b) { setPref(STR_AS_IncludeSerial, b); }
   //! \brief Sets whether to print reports in black and white, which can be more legible on non-color printers
   void setMonochromePrinting(bool b) { setPref(STR_AS_MonochromePrinting, b); }
+  // void setEventFlagSessionBar(bool b) { setPref(STR_AS_EventFlagSessionBar, b); }
   //! \brief Sets whether to allow double clicking on Y-Axis labels to change vertical scaling mode
   void setGraphTooltips(bool b) { setPref(STR_AS_GraphTooltips, m_graphTooltips=b); }
   //! \brief Sets the type of overlay flags (which are displayed over the Flow Waveform)
+  void setAlternatingColorsCombo(int b) { setPref(STR_AS_setAlternatingColorsCombo, m_alternatingColorsCombo=b); }
+#ifndef REMOVE_FITNESS
   void setOverviewLinechartMode(OverviewLinechartModes olm) { setPref(STR_AS_OverviewLinechartMode, (int)(m_olm=olm)); }
+#endif
   //! \brief Set the pen width of line plots.
   void setLineThickness(float size) { setPref(STR_AS_LineThickness, m_lineThickness=size); }
   //! \brief Sets whether to display Line Cursor
