@@ -1,6 +1,7 @@
 /* Common code and junk
  *
- * Copyright (C) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (C) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
@@ -14,7 +15,7 @@
 #include <QObject>
 #include <QThread>
 
-#define DEBUG_EFFICIENCY 1
+// #define DEBUG_EFFICIENCY 1   // Developers can define this for qmake if they want it
 
 #include <QLocale>
 #include "Graphs/glcommon.h"
@@ -67,6 +68,7 @@ struct ValueCount {
         :value(val), count(cnt), p(pp) {}
 
     ValueCount(const ValueCount &copy) = default;
+    ~ValueCount() {};
     EventDataType value;
     qint64 count;
     double p;
@@ -74,7 +76,7 @@ struct ValueCount {
 
 extern int idealThreads();
 
-void copyPath(QString src, QString dst);
+void copyPath(QString src, QString dst, bool overwrite=false);
 
 
 // Primarily sort by value
@@ -82,6 +84,13 @@ bool operator <(const ValueCount &a, const ValueCount &b);
 
 const float ounce_convert = 28.3495231F; // grams
 const float pound_convert = ounce_convert * 16;
+const float gram_ounce_convert = 0.0352754; // ounces in a gram
+const float pounds_per_kg = 2.2046226218 ; // kg to pounds
+const float kgs_per_pound = (1.0/pounds_per_kg) ; //pounds to kg.
+const float cms_per_inch  = 2.54 ; // inches to cm
+const float cms_per_foot  = (12.0*cms_per_inch) ; // feet to cm
+const float inches_per_cm = (1.0/cms_per_inch) ; // cm to inch
+
 
 QString weightString(float kg, UnitSystem us = US_Undefined);
 
@@ -149,6 +158,7 @@ const QString STR_GEN_DataFolder = "DataFolder";
 
 const QString STR_PREF_ReimportBackup = "ReimportBackup";
 const QString STR_PREF_LastCPAPPath = "LastCPAPPath";
+const QString STR_PREF_LastJournalPath = "LastJouralPath";
 const QString STR_PREF_LastOximetryPath = "LastOximetryPath";
 
 const QString STR_MACH_ResMed = "ResMed";
@@ -157,9 +167,11 @@ const QString STR_MACH_Journal = "Journal";
 const QString STR_MACH_Intellipap = "Intellipap";
 const QString STR_MACH_Weinmann= "Weinmann";
 const QString STR_MACH_FPIcon = "FPIcon";
+const QString STR_MACH_SleepStyle = "SleepStyle";
 const QString STR_MACH_MSeries = "MSeries";
 const QString STR_MACH_CMS50 = "CMS50";
 const QString STR_MACH_ZEO = "Zeo";
+const QString STR_MACH_Prisma = "Prisma";
 
 const QString STR_PREF_Language = "Language";
 
@@ -183,6 +195,7 @@ extern QString STR_UNIT_CMH2O;
 extern QString STR_UNIT_Hours;
 extern QString STR_UNIT_Minutes;
 extern QString STR_UNIT_Seconds;
+extern QString STR_UNIT_milliSeconds;
 extern QString STR_UNIT_h; // (h)ours, (m)inutes, (s)econds
 extern QString STR_UNIT_m;
 extern QString STR_UNIT_s;
@@ -242,11 +255,14 @@ extern QString STR_TR_Inclination;
 extern QString STR_TR_Orientation;
 extern QString STR_TR_Motion;
 
-// Machine type names.
+// Device type names.
 extern QString STR_TR_CPAP;    // Constant Positive Airway Pressure
 extern QString STR_TR_BIPAP;   // Bi-Level Positive Airway Pressure
 extern QString STR_TR_BiLevel; // Another name for BiPAP
 extern QString STR_TR_EPAP;    // Expiratory Positive Airway Pressure
+extern QString STR_TR_EEPAP;    // Expiratory Positive Airway Pressure
+extern QString STR_TR_EEPAPLo;  // End-Expiratory Positive Airway Pressure, Low
+extern QString STR_TR_EEPAPHi;  // End-Expiratory Positive Airway Pressure, High
 extern QString STR_TR_EPAPLo;  // Expiratory Positive Airway Pressure, Low
 extern QString STR_TR_EPAPHi;  // Expiratory Positive Airway Pressure, High
 extern QString STR_TR_IPAP;    // Inspiratory Positive Airway Pressure
@@ -261,6 +277,7 @@ extern QString STR_TR_Humidifier;
 
 extern QString STR_TR_H;       // Short form of Hypopnea
 extern QString STR_TR_OA;      // Short form of Obstructive Apnea
+extern QString STR_TR_A;       // Short form of Apnea
 extern QString STR_TR_UA;      // Short form of Unspecified Apnea
 extern QString STR_TR_CA;      // Short form of Clear Airway Apnea
 extern QString STR_TR_FL;      // Short form of Flow Limitation
@@ -269,13 +286,13 @@ extern QString STR_TR_LE;      // Short form of Leak Event
 extern QString STR_TR_EP;      // Short form of Expiratory Puff
 extern QString STR_TR_VS;      // Short form of Vibratory Snore
 extern QString
-STR_TR_VS2;     // Short form of Secondary Vibratory Snore (Some Philips Respironics Machines have two sources)
+STR_TR_VS2;     // Short form of Secondary Vibratory Snore (Some Philips Respironics devices have two sources)
 extern QString STR_TR_RERA;    // Acronym for Respiratory Effort Related Arousal
 extern QString STR_TR_PP;      // Short form for Pressure Pulse
 extern QString STR_TR_P;       // Short form for Pressure Event
 extern QString STR_TR_RE;      // Short form of Respiratory Effort Related Arousal
 extern QString STR_TR_NR;      // Short form of Non Responding event? (forgot sorry)
-extern QString STR_TR_NRI;     // Sorry I Forgot.. it's a flag on Intellipap machines
+extern QString STR_TR_NRI;     // Sorry I Forgot.. it's a flag on Intellipap devices
 extern QString STR_TR_O2;      // SpO2 Desaturation
 extern QString STR_TR_PC;      // Short form for Pulse Change
 extern QString STR_TR_UF1;     // Short form for User Flag 1
@@ -360,6 +377,7 @@ extern QString STR_TR_Start;
 extern QString STR_TR_End;
 extern QString STR_TR_On;
 extern QString STR_TR_Off;
+extern QString STR_TR_Auto;
 extern QString STR_TR_Yes;
 extern QString STR_TR_No;
 
