@@ -1,12 +1,15 @@
 /* SleepLib Day Class Implementation
  *
- * Copyright (c) 2019 The OSCAR Team
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
  * for more details. */
 
+#define TEST_MACROS_ENABLEDoff
+#include <test_macros.h>
+
 #include <QMultiMap>
 
 #include <algorithm>
@@ -98,6 +101,16 @@ Session *Day::find(SessionID sessid)
     return nullptr;
 }
 
+Session *Day::find(SessionID sessid, MachineType mt)
+{
+    for (auto & sess : sessions) {
+        if ((sess->session() == sessid) && (sess->s_machtype == mt)) {
+            return sess;
+        }
+    }
+    return nullptr;
+}
+
 void Day::addSession(Session *s)
 {
     if (s == nullptr) {
@@ -163,20 +176,20 @@ QString Day::calcMiddleLabel(ChannelID code)
 {
     int c = p_profile->general->prefCalcMiddle();
     if (c == 0) {
-        return QObject::tr("%1 %2").arg(STR_TR_Median).arg(schema::channel[code].label());
+        return QString("%1 %2").arg(STR_TR_Median).arg(schema::channel[code].label());
     } else if (c == 1) {
-        return QObject::tr("%1 %2").arg(STR_TR_WAvg).arg(schema::channel[code].label());
+        return QString("%1 %2").arg(STR_TR_WAvg).arg(schema::channel[code].label());
     } else {
-        return QObject::tr("%1 %2").arg(STR_TR_Avg).arg(schema::channel[code].label());
+        return QString("%1 %2").arg(STR_TR_Avg).arg(schema::channel[code].label());
     }
 }
 QString Day::calcMaxLabel(ChannelID code)
 {
-    return QObject::tr("%1 %2").arg(p_profile->general->prefCalcMax() ? QObject::tr("Peak") : STR_TR_Max).arg(schema::channel[code].label());
+    return QString("%1 %2").arg(p_profile->general->prefCalcMax() ? QString("99.5%") : STR_TR_Max).arg(schema::channel[code].label());
 }
 QString Day::calcPercentileLabel(ChannelID code)
 {
-    return QObject::tr("%1% %2").arg(p_profile->general->prefCalcPercentile(),0, 'f',0).arg(schema::channel[code].label());
+    return QString("%1% %2").arg(p_profile->general->prefCalcPercentile(),0, 'f',0).arg(schema::channel[code].label());
 }
 
 EventDataType Day::countInsideSpan(ChannelID span, ChannelID code)
@@ -194,6 +207,7 @@ EventDataType Day::lookupValue(ChannelID code, qint64 time, bool square)
 {
     for (auto & sess : sessions) {
         if (sess->enabled()) {
+            // drift is handled by session first/last/SearchValue
             if ((time > sess->first()) && (time < sess->last())) {
                 if (sess->channelExists(code)) {
                     return sess->SearchValue(code,time,square);
@@ -252,7 +266,7 @@ EventDataType Day::settings_max(ChannelID code)
     EventDataType value;
 
     for (auto & sess : sessions) {
-        if (sess->enabled()) {
+        if (sess->enabled() && sess->s_machtype != MT_JOURNAL) {
             value = sess->settings.value(code, min).toFloat();
             if (value > max) {
                 max = value;
@@ -269,7 +283,7 @@ EventDataType Day::settings_min(ChannelID code)
     EventDataType value;
 
     for (auto & sess : sessions) {
-        if (sess->enabled()) {
+        if (sess->enabled() && sess->s_machtype != MT_JOURNAL) {
             value = sess->settings.value(code, max).toFloat();
             if (value < min) {
                 min = value;
@@ -625,6 +639,12 @@ EventDataType Day::sum(ChannelID code)
     // Cache this?
     EventDataType val = 0;
 
+    if (code == AllAhiChannels) {
+        for (int i = 0; i < ahiChannels.size(); i++)
+            val += sum(ahiChannels.at(i));
+        return val;
+    }
+
     for (auto & sess : sessions) {
         if (sess->enabled() && sess->m_sum.contains(code)) {
             val += sess->sum(code);
@@ -735,7 +755,7 @@ qint64 Day::total_time()
     return total; //d_totaltime;
 }
 
-// Total session time in milliseconds, only considering machinetype
+// Total session time in milliseconds, only considering device type
 qint64 Day::total_time(MachineType type)
 {
     qint64 d_totaltime = 0;
@@ -822,18 +842,20 @@ ChannelID Day::getPressureChannelID() {
     // And why would ASV or AVAPS have Pressure channels?
     QList<ChannelID> preferredIDs = { CPAP_Pressure, CPAP_PressureSet, CPAP_IPAP, CPAP_IPAPSet };
     if (cpapmode == MODE_ASV || cpapmode == MODE_ASV_VARIABLE_EPAP || cpapmode == MODE_AVAPS ||
-        cpapmode == MODE_BILEVEL_FIXED || cpapmode == MODE_BILEVEL_AUTO_FIXED_PS || cpapmode == MODE_BILEVEL_AUTO_VARIABLE_PS) {
+        cpapmode == MODE_BILEVEL_FIXED || cpapmode == MODE_BILEVEL_AUTO_FIXED_PS || cpapmode == MODE_BILEVEL_AUTO_VARIABLE_PS || cpapmode == MODE_TRILEVEL_AUTO_VARIABLE_PDIFF) {
         preferredIDs = { CPAP_IPAP, CPAP_IPAPSet, CPAP_Pressure, CPAP_PressureSet };
     }
 
     for (auto & preferredID : preferredIDs) {
         // If preferred channel has data, return it
         if (channelHasData(preferredID)) {
-            //qDebug() << QString("Found pressure channel 0x%1").arg(preferredID, 4, 16, QChar('0'));
+//          if ( cpapmode == MODE_AVAPS )
+//              qDebug() << QString("Found pressure channel 0x%1").arg(preferredID, 4, 16, QChar('0'));
             return preferredID;
         }
     }
     
+    qDebug() << "No pressure channel for " << getCPAPModeStr();
     return NoChannel;
 }
 
@@ -1105,6 +1127,12 @@ EventDataType Day::count(ChannelID code)
 {
     EventDataType total = 0;
 
+    if (code == AllAhiChannels) {
+        for (int i = 0; i < ahiChannels.size(); i++)
+            total += count(ahiChannels.at(i));
+        return total;
+    }
+
     for (auto & sess : sessions) {
         if (sess->enabled() && sess->m_cnt.contains(code)) {
             total += sess->count(code);
@@ -1120,7 +1148,7 @@ bool Day::noSettings(Machine * mach)
             // If this day generally has just summary data.
             return true;
         } else if ((mach == sess->machine())  && sess->noSettings()) {
-            // Focus only on machine mach
+            // Focus only on device match
             return true;
         }
     }
@@ -1134,7 +1162,7 @@ bool Day::summaryOnly(Machine * mach)
             // If this day generally has just summary data.
             return true;
         } else if ((mach == sess->machine())  && sess->summaryOnly()) {
-            // Focus only on machine mach
+            // Focus only on device match
             return true;
         }
     }
@@ -1395,7 +1423,7 @@ void Day::removeMachine(Machine * mach)
     QList<Session*> list = sessions;  // make a copy so the iterator doesn't get broken by removals
     for (auto & sess : list) {
         if (sess->machine() == mach) {
-            // This indicates a problem with the Machine class not tracking all of its sessions, for
+            // This indicates a problem with the device class not tracking all of its sessions, for
             // example if there's a duplicate session ID.
             qCritical() << "Day object" << this->date().toString()
                         << "session" << sess->session() << "refers to machine" << mach->serial();
@@ -1431,7 +1459,7 @@ int Day::getCPAPMode()
 
 //    schema::Channel & chan = schema::channel[modechan];
 
-    // TODO: This is an awful hack that depends on the enum ordering of the machine-specific CPAP mode.
+    // TODO: This is an awful hack that depends on the enum ordering of the device-specific CPAP mode.
     // See the comment in getCPAPModeStr().
     int mode = (CPAPMode)(int)qRound(settings_wavg(modechan));
 
@@ -1449,7 +1477,7 @@ QString Day::getCPAPModeStr()
 
     schema::Channel & chan = schema::channel[modechan];
 
-    // TODO: This is an awful hack that depends on the enum ordering of the machine-specific CPAP mode.
+    // TODO: This is an awful hack that depends on the enum ordering of the device-specific CPAP mode.
     // Instead, we should calculate how long each mode was in operation and
     // determine the one that was running the longest, along with the settings
     // while that mode was in operation.
@@ -1467,7 +1495,9 @@ QString Day::getCPAPModeStr()
 //        return QObject::tr("Auto Bi-Level (Fixed PS)");
 //    } else if (mode == MODE_BILEVEL_AUTO_VARIABLE_PS) {
 //        return QObject::tr("Auto Bi-Level (Variable PS)");
-//    }  else if (mode == MODE_ASV) {
+//    } else if (mode == MODE_TRILEVEL_AUTO_VARIABLE_PDIFF) {
+//        return QObject::tr("Auto TriLevel (Variable PDIFF)");
+//    }   else if (mode == MODE_ASV) {
 //        return QObject::tr("ASV Fixed EPAP");
 //    } else if (mode == MODE_ASV_VARIABLE_EPAP) {
 //        return QObject::tr("ASV Variable EPAP");
@@ -1489,11 +1519,36 @@ QString Day::getPressureRelief()
     ChannelID pr_level_chan = loader->PresReliefLevel();
     ChannelID pr_mode_chan = loader->PresReliefMode();
 
+    // Separate calculation for SleepStyle devices
+    if (mach->info.loadername == "SleepStyle") {
+        pr_str = loader->PresReliefLabel();
+
+        int pr_level = -1;
+        if (pr_level_chan != NoChannel && settingExists(pr_level_chan)) {
+            pr_level = qRound(settings_wavg(pr_level_chan));
+        }
+        if (pr_level == -1)
+            return STR_TR_None;
+
+        if ((pr_mode_chan != NoChannel) && settingExists(pr_mode_chan)) {
+            schema::Channel & chan = schema::channel[pr_level_chan];
+            QString level = chan.option(pr_level);
+            if (level.isEmpty()) {
+                level = QString().number(pr_level) + " " + chan.units();
+            if (settings_min(pr_level_chan) != settings_max(pr_level_chan))
+                level = QObject::tr("varies");
+            }
+            pr_str += QString(" %1").arg(level);
+        }
+
+        return pr_str;
+    }
+
     if ((pr_mode_chan != NoChannel) && settingExists(pr_mode_chan)) {
         // TODO: This is an awful hack that depends on the enum ordering of the pressure relief mode.
         // See the comment in getCPAPModeStr().
         int pr_mode = qRound(settings_wavg(pr_mode_chan));
-        pr_str = QObject::tr("%1%2").arg(loader->PresReliefLabel()).arg(schema::channel[pr_mode_chan].option(pr_mode));
+        pr_str = QString("%1%2").arg(loader->PresReliefLabel()).arg(schema::channel[pr_mode_chan].option(pr_mode));
 
         int pr_level = -1;
         if (pr_level_chan != NoChannel && settingExists(pr_level_chan)) {
@@ -1532,18 +1587,31 @@ QString Day::getPressureSettings()
     QString units = schema::channel[CPAP_Pressure].units();
 
     if (mode == MODE_CPAP) {
-        return QObject::tr("Fixed %1 (%2)").arg(validPressure(settings_min(CPAP_Pressure))).arg(units);
+        return QObject::tr("Fixed %1 (%2)").arg(validPressure(settings_min(CPAP_Pressure))).
+                                            arg(units);
     } else if (mode == MODE_APAP) {
-        return QObject::tr("Min %1 Max %2 (%3)").arg(validPressure(settings_min(CPAP_PressureMin))).arg(validPressure(settings_max(CPAP_PressureMax))).arg(units);
+        return QObject::tr("Min %1 Max %2 (%3)").arg(validPressure(settings_min(CPAP_PressureMin))).
+                                                 arg(validPressure(settings_max(CPAP_PressureMax))).
+                                                 arg(units);
     } else if (mode == MODE_BILEVEL_FIXED ) {
-        return QObject::tr("EPAP %1 IPAP %2 (%3)").arg(validPressure(settings_min(CPAP_EPAP))).arg(validPressure(settings_max(CPAP_IPAP))).arg(units);
+        return QObject::tr("EPAP %1 IPAP %2 (%3)").arg(validPressure(settings_min(CPAP_EPAP))).
+                                                   arg(validPressure(settings_max(CPAP_IPAP))).
+                                                   arg(units);
     } else if (mode == MODE_BILEVEL_AUTO_FIXED_PS) {
-        return QObject::tr("PS %1 over %2-%3 (%4)").arg(validPressure(settings_max(CPAP_PS))).arg(validPressure(settings_min(CPAP_EPAPLo))).arg(validPressure(settings_max(CPAP_IPAPHi))).arg(units);
+        return QObject::tr("PS %1 over %2-%3 (%4)").arg(validPressure(settings_max(CPAP_PS))).
+                                                    arg(validPressure(settings_min(CPAP_EPAPLo))).
+                                                    arg(validPressure(settings_max(CPAP_IPAPHi))).
+                                                    arg(units);
     } else if (mode == MODE_BILEVEL_AUTO_VARIABLE_PS) {
         return QObject::tr("Min EPAP %1 Max IPAP %2 PS %3-%4 (%5)").arg(validPressure(settings_min(CPAP_EPAPLo))).
                 arg(validPressure(settings_max(CPAP_IPAPHi))).
                 arg(validPressure(settings_min(CPAP_PSMin))).
                 arg(validPressure(settings_max(CPAP_PSMax))).arg(units);
+    } else if (mode == MODE_TRILEVEL_AUTO_VARIABLE_PDIFF) {
+        return QObject::tr("Min EEPAP %1 Max EEPAP %2 PDIFF %3-%4 (%5)").arg(validPressure(settings_min(CPAP_EEPAPLo))).
+                arg(validPressure(settings_max(CPAP_EEPAPHi))).
+                arg(validPressure(settings_min(CPAP_PSMin))).
+                arg(validPressure(settings_max(CPAP_PSMax))).arg(units);
     } else if (mode == MODE_ASV) {
         return QObject::tr("EPAP %1 PS %2-%3 (%4)").arg(validPressure(settings_min(CPAP_EPAP))).
                 arg(validPressure(settings_min(CPAP_PSMin))).
@@ -1556,11 +1624,25 @@ QString Day::getPressureSettings()
                 arg(validPressure(settings_min(CPAP_PSMax))).
                 arg(units);
     } else if (mode == MODE_AVAPS) {
-        return QObject::tr("EPAP %1 IPAP %2-%3 (%4)").
+//      qDebug() << "AVAPS: EPAP" << settings_min(CPAP_EPAP) << "IPAP min" << settings_max(CPAP_IPAPLo) <<
+//                  "IPAP max" << settings_max(CPAP_IPAPHi);
+        QString retStr;
+        if (settings_min(CPAP_EPAPLo) == settings_max(CPAP_EPAPHi))
+            retStr = QObject::tr("EPAP %1 IPAP %2-%3 (%4)").
                 arg(validPressure(settings_min(CPAP_EPAP))).
                 arg(validPressure(settings_max(CPAP_IPAPLo))).
                 arg(validPressure(settings_max(CPAP_IPAPHi))).
                 arg(units);
+        else 
+            retStr = QObject::tr("EPAP %1-%2 IPAP %3-%4 (%5)").
+                arg(validPressure(settings_min(CPAP_EPAPLo))).
+                arg(validPressure(settings_min(CPAP_EPAPHi))).
+                arg(validPressure(settings_max(CPAP_IPAPLo))).
+                arg(validPressure(settings_max(CPAP_IPAPHi))).
+                arg(units);
+
+//      qDebug() << "AVAPS mode:" << retStr;
+        return retStr;
     }
 
     return STR_TR_Unknown;
