@@ -1,7 +1,7 @@
 /* SleepLib Day Class Header
  *
- * Copyright (c) 2019 The OSCAR Team
- * Copyright (C) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (C) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
@@ -17,7 +17,7 @@
 #include "SleepLib/session.h"
 
 /*! \class OneTypePerDay
-    \brief An Exception class to catch multiple machine records per day
+    \brief An Exception class to catch multiple device records per day
     */
 class OneTypePerDay
 {
@@ -27,7 +27,7 @@ class Machine;
 class Session;
 
 /*! \class Day
-    \brief Contains a list of all Sessions for single date, for a single machine
+    \brief Contains a list of all Sessions for single date, for a single device
     */
 class Day
 {
@@ -35,13 +35,13 @@ class Day
     Day();
     ~Day();
 
-    //! \brief Add a new machine to this day record
+    //! \brief Add a new device to this day record
     bool addMachine(Machine *m);
 
-    //! \brief Returns a machine record if present of specified machine type
+    //! \brief Returns a device record if present of specified device type
     Machine *machine(MachineType type);
 
-    //! \brief Returns a list of sessions for the specified machine type
+    //! \brief Returns a list of sessions for the specified device type
     QList<Session *> getSessions(MachineType type, bool ignore_enabled = false);
 
     //! \brief Add Session to this Day object (called during Load)
@@ -92,16 +92,16 @@ class Day
     //! \brief Returns if the cache contains SummaryType information about the requested code
     bool hasData(ChannelID code, SummaryType type);
 
-    //! \brief Returns true if Day has specific machine type
+    //! \brief Returns true if Day has specific device type
     inline bool hasMachine(MachineType mt) const { return machines.contains(mt); }
 
-    //! \brief Returns true if Day has specific machine record
+    //! \brief Returns true if Day has specific device record
     bool hasMachine(Machine * mach);
 
-    //! \brief Returns true if any sessions have records matching specific machine type
+    //! \brief Returns true if any sessions have records matching specific device type
     bool searchMachine(MachineType mt);
 
-    //! \brief Removes any lingering references to a specific machine record and emits a warning if there were any
+    //! \brief Removes any lingering references to a specific device record and emits a warning if there were any
     void removeMachine(Machine * mach);
 
     //! \brief Returns the Average of all Sessions setting 'code' for this day
@@ -138,10 +138,10 @@ class Day
     //! \brief Returns the last session time of this day
     qint64 last();
 
-    //! \brief Returns the first session time of this machine type for this day
+    //! \brief Returns the first session time of this device type for this day
     qint64 first(MachineType type);
 
-    //! \brief Returns the last session time of this machine type for this day
+    //! \brief Returns the last session time of this device type for this day
     qint64 last(MachineType type);
 
 
@@ -160,10 +160,10 @@ class Day
     //! \brief Returns the total time in milliseconds for this day
     qint64 total_time();
 
-    //! \brief Returns the total time in milliseconds for this day for given machine type
+    //! \brief Returns the total time in milliseconds for this day for given device type
     qint64 total_time(MachineType type);
 
-    //! \brief Returns true if this day has enabled sessions for supplied machine type
+    //! \brief Returns true if this day has enabled sessions for supplied device type
     bool hasEnabledSessions(MachineType);
 
     //! \brief Returns true if this day has enabled sessions
@@ -202,6 +202,8 @@ class Day
 
     Session *find(SessionID sessid);
 
+    Session *find(SessionID sessid, MachineType mt);
+
     //! \brief Returns the number of Sessions in this day record
     int size() { return sessions.size(); }
 
@@ -234,7 +236,7 @@ class Day
     //! \brief Returns a list of channels of supplied types, according to channel orders
     QList<ChannelID> getSortedMachineChannels(quint32 chantype);
 
-    //! \brief Returns a list of machine specific channels of supplied types, according to channel orders
+    //! \brief Returns a list of device specific channels of supplied types, according to channel orders
     QList<ChannelID> getSortedMachineChannels(MachineType type, quint32 chantype);
 
     // Some ugly CPAP specific stuff
@@ -248,14 +250,14 @@ class Day
 
     //! \brief Calculate AHI (Apnea Hypopnea Index)
     EventDataType calcAHI() {
-        EventDataType c = count(CPAP_Hypopnea) + count(CPAP_Obstructive) + count(CPAP_Apnea) + count(CPAP_ClearAirway);
+        EventDataType c = count(AllAhiChannels);
         EventDataType minutes = hours(MT_CPAP) * 60.0;
         return (c * 60.0) / minutes;
     }
 
     //! \brief Calculate RDI (Respiratory Disturbance Index)
     EventDataType calcRDI() {
-        EventDataType c = count(CPAP_Hypopnea) + count(CPAP_Obstructive) + count(CPAP_Apnea) + count(CPAP_ClearAirway) + count(CPAP_RERA);
+        EventDataType c = count(AllAhiChannels) + count(CPAP_RERA);
         EventDataType minutes = hours(MT_CPAP) * 60.0;
         return (c * 60.0) / minutes;
     }
@@ -278,13 +280,13 @@ class Day
 
     //! \brief SleepyyHead Events Index, AHI combined with OSCAR detected events.. :)
     EventDataType calcSHEI() {
-        EventDataType c = count(CPAP_Hypopnea) + count(CPAP_Obstructive) + count(CPAP_Apnea) + count(CPAP_ClearAirway) + count(CPAP_UserFlag1) + count(CPAP_UserFlag2);
+        EventDataType c = count(AllAhiChannels) + count(CPAP_UserFlag1) + count(CPAP_UserFlag2);
         EventDataType minutes = hours(MT_CPAP) * 60.0;
         return (c * 60.0) / minutes;
     }
     //! \brief Total duration of all Apnea/Hypopnea events in seconds,
     EventDataType calcTTIA() {
-        EventDataType c = sum(CPAP_Hypopnea) + sum(CPAP_Obstructive) + sum(CPAP_Apnea) + sum(CPAP_ClearAirway);
+        EventDataType c = sum(AllAhiChannels);
         return c;
     }
     bool hasEvents();
