@@ -1,6 +1,7 @@
 ﻿/* SleepLib Fisher & Paykel Icon Loader Implementation
  *
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
@@ -14,6 +15,16 @@
 
 #include "icon_loader.h"
 
+// The qt5.15 obsolescence of hex requires this change.
+// this solution to QT's obsolescence is only used in debug statements
+#if QT_VERSION >= QT_VERSION_CHECK(5,15,0)
+    #define QTHEX     Qt::hex
+    #define QTDEC     Qt::dec
+#else
+    #define QTHEX     hex
+    #define QTDEC     dec
+#endif
+
 const QString FPHCARE = "FPHCARE";
 
 FPIcon::FPIcon(Profile *profile, MachineID id)
@@ -35,8 +46,114 @@ FPIconLoader::~FPIconLoader()
 {
 }
 
+/*
+ * getIconDir - returns the path to the ICON directory
+ */
+QString getIconDir2 (QString givenpath) {
+
+    QString path = givenpath;
+
+    path = path.replace("\\", "/");
+
+    if (path.endsWith("/")) {
+        path.chop(1);
+    }
+
+    if (path.endsWith("/" + FPHCARE)) {
+        path = path.section("/",0,-2);
+    }
+
+    QDir dir(path);
+
+    if (!dir.exists()) {
+        return "";
+    }
+
+    // If this is a backup directory, higher level directories have been
+    // omitted.
+    if (path.endsWith("/Backup/", Qt::CaseInsensitive))
+        return path;
+
+    // F&P Icon have a folder called FPHCARE in the root directory
+    if (!dir.exists(FPHCARE)) {
+        return "";
+    }
+
+    // CHECKME: I can't access F&P ICON data right now
+    if (!dir.exists("FPHCARE/ICON")) {
+        return "";
+    }
+
+    return dir.filePath("FPHCARE/ICON");
+}
+
+/*
+ * getIconMachines returns a list of all Iocn device folders in the ICON directory
+ */
+QStringList getIconMachines (QString iconPath) {
+    QStringList iconMachines;
+
+    QDir iconDir (iconPath);
+
+    iconDir.setFilter(QDir::NoDotAndDotDot | QDir::Dirs | QDir::Files | QDir::Hidden | QDir::NoSymLinks);
+    iconDir.setSorting(QDir::Name);
+
+    QFileInfoList flist = iconDir.entryInfoList();   // List of Icon subdirectories
+
+    // Walk though directory list and save those that appear to be for SleepStyle machins.
+    for (int i = 0; i < flist.size(); i++) {
+        QFileInfo fi = flist.at(i);
+        QString filename = fi.fileName();
+
+        // directory is serial number and must have a SUM*.FPH file within it to be an Icon or SleepStyle folder
+
+        QDir machineDir (iconPath + "/" + filename);
+        machineDir.setFilter(QDir::NoDotAndDotDot | QDir::Files | QDir::Hidden | QDir::NoSymLinks);
+        machineDir.setSorting(QDir::Name);
+        QStringList filters;
+        filters << "SUM*.fph";
+        machineDir.setNameFilters(filters);
+        QFileInfoList flist = machineDir.entryInfoList();
+        if (flist.size() <= 0) {
+            continue;
+        }
+
+        // Find out what device model this is
+        QFile sumFile (flist.at(0).absoluteFilePath());
+
+        QString line;
+
+        sumFile.open(QIODevice::ReadOnly);
+        QTextStream instr(&sumFile);
+        for (int j = 0; j < 5; j++) {
+            line = "";
+            QString c = "";
+            while ((c = instr.read(1)) != "\r") {
+                line += c;
+            }
+        }
+        sumFile.close();
+        if (line.toUpper() == "ICON")
+            iconMachines.push_back(filename);
+
+    }
+
+    return iconMachines;
+}
+
 bool FPIconLoader::Detect(const QString & givenpath)
 {
+    QString iconPath = getIconDir2(givenpath);
+    if (iconPath.isEmpty())
+        return false;
+
+    QStringList machines = getIconMachines(iconPath);
+    if (machines.length() <= 0)
+        // Did not find any SleepStyle device directories
+        return false;
+
+    return true;
+/****
     QString path = givenpath;
     
     path = path.replace("\\", "/");
@@ -66,42 +183,13 @@ bool FPIconLoader::Detect(const QString & givenpath)
         return false;
     }
 
-    return true;
-}
+    // ICON serial numbers (directory names) are all digits (SleepStyle are mixed alpha and numeric)
+    QString serialDir(dir.path() + "/FPHCARE/ICON");
+    QDir iconDir(serialDir);
 
-
-int FPIconLoader::Open(const QString & path)
-{
-    QString tmp = path;
-
-    tmp = tmp.replace("\\", "/");
-    if (tmp.endsWith("/")) {
-        tmp.chop(1);
-    }
-
-    QString newpath;
-
-    if (tmp.endsWith("/" + FPHCARE)) {
-        newpath = tmp;
-    } else {
-        newpath = tmp + "/" + FPHCARE;
-    }
-
-    newpath += "/ICON/";
-
-    QString filename;
-
-    QDir dir(newpath);
-
-    if ((!dir.exists() || !dir.isReadable())) {
-        return -1;
-    }
-
-    dir.setFilter(QDir::NoDotAndDotDot | QDir::Dirs | QDir::Files | QDir::Hidden | QDir::NoSymLinks);
-    dir.setSorting(QDir::Name);
-    QFileInfoList flist = dir.entryInfoList();
-
-    QStringList SerialNumbers;
+    iconDir.setFilter(QDir::NoDotAndDotDot | QDir::Dirs | QDir::Files | QDir::Hidden | QDir::NoSymLinks);
+    iconDir.setSorting(QDir::Name);
+    QFileInfoList flist = iconDir.entryInfoList();
 
     bool ok;
 
@@ -111,22 +199,37 @@ int FPIconLoader::Open(const QString & path)
 
         filename.toInt(&ok);
 
-        if (ok) {
-            SerialNumbers.push_back(filename);
+        if (!ok) {
+            return false;
         }
     }
+    return true;
+****/
+}
+
+
+int FPIconLoader::Open(const QString & path)
+{
+    QString iconPath = getIconDir2(path);
+    if (iconPath.isEmpty())
+        return false;
+
+    QStringList serialNumbers = getIconMachines(iconPath);
+    if (serialNumbers.length() <= 0)
+        // Did not find any SleepStyle device directories
+        return false;
 
     Machine *m;
 
     QString npath;
 
     int c = 0;
-    for (int i = 0; i < SerialNumbers.size(); i++) {
+    for (int i = 0; i < serialNumbers.size(); i++) {
         MachineInfo info = newInfo();
-        info.serial = SerialNumbers[i];
+        info.serial = serialNumbers[i];
         m = p_profile->CreateMachine(info);
 
-        npath = newpath + "/" + info.serial;
+        npath = iconPath + "/" + info.serial;
 
         try {
             if (m) {
@@ -137,7 +240,7 @@ int FPIconLoader::Open(const QString & path)
             p_profile->DelMachine(m);
             MachList.erase(MachList.find(info.serial));
             QMessageBox::warning(nullptr, tr("Import Error"),
-                                 tr("This Machine Record cannot be imported in this profile.")+"\n\n"+tr("The Day records overlap with already existing content."),
+                                 tr("This device Record cannot be imported in this profile.")+"\n\n"+tr("The Day records overlap with already existing content."),
                                  QMessageBox::Ok);
             delete m;
         }
@@ -355,7 +458,9 @@ quint32 convertFLWDate(quint32 timestamp) // Bit format: hhhhhmmmmmmssssssYYYYYY
     QDateTime dt = QDateTime(QDate(year, month, day), QTime(hour, minute, second), Qt::UTC);
 
     if(!dt.isValid()){
-        dt = QDateTime(QDate(2015,1,1), QTime(0,0,1));
+        // make this date too early, then change test later
+        // dt = QDateTime(QDate(2015,1,1), QTime(0,0,1));
+        dt = QDateTime(QDate(2010,1,1), QTime(0,0,0));
     }
 //    Q NO!!! _ASSERT(dt.isValid());
 //    if ((year == 2013) && (month == 9) && (day == 18)) {
@@ -413,12 +518,12 @@ hour=(ts >> 12) & 0x1f; */
 
 // FLW Header Structure
 // 0x0000-0x01fe
-// newline (0x0d) seperated list of machine information strings.
+// newline (0x0d) seperated list of device information strings.
 // magic?         0201
 // version        1.5.0
 // serial number  12 digits
-// Machine Series "ICON"
-// Machine Model  "Auto"
+// Device Series "ICON"
+// Device Model  "Auto"
 // Remainder of header is 0 filled...
 // 0x01ff 8 bit additive sum checksum byte of previous header bytes
 
@@ -483,7 +588,7 @@ bool FPIconLoader::OpenFLW(Machine *mach, const QString & filename)
 
     ts = convertFLWDate(t2);
 
-    if (ts > QDateTime(QDate(2015,1,1), QTime(0,0,0)).toTime_t()) {
+    if (ts < QDateTime(QDate(2010,1,1), QTime(0,1,0)).toTime_t()) {
         return false;
     }
 
@@ -586,7 +691,7 @@ bool FPIconLoader::OpenFLW(Machine *mach, const QString & filename)
     } while (p < end);
 
     if (endMarker != 0x7fff) {
-        qDebug() << fname << "waveform does not end with the corrent marker" << hex << endMarker;
+        qDebug() << fname << "waveform does not end with the corrent marker" << QTHEX << endMarker;
     }
 
     if (sess) {
@@ -827,6 +932,7 @@ bool FPIconLoader::OpenDetail(Machine *mach, const QString & filename)
     quint8 recs;
 
     int totalrecs = 0;
+    Q_UNUSED( totalrecs );
 
     do {
         in >> ts;
