@@ -1,6 +1,7 @@
 ﻿/* SleepLib Somnopose Loader Header
  *
- * Copyright (C) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (C) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
@@ -26,8 +27,9 @@ class SomnoposeLoader : public MachineLoader
 
     virtual bool Detect(const QString & path) { Q_UNUSED(path); return false; }  // bypass autoscanner
 
-    virtual int Open(const QString & path);
+    virtual int Open(const QString & path) { Q_UNUSED(path); return 0; } // Only for CPAP
     virtual int OpenFile(const QString & filename);
+    virtual QStringList getNameFilter() { return QStringList("Somnopose CSV File (*.csv)"); }
     static void Register();
 
     virtual int Version() { return somnopose_data_version; }
