@@ -1,12 +1,15 @@
-/* SleepLib Machine Class Implementation
+/* SleepLib Device Class Implementation
  *
- * Copyright (c) 2019 The OSCAR Team
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
  * for more details. */
 
+#define TEST_MACROS_ENABLEDoff
+#include <test_macros.h>
+
 #include <QApplication>
 #include <QDir>
 #include <QDebug>
@@ -33,6 +36,7 @@
 #include "profiles.h"
 #include <algorithm>
 #include "SleepLib/schema.h"
+//#include "SleepLib/session.h"
 #include "SleepLib/day.h"
 #include "mainwindow.h"
 
@@ -81,15 +85,14 @@ void LoadTask::run()
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////
-// Machine Base-Class implmementation
+// Device Base-Class implmementation
 //////////////////////////////////////////////////////////////////////////////////////////
 Machine::Machine(Profile *_profile, MachineID id) : profile(_profile)
 {
     day.clear();
     highest_sessionid = 0;
-    m_unsupported = false;
     m_suppressUntestedWarning = false;
-    // TODO: Have the machine write m_suppressUntestedWarning and m_previousUnexpected
+    // TODO: Have the device write m_suppressUntestedWarning and m_previousUnexpected
     // to XML (along with the current OSCAR version number) so that they persist across
     // application launches (but reset with each new OSCAR version).
 
@@ -98,7 +101,7 @@ Machine::Machine(Profile *_profile, MachineID id) : profile(_profile)
         MachineID temp;
 
         bool found;
-        // Keep trying until we get a unique machineID for this profile
+        // Keep trying until we get a unique DeviceID for this profile
         do {
             temp = rand();
 
@@ -113,14 +116,14 @@ Machine::Machine(Profile *_profile, MachineID id) : profile(_profile)
     } else { m_id = id; }
     m_loader = nullptr;
 
-   // qDebug() << "Create Machine: " << hex << m_id; //%lx",m_id);
+   // qDebug() << "Create device: " << hex << m_id; //%lx",m_id);
     m_type = MT_UNKNOWN;
     firstsession = true;
 }
 Machine::~Machine()
 {
     saveSessionInfo();
-    //qDebug() << "Destroy Machine" << info.loadername << hex << m_id;
+    //qDebug() << "Destroy device" << info.loadername << hex << m_id;
 }
 Session *Machine::SessionExists(SessionID session)
 {
@@ -162,7 +165,7 @@ bool Machine::saveSessionInfo()
         Session * sess = s.value();
         if (sess->s_first != 0) {
             out << (quint32) sess->session();
-            out << (bool)(sess->enabled());
+            out << (quint8)(sess->enabled(true));
         } else {
             qWarning() << "Machine::SaveSessionInfo discarding session" << sess->s_session
                        << "["+QDateTime::fromTime_t(sess->s_session).toString("MMM dd, yyyy hh:mm:ss")+"]"
@@ -195,9 +198,11 @@ bool Machine::loadSessionInfo()
             Session * sess = s.value();
             QHash<ChannelID, QVariant>::iterator it = sess->settings.find(SESSION_ENABLED);
 
-            bool b = true;
+            quint8 b = true;
+            b &= 0x1;
             if (it != sess->settings.end()) {
                 b = it.value().toBool();
+            } else {
             }
             sess->setEnabled(b);        // Extract from session settings and save..
         }
@@ -231,11 +236,12 @@ bool Machine::loadSessionInfo()
     in >> size;
 
     quint32 sid;
-    bool b;
+    quint8 b;
 
     for (int i=0; i< size; ++i) {
         in >> sid;
         in >> b;
+        b &= 0x1;
 
         s = sessionlist.find(sid);
 
@@ -279,7 +285,11 @@ QDate Machine::pickDate(qint64 first)
     return date;
 }
 
-bool Machine::AddSession(Session *s)
+// allowOldSessions defaults to false and is only set to true when loading
+// summary data on profile load. This true setting prevents old sessions from
+// becoming lost if user preference indicates to not import sessions prior to a
+// given date.
+bool Machine::AddSession(Session *s, bool allowOldSessions)
 {
     if (s == nullptr) {
         qCritical() << "AddSession() called with a null object";
@@ -303,7 +313,10 @@ bool Machine::AddSession(Session *s)
         return false;
     }
 
-    if (profile->session->ignoreOlderSessions()) {
+    // allowOldSessions is true when loading summaries (already imported sessions)
+    // We don't want to throw away data already in the database in circumstances
+    // where user wants to ignore old sessions on import.
+    if (profile->session->ignoreOlderSessions() && !allowOldSessions) {
         qint64 ignorebefore = profile->session->ignoreOlderSessionsDate().toMSecsSinceEpoch();
         if (s->last() < ignorebefore) {
             qDebug() << s->session() << "Ignoring old session";
@@ -387,8 +400,7 @@ bool Machine::AddSession(Session *s)
 
     if (session_length < ignore_sessions) {
         // keep the session to save importing it again, but don't add it to the day record this time
-        qDebug() << s->session() << "Ignoring short session <" << ignore_sessions
-            << "["+QDateTime::fromMSecsSinceEpoch(s->first()).toString("MMM dd, yyyy hh:mm:ss")+"]";
+        // qDebug() << s->session() << "Ignoring short session <" << ignore_sessions << "["+QDateTime::fromMSecsSinceEpoch(s->first()).toString("MMM dd, yyyy hh:mm:ss")+"]";
         return true;
     }
 
@@ -409,7 +421,6 @@ bool Machine::AddSession(Session *s)
         dit = day.insert(date, profile->addDay(date));
     }
     dd = dit.value();
-
     dd->addSession(s);
 
     if (combine_next_day) {
@@ -466,7 +477,7 @@ bool Machine::unlinkSession(Session * sess)
 {
     MachineType mt = sess->type();
 
-    // Remove the object from the machine object's session list
+    // Remove the object from the device object's session list
     bool b=sessionlist.remove(sess->session());
 
     QList<QDate> dates;
@@ -503,7 +514,7 @@ bool Machine::unlinkSession(Session * sess)
     return b;
 }
 
-// This functions purpose is murder and mayhem... It deletes all of a machines data.
+// This functions purpose is murder and mayhem... It deletes all of a devices data.
 bool Machine::Purge(int secret)
 {
     // Boring api key to stop this function getting called by accident :)
@@ -553,7 +564,7 @@ bool Machine::Purge(int secret)
 
         delete sess;
     }
-    // Make sure there aren't any dangling references to this machine
+    // Make sure there aren't any dangling references to this device
     for (auto & d : days) {
         d->removeMachine(this);
     }
@@ -617,8 +628,11 @@ void Machine::setInfo(MachineInfo inf)
     m_loader = GetLoader(inf.loadername);
 }
 
+QString toHexid(quint32 id) { return QString("%1").arg(id,8,16,QLatin1Char('0')); };
+QString Machine::hexid() { return toHexid((qint32)m_id);};
+
 const QString Machine::getDataPath()
-{    
+{
     // TODO: Rework the underlying database so that file storage doesn't rely on consistent presence or absence of the serial number.
     m_dataPath = p_pref->Get("{home}/Profiles/")+profile->user->userName()+"/"+info.loadername + "_"
                  + (info.serial.isEmpty() ? hexid() : info.serial) + "/";
@@ -700,7 +714,7 @@ bool Machine::Load(ProgressDialog *progress)
         progress->setProgressValue(0);
         QApplication::processEvents();
 
-        QTime time;
+        QElapsedTimer time;
         time.start();
         dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
 
@@ -719,7 +733,7 @@ bool Machine::Load(ProgressDialog *progress)
         int size = filelist.size();
 
 
-        // Legacy crap.. Summary and Event stuff used to be in one big pile in the machine folder root
+        // Legacy crap.. Summary and Event stuff used to be in one big pile in the device folder root
         for (auto & filename : filelist) {
             QFile::rename(path+filename, summarypath+filename);
         }
@@ -783,7 +797,7 @@ bool Machine::Load(ProgressDialog *progress)
 
             // Forced to load it, because know nothing about this session..
             if (sess->LoadSummary()) {
-                AddSession(sess);
+                AddSession(sess, true);
             } else {
                 qWarning() << "Error loading summary file" << filename;
                 delete sess;
@@ -893,7 +907,7 @@ const int summaryxml_version=1;
 
 bool Machine::LoadSummary(ProgressDialog * progress)
 {
-    QTime time;
+    QElapsedTimer time;
     time.start();
     QString filename = getDataPath() + summaryFileName + ".gz";
 
@@ -1016,10 +1030,10 @@ bool Machine::LoadSummary(ProgressDialog * progress)
 //      if ((cnt % 100) == 0) {
 //          progress->setValue(cnt);
 //          //QApplication::processEvents();
-//      } 
+//      }
 *****************************************************************/
         Session * sess = it.value();
-        if ( ! AddSession(sess)) {
+        if ( ! AddSession(sess, true)) {
             delete sess;
         } else {
             if (loadSummaries) {
@@ -1077,7 +1091,7 @@ bool Machine::SaveSummaryCache()
         el.setAttribute("id", (quint32)sess->session());
         el.setAttribute("first", sess->realFirst());
         el.setAttribute("last", sess->realLast());
-        el.setAttribute("enabled", sess->enabled() ? "1" : "0");
+        el.setAttribute("enabled", sess->enabled(true) ? "1" : "0");
         el.setAttribute("events", sess->summaryOnly() ? "0" : "1");
 
         QHash<ChannelID, QVector<EventList *> >::iterator ev;
