@@ -1,6 +1,7 @@
-/* SleepLib Common Machine Header
+/* SleepLib Common Device Header
  *
- * Copyright (C) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (C) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
@@ -22,10 +23,11 @@ using namespace std;
 
 // Do not change these without considering the consequences.. For one the Loader needs changing & version increase
 typedef quint32 ChannelID;
-typedef long MachineID;
-typedef long SessionID;
+typedef quint32 MachineID;
+typedef quint32 SessionID;
 typedef float EventDataType;
 typedef qint16 EventStoreType;
+extern QString toHexid(quint32);
 
 //! \brief Exception class for out of Bounds error.. Unused.
 class BoundsError {};
@@ -48,22 +50,22 @@ qint64 timezoneOffset();
 enum SummaryType { ST_CNT, ST_SUM, ST_AVG, ST_WAVG, ST_PERC, ST_90P, ST_MIN, ST_MAX, ST_MID, ST_CPH, ST_SPH, ST_FIRST, ST_LAST, ST_HOURS, ST_SESSIONS, ST_SETMIN, ST_SETAVG, ST_SETMAX, ST_SETWAVG, ST_SETSUM, ST_SESSIONID, ST_DATE };
 
 /*! \enum MachineType
-    \brief Generalized type of a machine. MT_CPAP is any type of xPAP machine, MT_OXIMETER any type of Oximeter
+    \brief Generalized type of a device. MT_CPAP is any type of xPAP device, MT_OXIMETER any type of Oximeter
     \brief MT_SLEEPSTAGE stage of sleep detector (ZEO importer), MT_JOURNAL for optional notes, MT_POSITION for sleep position detector (Somnopose)
   */
-// TODO: This really needs to be a bitmask, since there are increasing numbers of machines that provide
+// TODO: This really needs to be a bitmask, since there are increasing numbers of devices that provide
 // multiple kinds of data, such as oximetry + motion/position, or sleep stage + oximetry, etc.
 //
-// Machine/loader classes will use the bitmask to identify which data they are capable of importing.
-// It may be that we ultimately prefer to have each machine identify a primary type instead or in addition.
+// Device/loader classes will use the bitmask to identify which data they are capable of importing.
+// It may be that we ultimately prefer to have each device identify a primary type instead or in addition.
 //
 // The channel schema's use of these is probably fine.
 //
-// Days/Sessions/etc. that currently search for data based on the machines they contain will instead
+// Days/Sessions/etc. that currently search for data based on the devices they contain will instead
 // need to search for channels with data of that MT type. And anywhere else the code makes decisions
 // based on MT.
 //
-// Unfortunately, this also includes previously imported data, as Session encodes the machine's type in
+// Unfortunately, this also includes previously imported data, as Session encodes the device's type in
 // each file on disk. We might be partially saved by the fact that MT_CPAP and MT_OXIMETER were originally
 // 1 and 2, which would only break MT_SLEEPSTAGE and higher.
 enum MachineType { MT_UNKNOWN = 0, MT_CPAP, MT_OXIMETER, MT_SLEEPSTAGE, MT_JOURNAL, MT_POSITION, MT_UNCATEGORIZED = 99};
@@ -94,11 +96,11 @@ const quint32 PAP_SplitNight              = 0x8000;  // Split night capabilities
     \brief CPAP Machines mode of operation
   */
 enum CPAPMode { //:short
-    MODE_UNKNOWN = 0, MODE_CPAP, MODE_APAP, MODE_BILEVEL_FIXED, MODE_BILEVEL_AUTO_FIXED_PS, MODE_BILEVEL_AUTO_VARIABLE_PS, MODE_ASV, MODE_ASV_VARIABLE_EPAP, MODE_AVAPS
+    MODE_UNKNOWN = 0, MODE_CPAP, MODE_APAP, MODE_BILEVEL_FIXED, MODE_BILEVEL_AUTO_FIXED_PS, MODE_BILEVEL_AUTO_VARIABLE_PS, MODE_ASV, MODE_ASV_VARIABLE_EPAP, MODE_AVAPS, MODE_TRILEVEL_AUTO_VARIABLE_PDIFF
 };
 
 /*! \enum PRTypes
-    \brief Pressure Relief Types, used by CPAP machines
+    \brief Pressure Relief Types, used by CPAP devices
   */
 enum PRTypes { //:short
     PR_UNKNOWN = 0, PR_NONE, PR_CFLEX, PR_CFLEXPLUS, PR_AFLEX, PR_BIFLEX, PR_EPR, PR_SMARTFLEX, PR_EASYBREATHE, PR_SENSAWAKE
@@ -111,6 +113,7 @@ enum PRTimeModes { //:short
 struct MachineInfo {
     MachineInfo() { type = MT_UNKNOWN; version = 0; cap=0; }
     MachineInfo(const MachineInfo & /*copy*/) = default;
+    ~MachineInfo() {};
 
     MachineInfo(MachineType type, quint32 cap, QString loadername, QString brand, QString model, QString modelnumber, QString serial, QString series, QDateTime lastimported, int version, QDate purgeDate = QDate()) :
         type(type), cap(cap), loadername(loadername), brand(brand), model(model), modelnumber(modelnumber), serial(serial), series(series), lastimported(lastimported), version(version), purgeDate(purgeDate) {}
@@ -126,6 +129,9 @@ struct MachineInfo {
     QDateTime lastimported;
     int version;
     QDate purgeDate;
+
+    //! \brief List of text device properties, like brand, model, etc...
+    QHash<QString, QString> properties;
 };
 
 
@@ -141,13 +147,15 @@ struct MachineInfo {
 enum MCDataType
 { MC_bool = 0, MC_int, MC_long, MC_float, MC_double, MC_string, MC_datetime };
 
+extern ChannelID AllAhiChannels;
+extern QVector<ChannelID> ahiChannels;
 
 extern ChannelID NoChannel, SESSION_ENABLED, CPAP_SummaryOnly;
-extern ChannelID CPAP_IPAP, CPAP_IPAPLo, CPAP_IPAPHi, CPAP_EPAP, CPAP_EPAPLo, CPAP_EPAPHi,
+extern ChannelID CPAP_IPAP, CPAP_IPAPLo, CPAP_IPAPHi, CPAP_EPAP, CPAP_EPAPLo, CPAP_EPAPHi, CPAP_EEPAP, CPAP_EEPAPLo, CPAP_EEPAPHi,
        CPAP_Pressure, CPAP_PS, CPAP_PSMin, CPAP_PSMax,
        CPAP_Mode, CPAP_AHI,
        CPAP_PressureMin, CPAP_PressureMax, CPAP_Ramp, CPAP_RampTime, CPAP_RampPressure, CPAP_Obstructive,
-       CPAP_Hypopnea,
+       CPAP_Hypopnea, CPAP_AllApnea,
        CPAP_ClearAirway, CPAP_Apnea, CPAP_PB, CPAP_CSR, CPAP_LeakFlag, CPAP_ExP, CPAP_NRI, CPAP_VSnore,
        CPAP_VSnore2,
        CPAP_RERA, CPAP_PressurePulse, CPAP_FlowLimit, CPAP_SensAwake, CPAP_FlowRate, CPAP_MaskPressure,
@@ -160,10 +168,13 @@ extern ChannelID CPAP_IPAP, CPAP_IPAPLo, CPAP_IPAPHi, CPAP_EPAP, CPAP_EPAPLo, CP
 
 extern ChannelID RMS9_E01, RMS9_E02, RMS9_SetPressure, RMS9_MaskOnTime;
 extern ChannelID CPAP_LargeLeak, PRS1_BND,
-       PRS1_FlexMode, PRS1_FlexLevel, PRS1_HumidStatus, PRS1_HumidLevel, CPAP_HumidSetting, PRS1_MaskResistLock,
+       PRS1_FlexMode, PRS1_FlexLevel, PRS1_HumidStatus, PRS1_HumidLevel, PRS1_HumidTargetTime, PRS1_MaskResistLock,
+       CPAP_HumidSetting,
        PRS1_MaskResistSet, PRS1_HoseDiam, PRS1_AutoOn, PRS1_AutoOff, PRS1_MaskAlert, PRS1_ShowAHI;
 
-extern ChannelID INTELLIPAP_Unknown1, INTELLIPAP_Unknown2;
+extern ChannelID INTELLIPAP_Unknown1, INTELLIPAP_Unknown2, INTP_SnoreFlag;
+
+//extern ChannelID SS_SenseAwakeLevel, SS_EPR, SS_EPRLevel, SS_Ramp;
 
 extern ChannelID OXI_Pulse, OXI_SPO2, OXI_Perf, OXI_PulseChange, OXI_SPO2Drop, OXI_Plethy;
 
