@@ -1,12 +1,15 @@
 /* SleepLib Profiles Implementation
  *
- * Copyright (c) 2019-2020 The OSCAR Team
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
  * for more details. */
 
+#define TEST_MACROS_ENABLEDoff
+#include <test_macros.h>
+
 #include <QString>
 #include <QDateTime>
 #include <QDir>
@@ -94,7 +97,7 @@ Profile::~Profile()
         removeLock();
     }
 
-    // delete machine objects...
+    // delete device objects...
     for (auto & mach : m_machlist) {
         delete mach;
     }
@@ -156,7 +159,7 @@ void Profile::addLock()
     QFile lockfile(p_path+"lockfile");
     lockfile.open(QFile::WriteOnly);
     QByteArray ba;
-    ba.append(QHostInfo::localHostName());
+    ba.append(QHostInfo::localHostName().toUtf8());
     lockfile.write(ba);
     lockfile.close();
 }
@@ -253,11 +256,11 @@ bool Profile::OpenMachines()
 
         Machine *m = nullptr;
 
-        // Create Machine needs a profile passed to it..
+        // Create device needs a profile passed to it..
 
         m = CreateMachine(info, m_id);
 
-        if (m) m->properties = prop;
+        if (m) m->info.properties = prop;
 
         elem = elem.nextSiblingElement();
     }
@@ -282,7 +285,7 @@ bool Profile::StoreMachines()
         QDomElement pe = doc.createElement("properties");
         me.appendChild(pe);
 
-        for (QHash<QString, QString>::iterator j = m->properties.begin(); j != m->properties.end(); j++) {
+        for (QHash<QString, QString>::iterator j = m->info.properties.begin(); j != m->info.properties.end(); j++) {
             QDomElement pp = doc.createElement(j.key());
             pp.appendChild(doc.createTextNode(j.value()));
             pe.appendChild(pp);
@@ -546,11 +549,11 @@ void Profile::DataFormatError(Machine *m)
 
     if (backups) {
         msg = msg + QObject::tr("<b>OSCAR maintains a backup of your devices data card that it uses for this purpose.</b>")+ "<br/><br/>";
-        msg = msg + QObject::tr("<i>Your old machine data should be regenerated provided this backup feature has not been disabled in preferences during a previous data import.</i>") + "<br/><br/>";
+        msg = msg + QObject::tr("<i>Your old device data should be regenerated provided this backup feature has not been disabled in preferences during a previous data import.</i>") + "<br/><br/>";
         backups = true;
     } else {
         msg = msg + "<font size=+1>"+STR_MessageBox_Warning+":</font> "+QObject::tr("OSCAR does not yet have any automatic card backups stored for this device.") + "<br/><br/>";
-        msg = msg + QObject::tr("This means you will need to import this machine data again afterwards from your own backups or data card.") + "<br/><br/>";
+        msg = msg + QObject::tr("This means you will need to import this device data again afterwards from your own backups or data card.") + "<br/><br/>";
     }
 
     msg += "<font size=+1>"+QObject::tr("Important:")+"</font> "+QObject::tr("Once you upgrade, you <font size=+1>cannot</font> use this profile with the previous version anymore.")+"<br/><br/>"+
@@ -558,7 +561,7 @@ void Profile::DataFormatError(Machine *m)
     msg = msg + "<font size=+1>"+QObject::tr("Are you ready to upgrade, so you can run the new version of OSCAR?")+"</font>";
 
 
-    QMessageBox * question = new QMessageBox(QMessageBox::Warning, QObject::tr("Machine Database Changes"), msg, QMessageBox::Yes | QMessageBox::No);
+    QMessageBox * question = new QMessageBox(QMessageBox::Warning, QObject::tr("Device Database Changes"), msg, QMessageBox::Yes | QMessageBox::No);
     question->setDefaultButton(QMessageBox::Yes);
 
     QFont font("Sans Serif", 11, QFont::Normal);
@@ -570,7 +573,7 @@ void Profile::DataFormatError(Machine *m)
             // Purge failed.. probably a permissions error.. let the user deal with it.
             QMessageBox::critical(nullptr, STR_MessageBox_Error,
                                   QObject::tr("Sorry, the purge operation failed, which means this version of OSCAR can't start.")+"\n\n"+
-                                  QObject::tr("The machine data folder needs to be removed manually.")+"\n\n"+
+                                  QObject::tr("The device data folder needs to be removed manually.")+"\n\n"+
                                   QObject::tr("This folder currently resides at the following location:")+"\n\n"+
                                   QDir::toNativeSeparators(Get(p_preferences[STR_GEN_DataFolder].toString())), QMessageBox::Ok);
             QApplication::exit(-1);
@@ -799,7 +802,7 @@ Day *Profile::addDay(QDate date)
     return day;
 }
 
-// Get Day record if data available for date and machine type,
+// Get Day record if data available for date and device type,
 // and has enabled session data, else return nullptr
 Day *Profile::GetGoodDay(QDate date, MachineType type)
 {
@@ -807,7 +810,7 @@ Day *Profile::GetGoodDay(QDate date, MachineType type)
     if (!day)
         return nullptr;
 
-    // For a machine match, find at least one enabled Session.
+    // For a device match, find at least one enabled Session.
 
     for (auto & sess : day->sessions) {
         if (((type == MT_UNKNOWN) || (sess->type() == type)) && sess->enabled()) {
@@ -826,7 +829,7 @@ Day *Profile::FindGoodDay(QDate date, MachineType type)
     if (!day)
         return nullptr;
 
-    // For a machine match, find at least one enabled Session.
+    // For a device match, find at least one enabled Session.
     for (auto & sess : day->sessions) {
         if (((type == MT_UNKNOWN) || (sess->type() == type)) && sess->enabled()) {
             return day;
@@ -877,27 +880,6 @@ Day *Profile::FindDay(QDate date, MachineType type)
 }
 
 
-int Profile::Import(QString path)
-{
-    int c = 0;
-    qDebug() << "Importing " << path;
-    path = path.replace("\\", "/");
-
-    if (path.endsWith("/")) {
-        path.chop(1);
-    }
-
-    QList<MachineLoader *>loaders = GetLoaders(MT_CPAP);
-
-    for(auto & loader : loaders) {
-        if (c += loader->Open(path)) {
-            break;
-        }
-    }
-
-    return c;
-}
-
 MachineLoader *GetLoader(QString name)
 {
     QList<MachineLoader *> loaders = GetLoaders();
@@ -912,7 +894,7 @@ MachineLoader *GetLoader(QString name)
 }
 
 
-// Returns a QVector containing all machine objects regisered of type t
+// Returns a QVector containing all device objects regisered of type t
 QList<Machine *> Profile::GetMachines(MachineType t)
 {
     QList<Machine *> vec;
@@ -941,7 +923,7 @@ Machine *Profile::GetMachine(MachineType t)
         return nullptr;
     }
 
-    // Find most recently imported machine
+    // Find most recently imported device
     int idx = 0;
 
     for (int i=1; i < vec.size(); i++) {
@@ -1067,6 +1049,8 @@ Profile *Get()
     return profiles[getUserName()];;
 }
 
+//profiles.xml is never read so it does not need to be saved.
+#if 0
 void saveProfileList()
 {
     QString filename = p_pref->Get("{home}/profiles.xml");
@@ -1096,6 +1080,7 @@ void saveProfileList()
 
     file.close();
 }
+#endif
 
 int CleanupProfile(Profile *prof)
 {
@@ -1108,7 +1093,11 @@ int CleanupProfile(Profile *prof)
                 << STR_AS_AntiAliasing << STR_AS_LineThickness << STR_AS_UsePixmapCaching
                 << STR_AS_SquareWave << STR_AS_RightPanelWidth << STR_US_TooltipTimeout
                 << STR_AS_Animations << STR_AS_AllowYAxisScaling << STR_AS_GraphTooltips
-                << STR_CS_UserEventPieChart << STR_AS_OverlayType << STR_AS_OverviewLinechartMode;
+                << STR_CS_UserEventPieChart << STR_AS_OverlayType
+                #ifndef REMOVE_FITNESS
+                << STR_AS_OverviewLinechartMode
+                #endif
+                ;
 
     int cnt = 0;
     for (auto & prf :migrateList) {
@@ -1160,6 +1149,15 @@ void Scan()
         Profile *prof = new Profile(npath);
         //prof->Open();
 
+        // validate user name in profile.
+        QString dbname = prof->user->userName();
+        QString fname = QFileInfo(fi).fileName();
+        if (fname != dbname) {
+            prof->user->setUserName(fname);
+            QString message = QString("%1 %2 %3 %4").arg("Changing Profile Name").arg(dbname).arg("==>").arg(fname);
+            qDebug() << message;
+        }
+
         profiles[fi.fileName()] = prof;
 
         // Migrate any old settings
@@ -1171,14 +1169,15 @@ void Scan()
         p_pref->Save();
     }
     // Update profiles.xml for mobile version
-    saveProfileList();
+    // profiles.xml is never read so it does not need to be saved.
+   // saveProfileList();
 }
 
 
 } // namespace Profiles
 
 
-// Returns a list of all days records matching machine type between start and end date
+// Returns a list of all days records matching device type between start and end date
 QList<Day *> Profile::getDays(MachineType mt, QDate start, QDate end)
 {
     QList<Day *> list;
@@ -1219,7 +1218,7 @@ QList<Day *> Profile::getDays(MachineType mt, QDate start, QDate end)
     return list;
 }
 
-// Counts number of days in range with data for specified machine type
+// Counts number of days in range with data for specified device type
 int Profile::countDays(MachineType mt, QDate start, QDate end)
 {
     if (!start.isValid()) {
@@ -1276,7 +1275,7 @@ int Profile::countCompliantDays(MachineType mt, QDate start, QDate end)
         Day *day = FindGoodDay(date, mt);
 
         if (day) {
-            if (day->hours(mt) > compliance) { days++; }
+            if (day->hours(mt) >= compliance) { days++; }   // compliant days includes the value specified.
         }
 
         date = date.addDays(1);
@@ -1874,7 +1873,7 @@ EventDataType Profile::calcPercentile(ChannelID code, EventDataType percent, Mac
     return v;
 }
 
-// Lookup first day record of the specified machine type, or return the first day overall if MT_UNKNOWN
+// Lookup first day record of the specified device type, or return the first day overall if MT_UNKNOWN
 QDate Profile::FirstDay(MachineType mt)
 {
     if ((mt == MT_UNKNOWN) || (!m_last.isValid()) || (!m_first.isValid())) {
@@ -1894,7 +1893,7 @@ QDate Profile::FirstDay(MachineType mt)
     return m_last;
 }
 
-// Lookup last day record of the specified machine type, or return the last day overall if MT_UNKNOWN
+// Lookup last day record of the specified device type, or return the last day overall if MT_UNKNOWN
 QDate Profile::LastDay(MachineType mt)
 {
     if ((mt == MT_UNKNOWN) || (!m_last.isValid()) || (!m_first.isValid())) {
@@ -2007,7 +2006,8 @@ const quint16 chandata_version = 1;
 void Profile::saveChannels()
 {
     // First save the XML version for Mobile versions
-    schema::channel.Save(Get("{DataFolder}/") + "channels.xml");
+    // Profile/User/chanels.xml is not read so it does not need to be saved
+    // schema::channel.Save(Get("{DataFolder}/") + "channels.xml");
 
     QString filename = Get("{DataFolder}/") + "channels.dat";
     QFile f(filename);
@@ -2118,7 +2118,7 @@ void Profile::loadChannels()
 
         if (chan->isNull()) {
             qDebug() << "loadChannels has no idea about channel" << name;
-            if (in.atEnd()) return;
+            if (in.atEnd()) break;
             continue;
         }
         chan->setEnabled(enabled);
@@ -2139,8 +2139,14 @@ void Profile::loadChannels()
         chan->setUpperThresholdColor(upperThresholdColor);
 
         chan->setShowInOverview(showOverview);
-        if (in.atEnd()) return;
+        if (in.atEnd()) break;
     }
-
     f.close();
+    resetOxiChannelPref();
 }
+
+void Profile::resetOxiChannelPref() {
+    schema::channel[OXI_Pulse].setLowerThreshold(oxi->flagPulseBelow());
+    schema::channel[OXI_Pulse].setUpperThreshold(oxi->flagPulseAbove());
+    schema::channel[OXI_SPO2].setLowerThreshold(oxi->oxiDesaturationThreshold());
+};
