@@ -1,7 +1,7 @@
 /* SleepLib Profiles Header
  *
- * Copyright (c) 2019-2020 The OSCAR Team
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
@@ -38,7 +38,7 @@ class SessionSettings;
   \class Profile
   \author Mark Watkins
   \date 28/04/11
-  \brief The User profile system, containing all information for a user, and an index into all Machine data
+  \brief The User profile system, containing all information for a user, and an index into all device data
  */
 class Profile : public Preferences
 {
@@ -57,7 +57,7 @@ class Profile : public Preferences
     qint64 diskSpaceBackups();
     qint64 diskSpace();
 
-    //! \brief Force some preferences for ResMed machines
+    //! \brief Force some preferences for ResMed devices
     virtual void forceResmedPrefs();
 
     //! \brief Returns hostname that locked profile, or empty string if unlocked
@@ -71,16 +71,16 @@ class Profile : public Preferences
     //! \brief Save Profile object (This is an extension to Preference::Save(..))
     virtual bool Save(QString filename = "");
 
-    //! \brief Add machine to this profiles machlist
+    //! \brief Add device to this profiles machlist
     void AddMachine(Machine *m);
 
-    //! \brief Remove machine from this profiles machlist
+    //! \brief Remove device from this profiles machlist
     void DelMachine(Machine *m);
 
-    //! \brief Loads all machine (summary) data belonging to this profile
+    //! \brief Loads all device (summary) data belonging to this profile
     void LoadMachineData(ProgressDialog *progress);
 
-    //! \brief Unloads all machine (summary) data for this profile to free up memory;
+    //! \brief Unloads all device (summary) data for this profile to free up memory;
     void UnloadMachineData();
 
     //! \brief Barf because data format has changed. This does a purge of CPAP data for machine *m
@@ -88,11 +88,6 @@ class Profile : public Preferences
 
     QString path() { return p_path; }
 
-        /*! \brief Import Machine Data
-        \param path containing import location
-     */
-    int Import(QString path);
-
     //! \brief Removes a given day from the date, destroying the daylist date record if empty
     bool unlinkDay(Day * day);
 
@@ -101,50 +96,50 @@ class Profile : public Preferences
     //! \brief Add Day record to Profile Day list
     Day *addDay(QDate date);
 
-    //! \brief Get Day record if data available for date and machine type, else return nullptr
+    //! \brief Get Day record if data available for date and device type, else return nullptr
     Day *GetDay(QDate date, MachineType type = MT_UNKNOWN);
 
     //! \brief Same as GetDay but does not open the summaries
     Day *FindDay(QDate date, MachineType type = MT_UNKNOWN);
 
-    //! \brief Get Day record if data available for date and machine type,
+    //! \brief Get Day record if data available for date and device type,
     //         and has enabled session data, else return nullptr
     Day *GetGoodDay(QDate date, MachineType type);
 
     //! \breif Same as GetGoodDay but does not open the summaries
     Day *FindGoodDay(QDate date, MachineType type);
 
-    //! \brief Returns a list of all machines of type t
+    //! \brief Returns a list of all devices of type t
     QList<Machine *> GetMachines(MachineType t = MT_UNKNOWN);
 
-    //! \brief Returns the machine of type t used on date, nullptr if none..
+    //! \brief Returns the device of type t used on date, nullptr if none..
     Machine *GetMachine(MachineType t, QDate date);
 
-    //! \brief return the first machine of type t
+    //! \brief return the first device of type t
     Machine *GetMachine(MachineType t);
 
     //! \brief Returns true if this profile stores this variable identified by key
     bool contains(QString key) { return p_preferences.contains(key); }
 
 
-    //! \brief Get all days records of machine type between start and end dates
+    //! \brief Get all days records of device type between start and end dates
     QList<Day *> getDays(MachineType mt, QDate start, QDate end);
 
-    //! \brief Returns a count of all days (with data) of machine type, between start and end dates
+    //! \brief Returns a count of all days (with data) of device type, between start and end dates
     int countDays(MachineType mt = MT_UNKNOWN, QDate start = QDate(), QDate end = QDate());
 
-    //! \brief Returns a count of all compliant days of machine type between start and end dates
+    //! \brief Returns a count of all compliant days of device type between start and end dates
     int countCompliantDays(MachineType mt, QDate start, QDate end);
 
-    //! \brief Returns a count of all event entries for code, matching machine type between start an end dates
+    //! \brief Returns a count of all event entries for code, matching device type between start an end dates
     EventDataType calcCount(ChannelID code, MachineType mt = MT_CPAP, QDate start = QDate(),
                             QDate end = QDate());
 
-    //! \brief Returns a sum of all event data for Channel code, matching machine type between start an end dates
+    //! \brief Returns a sum of all event data for Channel code, matching device type between start an end dates
     double calcSum(ChannelID code, MachineType mt = MT_CPAP, QDate start = QDate(),
                    QDate end = QDate());
 
-    //! \brief Returns a sum of all session durations for machine type, between start and end dates
+    //! \brief Returns a sum of all session durations for device type, between start and end dates
     EventDataType calcHours(MachineType mt = MT_CPAP, QDate start = QDate(), QDate end = QDate());
 
     //! \brief Calculates Channel Average (Sums and counts all events, returning the sum divided by the count.)
@@ -171,7 +166,7 @@ class Profile : public Preferences
     bool hasChannel(ChannelID code);
 
 
-    //! \brief Looks up if any machines report channel is available
+    //! \brief Looks up if any devices report channel is available
     bool channelAvailable(ChannelID code);
 
 
@@ -183,27 +178,27 @@ class Profile : public Preferences
     EventDataType calcSettingsMax(ChannelID code, MachineType mt = MT_CPAP, QDate start = QDate(),
                                   QDate end = QDate());
 
-    //! \brief Calculates the time channel code spends above threshold value for machine type, between start and end dates
+    //! \brief Calculates the time channel code spends above threshold value for device type, between start and end dates
     EventDataType calcAboveThreshold(ChannelID code, EventDataType threshold, MachineType mt = MT_CPAP,
                                      QDate start = QDate(), QDate end = QDate());
 
-    //! \brief Calculates the time channel code spends below threshold value for machine type, between start and end dates
+    //! \brief Calculates the time channel code spends below threshold value for device type, between start and end dates
     EventDataType calcBelowThreshold(ChannelID code, EventDataType threshold, MachineType mt = MT_CPAP,
                                      QDate start = QDate(), QDate end = QDate());
 
 
     Day * findSessionDay(Session * session);
 
-    //! \brief Looks for the first date containing a day record matching machinetype
+    //! \brief Looks for the first date containing a day record matching devicetype
     QDate FirstDay(MachineType mt = MT_UNKNOWN);
 
-    //! \brief Looks for the last date containing a day record matching machinetype
+    //! \brief Looks for the last date containing a day record matching devicetype
     QDate LastDay(MachineType mt = MT_UNKNOWN);
 
-    //! \brief Looks for the first date containing a day record with enabled sessions matching machinetype
+    //! \brief Looks for the first date containing a day record with enabled sessions matching devicetype
     QDate FirstGoodDay(MachineType mt = MT_UNKNOWN);
 
-    //! \brief Looks for the last date containing a day record with enabled sessions matching machinetype
+    //! \brief Looks for the last date containing a day record with enabled sessions matching devicetype
     QDate LastGoodDay(MachineType mt = MT_UNKNOWN);
 
     //! \brief Returns this profiles data folder
@@ -221,6 +216,7 @@ class Profile : public Preferences
 
     void loadChannels();
     void saveChannels();
+    void resetOxiChannelPref();
 
 
     bool is_first_day;
@@ -299,17 +295,21 @@ const QString STR_OS_EnableOximetry = "EnableOximetry";
 const QString STR_OS_DefaultDevice = "DefaultOxiDevice";
 const QString STR_OS_SyncOximeterClock = "SyncOximeterClock";
 const QString STR_OS_OximeterType = "OximeterType";
-const QString STR_OS_OxiDiscardThreshold = "OxiDiscardThreshold";
+const QString STR_OS_SkipOxiIntroScreen = "SkipOxiIntroScreen";
+
 const QString STR_OS_SPO2DropDuration = "SPO2DropDuration";
 const QString STR_OS_SPO2DropPercentage = "SPO2DropPercentage";
 const QString STR_OS_PulseChangeDuration = "PulseChangeDuration";
 const QString STR_OS_PulseChangeBPM = "PulseChangeBPM";
-const QString STR_OS_SkipOxiIntroScreen = "SkipOxiIntroScreen";
+const QString STR_OS_oxiDesaturationThreshold = "oxiDesaturationThreshold";
+const QString STR_OS_flagPulseAbove = "flagPulseAbove";
+const QString STR_OS_flagPulseBelow = "flagPulseBelow";
+const QString STR_OS_OxiDiscardThreshold = "OxiDiscardThreshold";
 
 
 // CPAPSettings Strings
 const QString STR_CS_ComplianceHours = "ComplianceHours";
-const QString STR_CS_ShowCompliance = "ShowCompliance";
+const QString STR_CS_ClinicalMode = "ClinicalMode";
 const QString STR_CS_ShowLeaksMode = "ShowLeaksMode";
 const QString STR_CS_MaskStartDate = "MaskStartDate";
 const QString STR_CS_MaskDescription = "MaskDescription";
@@ -355,6 +355,8 @@ const QString STR_IS_LockSummarySessions = "LockSummarySessions";
 const QString STR_IS_WarnOnUntestedMachine = "WarnOnUntestedMachine";
 const QString STR_IS_WarnOnUnexpectedData = "WarnOnUnexpectedData";
 
+//Apperance Settings
+const QString STR_AS_EventFlagSessionBar = "EventFlagSessionBar";
 
 // UserSettings Strings
 const QString STR_US_UnitSystem = "UnitSystem";
@@ -368,7 +370,12 @@ const QString STR_US_PrefCalcPercentile = "PrefCalcPercentile";
 const QString STR_US_PrefCalcMax = "PrefCalcMax";
 const QString STR_US_ShowUnknownFlags = "ShowUnknownFlags";
 const QString STR_US_StatReportMode = "StatReportMode";
+const QString STR_US_StatReportDate = "StatReportDate";
+const QString STR_US_StatReportRangeStart = "StatReportRangeStart";
+const QString STR_US_StatReportRangeEnd = "StatReportRangeEnd";
 const QString STR_US_LastOverviewRange = "LastOverviewRange";
+const QString STR_US_CustomOverviewRangeStart = "CustomOverviewRangeStart";
+const QString STR_US_CustomOverviewRangeEnd = "CustomOverviewRangeEnd";
 
 // Values for StatReportMode
 const int STAT_MODE_STANDARD = 0;
@@ -480,29 +487,52 @@ class OxiSettings : public PrefSettings
     OxiSettings(Profile *profile)
       : PrefSettings(profile)
     {
+
+        // Intialized non-user changable item - set during import of data?
         initPref(STR_OS_EnableOximetry, false);
         initPref(STR_OS_DefaultDevice, QString());
         initPref(STR_OS_SyncOximeterClock, true);
         initPref(STR_OS_OximeterType, 0);
-        initPref(STR_OS_OxiDiscardThreshold, 0.0);
-        initPref(STR_OS_SPO2DropDuration, 8.0);
-        initPref(STR_OS_SPO2DropPercentage, 3.0);
-        initPref(STR_OS_PulseChangeDuration, 8.0);
-        initPref(STR_OS_PulseChangeBPM, 5.0);
         initPref(STR_OS_SkipOxiIntroScreen, false);
+
+        // Initialize Changeable via GUI parameters with default values
+        initPref(STR_OS_SPO2DropDuration, defaultValue_OS_SPO2DropDuration);
+        initPref(STR_OS_SPO2DropPercentage, defaultValue_OS_SPO2DropPercentage);
+        initPref(STR_OS_PulseChangeDuration, defaultValue_OS_PulseChangeDuration);
+        initPref(STR_OS_PulseChangeBPM, defaultValue_OS_PulseChangeBPM);
+
+        initPref(STR_OS_OxiDiscardThreshold, defaultValue_OS_OxiDiscardThreshold);
+        initPref(STR_OS_oxiDesaturationThreshold, defaultValue_OS_oxiDesaturationThreshold);
+        initPref(STR_OS_flagPulseAbove, defaultValue_OS_flagPulseAbove);
+        initPref(STR_OS_flagPulseBelow, defaultValue_OS_flagPulseBelow);
+
     }
 
+    const double defaultValue_OS_SPO2DropDuration = 8.0;
+    const double defaultValue_OS_SPO2DropPercentage = 3.0;
+    const double defaultValue_OS_PulseChangeDuration = 8.0;
+    const double defaultValue_OS_PulseChangeBPM = 5.0;
+
+    const double defaultValue_OS_OxiDiscardThreshold = 0.0;
+    const double defaultValue_OS_oxiDesaturationThreshold = 88.0;
+    const double defaultValue_OS_flagPulseAbove = 99.0;
+    const double defaultValue_OS_flagPulseBelow = 40.0;
+
     bool oximetryEnabled() const { return getPref(STR_OS_EnableOximetry).toBool(); }
     QString defaultDevice() const { return getPref(STR_OS_DefaultDevice).toString(); }
     bool syncOximeterClock() const { return getPref(STR_OS_SyncOximeterClock).toBool(); }
     int oximeterType() const { return getPref(STR_OS_OximeterType).toInt(); }
-    double oxiDiscardThreshold() const { return getPref(STR_OS_OxiDiscardThreshold).toDouble(); }
+    bool skipOxiIntroScreen() const { return getPref(STR_OS_SkipOxiIntroScreen).toBool(); }
+
     double spO2DropDuration() const { return getPref(STR_OS_SPO2DropDuration).toDouble(); }
     double spO2DropPercentage() const { return getPref(STR_OS_SPO2DropPercentage).toDouble(); }
     double pulseChangeDuration() const { return getPref(STR_OS_PulseChangeDuration).toDouble(); }
     double pulseChangeBPM() const { return getPref(STR_OS_PulseChangeBPM).toDouble(); }
-    bool skipOxiIntroScreen() const { return getPref(STR_OS_SkipOxiIntroScreen).toBool(); }
 
+    double oxiDiscardThreshold() const { return getPref(STR_OS_OxiDiscardThreshold).toDouble(); }
+    double  oxiDesaturationThreshold() const { return getPref(STR_OS_oxiDesaturationThreshold).toDouble(); }
+    double  flagPulseAbove() const { return getPref(STR_OS_flagPulseAbove).toDouble(); }
+    double  flagPulseBelow() const { return getPref(STR_OS_flagPulseBelow).toDouble(); }
 
     void setOximetryEnabled(bool enabled) { setPref(STR_OS_EnableOximetry, enabled); }
     void setDefaultDevice(QString name) { setPref(STR_OS_DefaultDevice, name); }
@@ -518,6 +548,9 @@ class OxiSettings : public PrefSettings
     void setPulseChangeDuration(double duration) {
         setPref(STR_OS_PulseChangeDuration, duration);
     }
+    void setOxiDesaturationThreshold(double value) { setPref(STR_OS_oxiDesaturationThreshold, value); }
+    void setFlagPulseAbove(double value) { setPref(STR_OS_flagPulseAbove, value); }
+    void setFlagPulseBelow(double value) { setPref(STR_OS_flagPulseBelow, value); }
 };
 
 /*! \class CPAPSettings
@@ -530,9 +563,9 @@ class CPAPSettings : public PrefSettings
       : PrefSettings(profile)
     {
         m_complianceHours = initPref(STR_CS_ComplianceHours, 4.0f).toFloat();
-        initPref(STR_CS_ShowCompliance, true);
+        m_clinicalMode = initPref(STR_CS_ClinicalMode, false).toBool();
         initPref(STR_CS_ShowLeaksMode, 0);
-        // TODO: jedimark: Check if this date is initiliazed yet
+        // TODO: Check if this date is initiliazed yet
         initPref(STR_CS_MaskStartDate, QDate());
         initPref(STR_CS_MaskDescription, QString());
         initPref(STR_CS_MaskType, Mask_Unknown);
@@ -556,6 +589,9 @@ class CPAPSettings : public PrefSettings
         initPref(STR_CS_AutoImport, false);
         initPref(STR_CS_BrickWarning, true);
 
+        // From old zMaskProfile::calcLeak comments:
+        // Average mask leak minimum at pressure 4 = 20.167
+        // Average mask slope = 1.76
         m_calcUnintentionalLeaks = initPref(STR_CS_CalculateUnintentionalLeaks, true).toBool();
         m_4cmH2OLeaks = initPref(STR_CS_4cmH2OLeaks, 20.167).toDouble();
         m_20cmH2OLeaks = initPref(STR_CS_20cmH2OLeaks, 48.333).toDouble();
@@ -564,7 +600,7 @@ class CPAPSettings : public PrefSettings
 
     //Getters
     double complianceHours() const { return m_complianceHours; }
-    bool showComplianceInfo() const { return getPref(STR_CS_ShowCompliance).toBool(); }
+    bool clinicalMode() const { return m_clinicalMode; }
     int leakMode() const { return getPref(STR_CS_ShowLeaksMode).toInt(); }
     QDate maskStartDate() const { return getPref(STR_CS_MaskStartDate).toDate(); }
     QString maskDescription() const { return getPref(STR_CS_MaskDescription).toString(); }
@@ -602,7 +638,7 @@ class CPAPSettings : public PrefSettings
     void setNotes(QString notes) { setPref(STR_CS_Notes, notes); }
     void setDateDiagnosed(QDate date) { setPref(STR_CS_DateDiagnosed, date); }
     void setComplianceHours(EventDataType hours) { setPref(STR_CS_ComplianceHours, m_complianceHours=hours); }
-    void setShowComplianceInfo(bool b) { setPref(STR_CS_ShowCompliance, b); }
+    void setClinicalMode(bool mode) { setPref(STR_CS_ClinicalMode, m_clinicalMode=mode); }
     void setLeakMode(int leakmode) { setPref(STR_CS_ShowLeaksMode, (int)leakmode); }
     void setMaskStartDate(QDate date) { setPref(STR_CS_MaskStartDate, date); }
     void setMaskType(MaskType masktype) { setPref(STR_CS_MaskType, (int)masktype); }
@@ -630,7 +666,7 @@ class CPAPSettings : public PrefSettings
     int m_clock_drift;
     double m_4cmH2OLeaks, m_20cmH2OLeaks;
     bool m_userEventFlagging, m_userEventDuplicates, m_calcUnintentionalLeaks, m_resyncFromUserFlagging, m_ahiReset;
-    bool m_showLeakRedline;
+    bool m_showLeakRedline, m_clinicalMode;
 
     EventDataType m_leakRedLine, m_complianceHours, m_ahiWindow;
 
@@ -705,8 +741,16 @@ class AppearanceSettings : public PrefSettings
     AppearanceSettings(Profile *profile)
       : PrefSettings(profile)
     {
-
+        m_eventFlagSessionBar = initPref(STR_AS_EventFlagSessionBar, false).toBool();
     }
+
+    //Getters
+    bool eventFlagSessionBar() const { return m_eventFlagSessionBar; }
+
+    //Setters
+    void setEventFlagSessionBar(bool b) { setPref(STR_AS_EventFlagSessionBar, m_eventFlagSessionBar = b); }
+
+    bool m_eventFlagSessionBar;
 };
 
 /*! \class UserSettings
@@ -719,14 +763,17 @@ class UserSettings : public PrefSettings
       : PrefSettings(profile)
     {
         initPref(STR_US_UnitSystem, US_Metric);
-        initPref(STR_US_EventWindowSize, 4.0);
+        setPref(STR_US_EventWindowSize, 3.0);
         m_skipEmptyDays = initPref(STR_US_SkipEmptyDays, true).toBool();
-        initPref(STR_US_RebuildCache, false); // FIXME: jedimark: can't remember...
+        initPref(STR_US_RebuildCache, false); // FIXME: can't remember...
         m_calculateRDI = initPref(STR_US_CalculateRDI, false).toBool();
         m_prefCalcMiddle = initPref(STR_US_PrefCalcMiddle, (int)0).toInt();
         m_prefCalcPercentile = initPref(STR_US_PrefCalcPercentile, (double)95.0).toDouble();
         m_prefCalcMax = initPref(STR_US_PrefCalcMax, (int)0).toInt();
         initPref(STR_US_StatReportMode, 0);
+        initPref(STR_US_StatReportDate, QDate(1,1,2000));
+        initPref(STR_US_StatReportRangeStart, QDate(1,1,2000));
+        initPref(STR_US_StatReportRangeEnd, QDate(1,1,2000));
         m_showUnownFlags = initPref(STR_US_ShowUnknownFlags, false).toBool();
         initPref(STR_US_LastOverviewRange, 4);
     }
@@ -740,8 +787,13 @@ class UserSettings : public PrefSettings
     inline double prefCalcPercentile() const { return m_prefCalcPercentile; }
     inline int prefCalcMax() const { return m_prefCalcMax; }
     int statReportMode() const { return getPref(STR_US_StatReportMode).toInt(); }
+    QDate statReportDate() const { return getPref(STR_US_StatReportDate).toDate(); }
+    QDate statReportRangeStart() const { return getPref(STR_US_StatReportRangeStart).toDate(); }
+    QDate statReportRangeEnd() const { return getPref(STR_US_StatReportRangeEnd).toDate(); }
     inline bool showUnknownFlags() const { return m_showUnownFlags; }
     int lastOverviewRange() const { return getPref(STR_US_LastOverviewRange).toInt(); }
+    QDate customOverviewRangeStart () const { return getPref(STR_US_CustomOverviewRangeStart).toDate(); }
+    QDate customOverviewRangeEnd () const { return getPref(STR_US_CustomOverviewRangeEnd).toDate(); }
 
     void setUnitSystem(UnitSystem us) { setPref(STR_US_UnitSystem, (int)us); }
     void setEventWindowSize(double size) { setPref(STR_US_EventWindowSize, size); }
@@ -752,8 +804,13 @@ class UserSettings : public PrefSettings
     void setPrefCalcPercentile(double p) { setPref(STR_US_PrefCalcPercentile, m_prefCalcPercentile=p); }
     void setPrefCalcMax(int i) { setPref(STR_US_PrefCalcMax, m_prefCalcMax=i); }
     void setStatReportMode(int i) { setPref(STR_US_StatReportMode, i); }
+    void setStatReportDate(QDate i) { setPref(STR_US_StatReportDate, i); }
+    void setStatReportRangeStart(QDate i) { setPref (STR_US_StatReportRangeStart, i); }
+    void setStatReportRangeEnd(QDate i) { setPref (STR_US_StatReportRangeEnd, i); }
     void setShowUnknownFlags(bool b) { setPref(STR_US_ShowUnknownFlags, m_showUnownFlags=b); }
     void setLastOverviewRange(int i) { setPref(STR_US_LastOverviewRange, i); }
+    void setCustomOverviewRangeStart(QDate i) { setPref(STR_US_CustomOverviewRangeStart, i); }
+    void setCustomOverviewRangeEnd(QDate i) { setPref(STR_US_CustomOverviewRangeEnd, i); }
 
     bool m_calculateRDI, m_showUnownFlags,  m_skipEmptyDays;
     int m_prefCalcMiddle, m_prefCalcMax;
