@@ -1,13 +1,16 @@
 /* SleepLib Session Implementation
  * This stuff contains the base calculation smarts
  *
- * Copyright (c) 2019-2020 The OSCAR Team
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
  * for more details. */
 
+#define TEST_MACROS_ENABLEDoff
+#include <test_macros.h>
+
 #include "session.h"
 #include "version.h"
 #include <cmath>
@@ -18,10 +21,12 @@
 #include <algorithm>
 #include <limits>
 
+#include "SleepLib/machine_common.h"
 #include "SleepLib/calcs.h"
 #include "SleepLib/profiles.h"
 
 using namespace std;
+#define FIX_FOR_SINGLE_EVENT            // fixes ibreeze "No valuesummary for channel" error during import.
 
 // This is the uber important database version for OSCAR's internal storage
 // Increment this after stuffing with Session's save & load code.
@@ -88,6 +93,12 @@ void Session::TrashEvents()
     eventlist.squeeze();
 }
 
+bool Session::enabled(bool realValues) const
+{
+    if (p_profile->cpap->clinicalMode() && !realValues) return true;
+    return s_enabled;
+}
+
 void Session::setEnabled(bool b)
 {
     s_enabled = b;
@@ -101,11 +112,11 @@ void Session::setEnabled(bool b)
 
 QString Session::eventFile() const
 {
-    return s_machine->getEventsPath()+QString().sprintf("%08lx.001", s_session);
+    return s_machine->getEventsPath()+toHexid(s_session)+".001";
 }
 
 //const int max_pack_size=128;
-bool Session::OpenEvents()
+bool Session::OpenEvents(bool debug)
 {
     if (s_events_loaded) {
         return true;
@@ -119,11 +130,13 @@ bool Session::OpenEvents()
 
 
     QString filename = eventFile();
-    qDebug() << "Loading" << s_machine->loaderName().toLocal8Bit().data() << "Events:" << filename.toLocal8Bit().data();
-    bool b = LoadEvents(filename);
+#ifdef DEBUG_EVENTS
+    if (debug) qDebug() << "Loading" << s_machine->loaderName().toLocal8Bit().data() << "Events:" << filename.toLocal8Bit().data();
+#endif
+    bool b = LoadEvents(filename,debug);
 
     if ( ! b) {
-        qWarning() << "Error Loading Events" << filename;
+        if (debug) qWarning() << "Error Loading Events" << filename;
         return false;
     }
 
@@ -134,7 +147,7 @@ bool Session::Destroy()
 {
     QDir dir;
     QString base;
-    base.sprintf("%08lx", s_session);
+    base=toHexid(s_session);
 
     QString summaryfile = s_machine->getSummariesPath() + base + ".000";
     QString eventfile = s_machine->getEventsPath() + base + ".001";
@@ -277,7 +290,7 @@ bool Session::Store(QString path)
 //    in >> s_summaryOnly;
 //
 //    s_enabled = 1;
-//} 
+//}
 
 QDataStream & operator>>(QDataStream & in, SessionSlice & slice)
 {
@@ -309,7 +322,7 @@ bool Session::StoreSummary()
         return false;
     }
 
-    QString filename = s_machine->getSummariesPath() + QString().sprintf("%08lx.000", s_session);
+    QString filename = s_machine->getSummariesPath() + toHexid(s_session) + ".000" ;
 
     QFile file(filename);
     if (!file.open(QIODevice::WriteOnly)) {
@@ -381,22 +394,22 @@ bool Session::StoreSummary()
 }
 
 
-bool Session::LoadSummary()
+bool Session::LoadSummary(bool debug)
 {
 //    static int sumcnt = 0;
 
     if (s_summary_loaded) return true;
-    QString filename = s_machine->getSummariesPath() + QString().sprintf("%08lx.000", s_session);
+    QString filename = s_machine->getSummariesPath() + toHexid(s_session) + ".000";
 
     if (filename.isEmpty()) {
-        qDebug() << "Empty summary filename";
+        if (debug) qDebug() << "Empty summary filename";
         return false;
     }
 
     QFile file(filename);
 
     if (!file.open(QIODevice::ReadOnly)) {
-        qWarning() << "Could not open summary file" << filename << "for reading, error code" << file.error() << file.errorString();
+        if (debug) qWarning() << "Could not open summary file" << filename << "for reading, error code" << file.error() << file.errorString();
         return false;
     }
 
@@ -415,7 +428,7 @@ bool Session::LoadSummary()
     in >> t32;
 
     if (t32 != magic) {
-        qDebug() << "Wrong magic number in " << filename;
+        if (debug) qDebug() << "Wrong magic number in " << filename;
         return false;
     }
 
@@ -437,11 +450,11 @@ bool Session::LoadSummary()
     }
 
 
-    qint32 ts32;
+    quint32 ts32;
     in >> ts32;      // MachineID (dont need this result)
 
     bool upgrade = false;
-    if (ts32 != s_machine->id()) {
+    if ( ts32 != s_machine->id()) {
         upgrade = true;
         qWarning() << "Machine ID does not match in" << filename <<
                    " I will try to load anyway in case you know what your doing.";
@@ -611,7 +624,7 @@ bool Session::LoadSummary()
             in >> m_upperThreshold;
             in >> m_timeBelowTheshold;
             in >> m_lowerThreshold;
-        } // else this is ugly.. forced machine database upgrade will solve it though.
+        } // else this is ugly.. forced device database upgrade will solve it though.
 
         if (version == 13) {
             QHash<ChannelID, QVariant>::iterator it = settings.find(CPAP_SummaryOnly);
@@ -667,7 +680,7 @@ bool Session::StoreEvents()
     QString path = s_machine->getEventsPath();
     QDir dir;
     dir.mkpath(path);
-    QString filename = path+QString().sprintf("%08lx.001", s_session);
+    QString filename = path+ toHexid(s_session) +".001";
 
     QFile file(filename);
     if (!file.open(QIODevice::WriteOnly)) {
@@ -683,7 +696,7 @@ bool Session::StoreEvents()
     header << (quint32)magic;      // New Magic Number
     header << (quint16)events_version; // File Version
     header << (quint16)filetype_data;  // File type 1 == Event
-    header << (quint32)s_machine->id();// Machine Type
+    header << (quint32)s_machine->id();// Device Type
     header << (quint32)s_session;      // This session's ID
     header << s_first;
     header << s_last;
@@ -696,7 +709,7 @@ bool Session::StoreEvents()
 
     header << (quint16)compress;
 
-    header << (quint16)s_machine->type();// Machine Type
+    header << (quint16)s_machine->type();// Device Type
 
     QByteArray databytes;
     QDataStream out(&databytes, QIODevice::WriteOnly);
@@ -802,7 +815,7 @@ bool Session::StoreEvents()
     return true;
 }
 
-bool Session::LoadEvents(QString filename)
+bool Session::LoadEvents(QString filename, bool debug)
 {
     quint32 magicnum, machid, sessid;
     quint16 version, type, crc16, machtype, compmethod;
@@ -816,9 +829,9 @@ bool Session::LoadEvents(QString filename)
 
     QFile file(filename);
 
-    if (!file.open(QIODevice::ReadOnly)) {
+    if ( ! file.open(QIODevice::ReadOnly)) {
 //        qDebug() << "No Event/Waveform data available for" << s_session;
-        qWarning() << "No Event/Waveform data available for" << s_session << "filename" << filename << "error code" << file.error() << file.errorString();
+        if (debug) qWarning() << "No Event/Waveform data available for" << s_session << "filename" << filename << "error code" << file.error() << file.errorString();
         return false;
     }
 
@@ -831,11 +844,15 @@ bool Session::LoadEvents(QString filename)
     header >> magicnum;         // Magic Number (quint32)
     header >> version;          // Version (quint16)
     header >> type;             // File type (quint16)
-    header >> machid;           // Machine ID (quint32)
+    header >> machid;           // Device ID (quint32)
     header >> sessid;           //(quint32)
     header >> s_first;          //(qint64)
     header >> s_last;           //(qint64)
 
+#ifdef DEBUG_EVENTS
+    qDebug() << "Session ID" << sessid << "Start Time" << QDateTime::fromMSecsSinceEpoch(s_first);
+#endif
+
     if (type != filetype_data) {
         qDebug() << "Wrong File Type in " << filename;
         return false;
@@ -855,7 +872,7 @@ bool Session::LoadEvents(QString filename)
         file.seek(32);
     } else {
         header >> compmethod;   // Compression Method (quint16)
-        header >> machtype;     // Machine Type (quint16)
+        header >> machtype;     // Device Type (quint16)
         header >> datasize;     // Size of Uncompressed Data (quint32)
         header >> crc16;        // CRC16 of Uncompressed Data (quint16)
     }
@@ -892,7 +909,10 @@ bool Session::LoadEvents(QString filename)
     in.setByteOrder(QDataStream::LittleEndian);
 
     qint16 mcsize;
-    in >> mcsize;   // number of Machine Code lists
+    in >> mcsize;   // number of Device Code lists
+#ifdef DEBUG_EVENTS
+    qDebug() << "Number of Channels" << mcsize;
+#endif
 
     ChannelID code;
     qint64 ts1, ts2;
@@ -916,10 +936,17 @@ bool Session::LoadEvents(QString filename)
         mcorder.push_back(code);
         in >> size2;
         sizevec.push_back(size2);
+#ifdef DEBUG_EVENTS
+        qDebug() << "For Channel (hex)" << QString::number(code, 16) << "there are" << size2 << "EventLists";
+#endif
 
         for (int j = 0; j < size2; j++) {
             in >> ts1;
             in >> ts2;
+#ifdef DEBUG_EVENTS
+            qDebug() << "Start:" << QDateTime::fromMSecsSinceEpoch(ts1).toString() <<
+                        "Finish:" << QDateTime::fromMSecsSinceEpoch(ts2).toString();
+#endif
             in >> evcount;
             in >> t8;
             elt = (EventListType)t8;
@@ -953,7 +980,7 @@ bool Session::LoadEvents(QString filename)
         }
     }
 
-    //EventStoreType t;
+    //EventStoreType t;     // qint16
     //quint32 x;
 
     for (int i = 0; i < mcsize; i++) {
@@ -970,6 +997,7 @@ bool Session::LoadEvents(QString filename)
             in.readRawData((char *)ptr, evec.m_count << 1);
 
             //*** Don't delete these comments ***
+            //*** They explain what the above ReadRawData is doing!
             //            for (quint32 c=0;c<evec.m_count;c++) {
             //                in >> t;
             //                *ptr++=t;
@@ -980,6 +1008,7 @@ bool Session::LoadEvents(QString filename)
 
                 in.readRawData((char *)ptr, evec.m_count << 1);
                 //*** Don't delete these comments ***
+                //*** They explain what the above ReadRawData is doing!
                 //                for (quint32 c=0;c<evec.m_count;c++) {
                 //                    in >> t;
                 //                    *ptr++=t;
@@ -1039,13 +1068,16 @@ void Session::destroyEvent(ChannelID code)
 
 // TODO: The below assumes values are held for their duration. This does not properly handle
 // CPAP_PressureSet/EPAPSet/IPAPSet or other interpolated channels. The proper "value" held
-// for any given duration is the average of the starding and ending values, for the duration
+// for any given duration is the average of the starting and ending values, for the duration
 // between them.
 void Session::updateCountSummary(ChannelID code)
 {
     QHash<ChannelID, QVector<EventList *> >::iterator ev = eventlist.find(code);
 
-    if (ev == eventlist.end()) { return; }
+    if (ev == eventlist.end()) {
+        qDebug() << "No events for channel (hex)" << QString::number(code, 16);
+        return;
+    }
 
     QHash<ChannelID, QHash<EventStoreType, EventStoreType> >::iterator vs = m_valuesummary.find(code);
 
@@ -1078,9 +1110,20 @@ void Session::updateCountSummary(ChannelID code)
         m_gain[code] = e.gain();
 
         if (e.type() == EVL_Event) {
-            lastraw = *dptr++;
             tptr = e.rawTime();
+            // The last event event raw time is never updated with the len to the next.
+            // this is true for multiple samples or for a single sample.
+            // a single sample is the first as well as the last sample.
+            // so a single sample should work.
+            // the iBreeze loader had snore events with a single snore sample for a session.
+            // which triggered this investigation.
+            #if defined(FIX_FOR_SINGLE_EVENT)
+            lastraw = *dptr;
+            lasttime = start + *tptr;
+            #else
+            lastraw = *dptr++;
             lasttime = start + *tptr++;
+            #endif
             // Event version
 
             for (; dptr < eptr; dptr++) {
@@ -1096,7 +1139,7 @@ void Session::updateCountSummary(ChannelID code)
 
                 lastraw = raw;
                 lasttime = time;
-            }
+            } 
         } else {
             // Waveform version, first just count
             for (; dptr < eptr; dptr++) {
@@ -1118,7 +1161,19 @@ void Session::updateCountSummary(ChannelID code)
         }
     }
 
-    if (valsum.size() == 0) { return; }
+    if ( valsum.size() == 0) {  // no value summary for this channel
+        using namespace schema;
+        Channel *ch_p = channel.channels[code];
+        if (  ! ch_p->isNull() ) {                      // the channel was found in the channel list
+            if ( ((ch_p->type() & (FLAG|SPAN|MINOR_FLAG)) == 0) ) {  // the channel is not a flag or span type
+                qDebug() << "No valuesummary for channel " << ch_p->label() <<  " " << QDateTime::fromMSecsSinceEpoch( realFirst()).toString() ;    // so tell about missing summary
+            }
+        } else {
+            // This channel wasn't added to the channel list, so we can't check its type
+            qDebug() << "No valuesummary for channel (hex)" << QString::number(code, 16);
+        }
+        return;
+    }
 
     m_valuesummary[code] = valsum;
     m_timesummary[code] = timesum;
@@ -1137,7 +1192,7 @@ void Session::UpdateSummaries()
     // Generate unintentional leaks if not present
     calcLeaks(this);
 
-    // Flag the Large Leaks if unintentional leaks is available, and no LargeLeaks weren't flagged by the machine already.
+    // Flag the Large Leaks if unintentional leaks is available, and no LargeLeaks weren't flagged by the device already.
     flagLargeLeaks(this);
 
     calcSPO2Drop(this);
@@ -1192,6 +1247,11 @@ void Session::UpdateSummaries()
 
 EventDataType Session::SearchValue(ChannelID code, qint64 time, bool square)
 {
+    qint64 drift = qint64(p_profile->cpap->clockDrift()) * 1000L;
+    // Address clock drift for CPAP so correct value is displayed
+    if (s_machine->type() == MT_CPAP) {
+        time -= drift;
+    }
     qint64 t1, t2, start;
     QHash<ChannelID, QVector<EventList *> >::iterator it;
     it = eventlist.find(code);
@@ -1220,13 +1280,15 @@ EventDataType Session::SearchValue(ChannelID code, qint64 time, bool square)
 
                     a = el->data(i1);
 
-                    if (i2 >= cnt) { return a; }
+                    // Don't interpolate if next data point is past end or on exact data point
+                    if (i2 >= cnt || i1 == i2) { return a; }
 
                     qint64 t1 = i1 * el->rate();
                     qint64 t2 = i2 * el->rate();
 
                     c = EventDataType(t2 - t1);
-                    if (c == 0) return 0;
+                    // Don't interpolate if t2-t1==0 (should be caught by i1==i2 above)
+                    if (c == 0) return a;
                     d = EventDataType(t2 - tt);
 
                     e = d/c;
@@ -1517,7 +1579,9 @@ bool Session::channelDataExists(ChannelID id)
 }
 bool Session::channelExists(ChannelID id)
 {
-    if (!enabled()) { return false; }
+    if ( ! enabled()) {
+        return false;
+    }
 
     if (s_events_loaded) {
         QHash<ChannelID, QVector<EventList *> >::iterator j = eventlist.find(id);
@@ -1532,7 +1596,9 @@ bool Session::channelExists(ChannelID id)
             return false;
         }
 
-        if (q.value() == 0) { return false; }
+        if (q.value() == 0) {
+            return false;
+        }
     }
 
     return true;
@@ -1595,6 +1661,14 @@ EventDataType Session::countInsideSpan(ChannelID span, ChannelID code)
 
 EventDataType Session::rangeCount(ChannelID id, qint64 first, qint64 last)
 {
+    int total = 0, cnt;
+
+    if (id == AllAhiChannels) {
+        for (int i = 0; i < ahiChannels.size(); i++)
+            total += rangeCount(ahiChannels.at(i), first, last);
+        return (EventDataType)total;
+    }
+
     QHash<ChannelID, QVector<EventList *> >::iterator j = eventlist.find(id);
 
     if (j == eventlist.end()) {
@@ -1602,7 +1676,6 @@ EventDataType Session::rangeCount(ChannelID id, qint64 first, qint64 last)
     }
 
     QVector<EventList *> &evec = j.value();
-    int total = 0, cnt;
 
     qint64 t, start;
 
@@ -1869,6 +1942,14 @@ EventDataType Session::rangeMax(ChannelID id, qint64 first, qint64 last)
 
 EventDataType Session::count(ChannelID id)
 {
+    int sum = 0;
+
+    if (id == AllAhiChannels) {
+        for (int i = 0; i < ahiChannels.size(); i++)
+            sum += count(ahiChannels.at(i));
+        return sum;
+    }
+
     QHash<ChannelID, EventDataType>::iterator i = m_cnt.find(id);
 
     if (i != m_cnt.end()) {
@@ -1884,7 +1965,6 @@ EventDataType Session::count(ChannelID id)
 
     QVector<EventList *> &evec = j.value();
 
-    int sum = 0;
     int evec_size=evec.size();
     if (evec_size == 0)
         return 0;
