@@ -2,8 +2,8 @@
  *
  * This stuff contains the session calculation smarts
  *
- * Copyright (c) 2019 The OSCAR Team
- * Copyright (C) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (C) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
@@ -51,7 +51,7 @@ public:
 };
 
 /*! \class Session
-    \brief Contains a single Sessions worth of machine event/waveform information.
+    \brief Contains a single Sessions worth of device event/waveform information.
 
     This class also contains all the primary database logic for SleepLib
     */
@@ -61,7 +61,7 @@ class Session
     friend class Machine;
   public:
     /*! \fn Session(Machine *,SessionID);
-        \brief Create a session object belonging to Machine, with supplied SessionID
+        \brief Create a session object belonging to device, with supplied SessionID
         If sessionID is 0, the next in sequence will be picked
         */
     Session(Machine *, SessionID);
@@ -90,13 +90,16 @@ class Session
 //    void LoadSummaryData(QDataStream & in);
 
     //! \brief Loads the Sessions Summary Indexes from filename, from SleepLibs custom data format.
-    bool LoadSummary();
+    // debug option is set to false because file errors are normal when there is summary but no details data
+    bool LoadSummary(bool debug=false);
 
     //! \brief Loads the Sessions EventLists from filename, from SleepLibs custom data format.
-    bool LoadEvents(QString filename);
+    // debug option is set to false because file errors are normal when there is summary but no details data
+    bool LoadEvents(QString filename, bool debug=false);
 
     //! \brief Loads the events for this session when requested (only the summaries are loaded at startup)
-    bool OpenEvents();
+    // debug option is set to false because file errors are normal when there is summary but no details data
+    bool OpenEvents(bool debug=false);
 
     //! \brief Put the events away until needed again, freeing memory
     void TrashEvents();
@@ -113,7 +116,7 @@ class Session
     }
 
     //! \brief Returns whether or not session is being used.
-    inline bool enabled() const { return s_enabled; }
+    bool enabled(bool realValues=false) const;
 
     //! \brief Sets whether or not session is being used.
     void setEnabled(bool b);
@@ -132,7 +135,8 @@ class Session
 
     //! \brief Return the millisecond length of this session
     qint64 length() {
-        return s_last - s_first;
+        qint64 duration=s_last - s_first;
+        return duration<0?0:duration;
 //        qint64 t;
 //        int size = m_slices.size();
 //        if (size == 0) {
@@ -200,7 +204,7 @@ class Session
         return t;
     }
 
-    //! \brief Flag this Session as dirty, so Machine object can save it
+    //! \brief Flag this Session as dirty, so device object can save it
     void SetChanged(bool val) {
         s_changed = val;
         s_events_loaded = val; // dirty hack putting this here
@@ -230,7 +234,7 @@ class Session
     QHash<ChannelID, EventDataType> m_min; // The actual minimum
     QHash<ChannelID, EventDataType> m_max;
 
-    // This could go in channels, but different machines interpret it differently
+    // This could go in channels, but different devices interpret it differently
     // Under the new SleepyLib data Device model this can be done, but unfortunately not here..
     QHash<ChannelID, EventDataType> m_physmin; // The physical minimum for graph display purposes
     QHash<ChannelID, EventDataType> m_physmax; // The physical maximum
@@ -362,7 +366,7 @@ class Session
     EventDataType calcMiddle(ChannelID code);
     EventDataType calcMax(ChannelID code);
     EventDataType calcPercentile(ChannelID code);
-    
+
     //! \brief Returns true if the channel has events loaded, or a record of a count for when they are not
     bool channelExists(ChannelID name);
 
@@ -394,7 +398,7 @@ class Session
                             EventDataType offset = 0.0, EventDataType min = 0.0, EventDataType max = 0.0,
                             EventDataType rate = 0.0, bool second_field = false);
 
-    //! \brief Returns this sessions MachineID
+    //! \brief Returns this sessions DeviceID
     Machine *machine() { return s_machine; }
 
     //! \brief Returns true if session only contains summary data
@@ -417,23 +421,14 @@ class Session
     //! \brief Completely purges Session from memory and disk.
     bool Destroy();
 
-    void wipeSummary() {
-        s_first = s_last = 0;
-        s_enabled = true;
-        m_cph.clear();
-        m_sum.clear();
-        m_cnt.clear();
-    }
-
-
     QString eventFile() const;
 
-    //! \brief Returns MachineType for this session
+    //! \brief Returns DeviceType for this session
     MachineType type() { return s_machtype; }
 
 
 
-#ifdef SESSION_DEBUG 
+#ifdef SESSION_DEBUG
     QStringList session_files;
 #endif
 
@@ -456,7 +451,7 @@ protected:
 
     bool s_summary_loaded;
     bool s_events_loaded;
-    bool s_enabled;
+    quint8 s_enabled;
 
     // for debugging
     bool destroyed;
