@@ -1,7 +1,7 @@
 /* ExportCSV module implementation
  *
- * Copyright (c) 2020 The OSCAR Team
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
@@ -71,7 +71,7 @@ ExportCSV::~ExportCSV()
 
 void ExportCSV::on_filenameBrowseButton_clicked()
 {
-    QString timestamp = tr("OSCAR_");
+    QString timestamp = QString("OSCAR_");
     timestamp += p_profile->Get("UserName") + "_";
 
     if (ui->rb1_details->isChecked()) { timestamp += tr("Details_"); }
@@ -168,10 +168,14 @@ void ExportCSV::on_exportButton_clicked()
 
 
     QList<ChannelID> countlist, avglist, p90list, maxlist;
-    countlist.append(CPAP_Hypopnea);
-    countlist.append(CPAP_Obstructive);
-    countlist.append(CPAP_Apnea);
-    countlist.append(CPAP_ClearAirway);
+    for (int i = 0; i < ahiChannels.size(); i++)
+        countlist.append(ahiChannels.at(i));
+
+//    countlist.append(CPAP_Hypopnea);
+//    countlist.append(CPAP_Obstructive);
+//    countlist.append(CPAP_Apnea);
+//    countlist.append(CPAP_ClearAirway);
+//    countlist.append(CPAP_AllApnea);
     countlist.append(CPAP_VSnore);
     countlist.append(CPAP_VSnore2);
     countlist.append(CPAP_RERA);
@@ -215,7 +219,7 @@ void ExportCSV::on_exportButton_clicked()
         }
 
         for (int i = 0; i < p90list.size(); i++) {
-            header += sep + tr("%1% ").arg(percent*100.0, 0, 'f', 0) + schema::channel[p90list[i]].label();
+            header += sep + QString("%1% ").arg(percent*100.0, 0, 'f', 0) + schema::channel[p90list[i]].label();
         }
 
         for (int i = 0; i < maxlist.size(); i++) {
@@ -248,14 +252,14 @@ void ExportCSV::on_exportButton_clicked()
                 data += sep + QString::number(day->size(), 10);
                 data += sep + start.toString(Qt::ISODate);
                 data += sep + end.toString(Qt::ISODate);
-                int time = day->total_time() / 1000L;
+                // Given this is a CPAP specific report, just report CPAP hours
+                int time = int(day->hours(MT_CPAP) * 3600L);
                 int h = time / 3600;
                 int m = int(time / 60) % 60;
                 int s = int(time) % 60;
-                data += sep + QString().sprintf("%02i:%02i:%02i", h, m, s);
-                float ahi = day->count(CPAP_Obstructive) + day->count(CPAP_Hypopnea) + day->count(
-                                CPAP_Apnea) + day->count(CPAP_ClearAirway);
-                ahi /= day->hours();
+                data += sep + QString::asprintf("%02i:%02i:%02i", h, m, s);
+
+                float ahi = day->calcAHI();
                 data += sep + QString::number(ahi, 'f', 3);
 
                 for (int i = 0; i < countlist.size(); i++) {
@@ -289,6 +293,7 @@ void ExportCSV::on_exportButton_clicked()
                     QDateTime start = QDateTime::fromTime_t(sess->first() / 1000L);
                     QDateTime end = QDateTime::fromTime_t(sess->last() / 1000L);
 
+                    sess->OpenEvents();
                     data = date.toString(Qt::ISODate);
                     data += sep + QString::number(sess->session(), 10);
                     data += sep + start.toString(Qt::ISODate);
@@ -297,10 +302,11 @@ void ExportCSV::on_exportButton_clicked()
                     int h = time / 3600;
                     int m = int(time / 60) % 60;
                     int s = int(time) % 60;
-                    data += sep + QString().sprintf("%02i:%02i:%02i", h, m, s);
+                    data += sep + QString::asprintf("%02i:%02i:%02i", h, m, s);
 
-                    float ahi = sess->count(CPAP_Obstructive) + sess->count(CPAP_Hypopnea) + sess->count(
-                                    CPAP_Apnea) + sess->count(CPAP_ClearAirway);
+                    float ahi = sess->count(AllAhiChannels);
+                                //sess->count(CPAP_AllApnea) + sess->count(CPAP_Obstructive) + sess->count(CPAP_Hypopnea)
+                                // + sess->count(CPAP_Apnea) + sess->count(CPAP_ClearAirway);
                     ahi /= sess->hours();
                     data += sep + QString::number(ahi, 'f', 3);
 
