@@ -1,7 +1,7 @@
 /* OSCAR Logger module implementation
  *
- * Copyright (c) 2020 The OSCAR Team
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
@@ -23,23 +23,26 @@ void MyOutputHandler(QtMsgType type, const QMessageLogContext &context, const QS
         return;
     }
 
-    QString msg, typestr;
+    QString msg, typestr, contextstr;
+#ifdef VERBOSE_LOGGING
+    contextstr = QString(context.file) + " " + context.function + ":" + QString::number(context.line) + " ";
+#endif
 
     switch (type) {
     case QtWarningMsg:
-        typestr = QString("Warning: ");
+        typestr = QString("Warning:  ") + contextstr;
         break;
 
     case QtFatalMsg:
-        typestr = QString("Fatal: ");
+        typestr = QString("Fatal:    ") + contextstr;
         break;
 
     case QtCriticalMsg:
-        typestr = QString("Critical: ");
+        typestr = QString("Critical: ") + contextstr;
         break;
 
     default:
-        typestr = QString("Debug: ");
+        typestr = QString("Debug:    ") + contextstr;
         break;
     }
 
@@ -70,7 +73,9 @@ void initializeLogger()
         s_LoggerRunning.lock();  // wait until the thread begins running
         s_LoggerRunning.unlock();  // we no longer need the lock
     }
+#ifndef HARDLOG
     qInstallMessageHandler(MyOutputHandler);  // NOTE: comment this line out when debugging a crash, otherwise the deferred output will mislead you.
+#endif
     if (b) {
         qDebug() << "Started logging thread";
     } else {
@@ -188,7 +193,11 @@ void LogThread::run()
         while (connected && m_logFile && !buffer.isEmpty()) {
             QString msg = buffer.takeFirst();
             if (m_logStream) {
-                *m_logStream << msg << endl;
+                #if QT_VERSION >= QT_VERSION_CHECK(5,14,0)
+                    *m_logStream << msg << Qt::endl;
+                #else
+                    *m_logStream << msg << endl;
+                #endif
             }
             emit outputLog(msg);
         }
