@@ -1,6 +1,7 @@
 /* OSCAR MainWindow Headers
  *
- * Copyright (C) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (C) 2011-2018 Mark Watkins
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
@@ -40,9 +41,9 @@ class MainWindow;
 
  Open Source CPAP Analysis Reporter (OSCAR) is a program derived from the SleepyHead program written by Mark Watkins.
 
- SleepyHead was a Cross-Platform Open-Source software for reviewing data from %CPAP machines, which are used in the treatment of Sleep Disorders.
+ SleepyHead was a Cross-Platform Open-Source software for reviewing data from %CPAP devices, which are used in the treatment of Sleep Disorders.
 
- SleepyHead was created by <a href="http://jedimark64.blogspot.com">Mark Watkins</a> (JediMark), an Australian software developer.
+ SleepyHead was created by Mark Watkins, an Australian software developer.
 
  This document is an attempt to provide a little technical insight into OSCAR's program internals.
 
@@ -53,13 +54,11 @@ class MainWindow;
 
  Data structures are described in a <a href="http://www.apneaboard.com/wiki/index.php?title=OSCAR_Data_Information">OSCAR Data Information Wiki</a>.
 
- The SleepyHead project was hosted on sourceforge, and it's original project page can be reached at <a href="http://sourceforge.net/projects/sleepyhead">http://sourceforge.net/projects/sleepyhead</a>.
- There was also a <a href="http://sourceforge.net/apps/mediawiki/sleepyhead/index.php?title=Main_Page">SleepyHead Wiki</a> containing further information.
 
  \section structure Program Structure
  OSCAR is written in C++ using Qt Toolkit library, and comprises of 3 main components
  \list
- \li The SleepLib Database, a specialized database for working with multiple sources of Sleep machine data.
+ \li The SleepLib Database, a specialized database for working with multiple sources of Sleep device data.
  \li A custom designed, high performance and interactive OpenGL Graphing Library.
  \li and the main Application user interface.
  \endlist
@@ -137,6 +136,8 @@ class MainWindow : public QMainWindow
     //! \brief Returns the Overview Tab object
     Overview *getOverview() { return overview; }
 
+    void updateOverview();
+
     /*! \fn void RestartApplication(bool force_login=false);
         \brief Closes down OSCAR and restarts it
         \param bool force_login
@@ -179,8 +180,6 @@ class MainWindow : public QMainWindow
     void doReprocessEvents();
     void doRecompressEvents();
 
-    void MachineUnsupported(Machine * m);
-
 
   protected:
     void closeEvent(QCloseEvent *) override;
@@ -268,6 +267,11 @@ class MainWindow : public QMainWindow
 
     //! \brief Destroy the CPAP data for the currently selected day, so it can be freshly imported again
     void on_actionPurge_Current_Day_triggered();
+    void on_actionPurgeCurrentDayOximetry_triggered();
+    void on_actionPurgeCurrentDaySleepStage_triggered();
+    void on_actionPurgeCurrentDayPosition_triggered();
+    void on_actionPurgeCurrentDayAllExceptNotes_triggered();
+    void on_actionPurgeCurrentDayAll_triggered();
 
     void on_action_Sidebar_Toggle_toggled(bool arg1);
 
@@ -312,12 +316,19 @@ class MainWindow : public QMainWindow
 
     void on_reportModeStandard_clicked();
 
+    void init_reportModeUi();
+    void reset_reportModeUi();
+
     void on_actionRebuildCPAP(QAction *action);
 
     void on_actionPurgeMachine(QAction *action);
 
     void on_reportModeRange_clicked();
 
+    void on_statEndDate_dateChanged(const QDate &date);
+
+    void on_statStartDate_dateChanged(const QDate &date);
+
     void on_actionPurgeCurrentDaysOximetry_triggered();
 
     void logMessage(QString msg);
@@ -334,6 +345,8 @@ class MainWindow : public QMainWindow
 
     void on_actionExport_Journal_triggered();
 
+    void on_actionImport_Journal_triggered();
+
     void on_actionShow_Performance_Counters_toggled(bool arg1);
 
     void on_actionExport_CSV_triggered();
@@ -354,8 +367,6 @@ class MainWindow : public QMainWindow
 
     void on_profilesButton_clicked();
 
-    void reloadProfile();
-
     void on_bookmarkView_anchorClicked(const QUrl &arg1);
 
     void on_recordsBox_anchorClicked(const QUrl &linkurl);
@@ -364,14 +375,18 @@ class MainWindow : public QMainWindow
 
     void on_actionShowPersonalData_toggled(bool visible);
 
+public slots:
+    void reloadProfile();
 
 private:
     QString getMainWindowTitle();
     void importCPAPBackups();
     QList<ImportPath> detectCPAPCards();
-    QList<ImportPath> selectCPAPDataCards(const QString & prompt);
+    QList<ImportPath> selectCPAPDataCards(const QString & prompt, bool alwaysPrompt = false);
     void importCPAPDataCards(const QList<ImportPath> & datacards);
     void addMachineToMenu(Machine* mach, QMenu* menu);
+    void purgeDay(MachineType type);
+    void importNonCPAP(MachineLoader &loader);
 
 //    QString getWelcomeHTML();
     void FreeSessions();
@@ -392,11 +407,12 @@ private:
 //    gGraphView *SnapshotGraph;
     QString bookmarkFilter;
     bool m_restartRequired;
+    bool m_clinicalMode = false;
     volatile bool m_inRecalculation;
 
     void PopulatePurgeMenu();
 
-    //! \brief Destroy ALL the CPAP data for the selected machine
+    //! \brief Destroy ALL the CPAP data for the selected device
     void purgeMachine(Machine *);
 
     int warnidx;
