@@ -1,12 +1,15 @@
 /* Create New Profile Implementation
  *
- * Copyright (c) 2019-2020 The OSCAR Team
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
  * for more details. */
 
+#define TEST_MACROS_ENABLEDoff
+#include <test_macros.h>
+
 #include <QMessageBox>
 #include <QFile>
 #include <QTextStream>
@@ -30,9 +33,11 @@ NewProfile::NewProfile(QWidget *parent, const QString *user) :
     ui(new Ui::NewProfile)
 {
     ui->setupUi(this);
-    if (user)
+    if (user) {
+      originalProfileName=*user;
       ui->userNameEdit->setText(*user);
 //    ui->userNameEdit->setText(getUserName());
+    }
     QLocale locale = QLocale::system();
     QString shortformat = locale.dateFormat(QLocale::ShortFormat);
 
@@ -110,7 +115,7 @@ QString NewProfile::getIntroHTML()
            "<body>"
            "<div align=center><h1>" + tr("Welcome to the Open Source CPAP Analysis Reporter") + "</h1></div>"
 
-           "<p>" + tr("This software is being designed to assist you in reviewing the data produced by your CPAP machines and related equipment.")
+           "<p>" + tr("This software is being designed to assist you in reviewing the data produced by your CPAP Devices and related equipment.")
            + "</p>"
 
            "<p>" + tr("OSCAR has been released freely under the <a href='qrc:/COPYING'>GNU Public License v3</a>, and comes with no warranty, and without ANY claims to fitness for any purpose.")
@@ -131,12 +136,23 @@ QString NewProfile::getIntroHTML()
            "<p><b><font size=+1>" + tr("Use of this software is entirely at your own risk.") +
            "</font></b></p>"
 
-           "<p><i>" + tr("OSCAR is copyright &copy;2011-2018 Mark Watkins and portions &copy;2019-2020 The OSCAR Team") + "<i></p>"
+           "<p><i>" + tr("OSCAR is copyright &copy;2011-2018 Mark Watkins and portions &copy;2019-2024 The OSCAR Team") + "<i></p>"
            "</div>"
            "</body>"
            "</html>";
 }
 
+int cmToFeetInch( double cm, double& inches ) {
+    inches = cm * inches_per_cm;
+    int feet = inches / 12;
+    inches -=   (double)(feet *12);
+    return feet;
+}
+
+double feetInchToCm( int feet , double inches ) {
+    return cms_per_inch*(inches + (double)(feet *12));
+}
+
 void NewProfile::on_nextButton_clicked()
 {
     const QString xmlext = ".xml";
@@ -192,14 +208,23 @@ void NewProfile::on_nextButton_clicked()
         ui->stackedWidget->setCurrentIndex(index);
     } else {
         // Finish button clicked.
-        QString username = ui->userNameEdit->text().trimmed();
+        newProfileName = ui->userNameEdit->text().simplified();
+        QString profileName;
+        if (originalProfileName.isEmpty() ) {
+            profileName = newProfileName;
+        } else {
+            profileName = originalProfileName;
+            ui->userNameEdit->setText(originalProfileName);
+            //QString profileName = originalProfileName.isEmpty()? newProfileName : originalProfileName;
+        }
+        //QString profileName = originalProfileName.isEmpty()? newProfileName : originalProfileName;
 
         if (QMessageBox::question(this, tr("Profile Changes"), tr("Accept and save this information?"),
                                   QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes) {
-            Profile *profile = Profiles::Get(username);
 
+            Profile *profile = Profiles::Get(profileName);
             if (!profile) { // No profile, create one.
-                profile = Profiles::Create(username);
+                profile = Profiles::Create(profileName);
             }
 
             Profile &prof = *profile;
@@ -237,36 +262,51 @@ void NewProfile::on_nextButton_clicked()
                                            ui->timezoneCombo->currentIndex()).toString());
             profile->user->setCountry(ui->countryCombo->currentText());
             profile->user->setDaylightSaving(ui->DSTcheckbox->isChecked());
-            UnitSystem us;
-
-            if (ui->heightCombo->currentIndex() == 0) { us = US_Metric; }
-            else if (ui->heightCombo->currentIndex() == 1) { us = US_English; }
-            else { us = US_Metric; }
 
+            UnitSystem us = US_Metric;
+            if (ui->heightCombo->currentIndex() == 1) { us = US_English; };
             if (profile->general->unitSystem() != us) {
                 profile->general->setUnitSystem(us);
-
                 if (mainwin && mainwin->getDaily()) { mainwin->getDaily()->UnitsChanged(); }
             }
 
-            double v = 0;
-
-            if (us == US_English) {
-                // convert to metric
-                v = (ui->heightEdit->value() * 30.48);
-                v += ui->heightEdit2->value() * 2.54;
-            } else {
-                v = ui->heightEdit->value();
+            if (m_height_modified) {
+                profile->user->setHeight(m_tmp_height_cm);
+                // also call unitsChanged if height also changed. Need for update BMI.
+                if (mainwin && mainwin->getDaily()) { mainwin->getDaily()->UnitsChanged(); }
             }
-
-            profile->user->setHeight(v);
-
-            //profile->user->setUserName(username);
-            AppSetting->setProfileName(username);
+            AppSetting->setProfileName(profileName);
 
             profile->Save();
-
-            this->accept();
+            if ( !originalProfileName.isEmpty() && !newProfileName.isEmpty() && (originalProfileName != newProfileName)) {
+                QString originalProfileFullName = p_pref->Get("{home}/Profiles/") + originalProfileName;
+                QString newProfileFullName = p_pref->Get("{home}/Profiles/") + newProfileName;
+                QFile file(originalProfileFullName);
+                if (file.exists()) {
+                    bool status = file.rename(newProfileFullName);
+                    if (status) {  // successful rename
+                        Profiles::profiles[newProfileName] = p_profile;
+                        AppSetting->setProfileName(newProfileName);
+                        if (mainwin) mainwin->CloseProfile();
+                        QCoreApplication::processEvents();
+                        mainwin->RestartApplication(true,"-l");
+                        QCoreApplication::processEvents();
+                        exit(0);
+                    } else {
+                        QMessageBox::question(this, tr("Duplicate or Invalid User Name"), tr("Please Change User Name "), QMessageBox::Ok);
+                        index=1;
+                        ui->stackedWidget->setCurrentIndex(index);
+                        ui->userNameEdit->setText(newProfileName);
+                    }
+                } else {
+                    qWarning() << "Rename Profile failed";
+                }
+            } else {
+                if (mainwin) {
+                    mainwin->GenerateStatistics();
+                }
+                this->accept();
+            }
         }
     }
 
@@ -319,6 +359,7 @@ void NewProfile::skipWelcomeScreen()
     ui->backButton->setEnabled(false);
     ui->nextButton->setEnabled(true);
 }
+
 void NewProfile::edit(const QString name)
 {
     skipWelcomeScreen();
@@ -329,7 +370,7 @@ void NewProfile::edit(const QString name)
     }
 
     ui->userNameEdit->setText(name);
-    ui->userNameEdit->setReadOnly(true);
+    // ui->userNameEdit->setReadOnly(true);
     ui->firstNameEdit->setText(profile->user->firstName());
     ui->lastNameEdit->setText(profile->user->lastName());
 
@@ -363,6 +404,8 @@ void NewProfile::edit(const QString name)
     ui->untreatedAHIEdit->setValue(profile->cpap->untreatedAHI());
     ui->cpapModeCombo->setCurrentIndex((int)profile->cpap->mode());
 
+    on_cpapModeCombo_activated(profile->cpap->mode());
+
     ui->doctorNameEdit->setText(profile->doctor->name());
     ui->doctorPracticeEdit->setText(profile->doctor->practiceName());
     ui->doctorPhoneEdit->setText(profile->doctor->phone());
@@ -383,25 +426,9 @@ void NewProfile::edit(const QString name)
 
     ui->heightCombo->setCurrentIndex(i);
 
-    double v = profile->user->height();
-
-    if (us == US_English)  { // evil non-metric
-        int ti = v / 2.54;
-        int feet = ti / 12;
-        int inches = ti % 12;
-        ui->heightEdit->setValue(feet);
-        ui->heightEdit2->setValue(inches);
-        ui->heightEdit2->setVisible(true);
-        ui->heightEdit->setDecimals(0);
-        ui->heightEdit2->setDecimals(0);
-        ui->heightEdit->setSuffix(STR_UNIT_FOOT); // foot
-        ui->heightEdit2->setSuffix(STR_UNIT_INCH); // inches
-    } else { // good wholesome metric
-        ui->heightEdit->setValue(v);
-        ui->heightEdit2->setVisible(false);
-        ui->heightEdit->setDecimals(0);
-        ui->heightEdit->setSuffix(STR_UNIT_CM);
-    }
+    m_tmp_height_cm = profile->user->height();
+    m_height_modified = false;
+    on_heightCombo_currentIndexChanged(i);
 }
 
 void NewProfile::on_passwordEdit1_editingFinished()
@@ -416,26 +443,40 @@ void NewProfile::on_passwordEdit2_editingFinished()
 
 void NewProfile::on_heightCombo_currentIndexChanged(int index)
 {
+    ui->heightEdit->blockSignals(true);
+    ui->heightEdit2->blockSignals(true);
     if (index == 0) {
         //metric
-        ui->heightEdit2->setVisible(false);
-        ui->heightEdit->setDecimals(0);
+        ui->heightEdit->setDecimals(1);
         ui->heightEdit->setSuffix(STR_UNIT_CM);
-        double v = ui->heightEdit->value() * 30.48;
-        v += ui->heightEdit2->value() * 2.54;
-        ui->heightEdit->setValue(v);
+        ui->heightEdit->setValue(m_tmp_height_cm);
+        ui->heightEdit2->setVisible(false);
     } else {        //evil
-        ui->heightEdit->setDecimals(0);
-        ui->heightEdit2->setDecimals(0);
+        ui->heightEdit->setDecimals(0);  // feet are always a whole number.
+        ui->heightEdit2->setDecimals(1);  // inches can be seen as double.
         ui->heightEdit->setSuffix(STR_UNIT_FOOT);
         ui->heightEdit2->setVisible(true);
         ui->heightEdit2->setSuffix(STR_UNIT_INCH);
-        int v = ui->heightEdit->value() / 2.54;
-        int feet = v / 12;
-        int inches = v % 12;
-        ui->heightEdit->setValue(feet);
+        double inches=0;
+        ui->heightEdit->setValue(cmToFeetInch(m_tmp_height_cm,inches));
         ui->heightEdit2->setValue(inches);
     }
+    ui->heightEdit->blockSignals(false);
+    ui->heightEdit2->blockSignals(false);
+}
+
+void NewProfile::on_heightEdit_valueChanged(double ) {
+    m_height_modified = true;
+    double cm = ui->heightEdit->value();
+    if (ui->heightCombo->currentIndex() == 1) {
+        //US_English;
+        cm = feetInchToCm (cm,ui->heightEdit2->value());
+    };
+    m_tmp_height_cm = cm;
+}
+
+void NewProfile::on_heightEdit2_valueChanged(double value) {
+    on_heightEdit_valueChanged(value);
 }
 
 void NewProfile::on_textBrowser_anchorClicked(const QUrl &arg1)
