@@ -1,16 +1,21 @@
 /* Oximeter Import Wizard Implementation
  *
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
  * for more details. */
 
+#define TEST_MACROS_ENABLEDoff
+#include <test_macros.h>
+
 #include <QThread>
 #include <QMessageBox>
 #include <QFileDialog>
 #include <QStandardPaths>
 #include <QDesktopServices>
+#include <QElapsedTimer>
 
 #include "Graphs/gYAxis.h"
 #include "Graphs/gXAxis.h"
@@ -189,7 +194,7 @@ SerialOximeter * OximeterImport::detectOximeter()
 
     ui->progressBar->setMaximum(PORTSCAN_TIMEOUT);
 
-    QTime time;
+    QElapsedTimer time;
     time.start();
 
     oximodule = nullptr;
@@ -295,7 +300,7 @@ void OximeterImport::on_directImportButton_clicked()
        //   item->setData(Qt::UserRole+2, duration);
             item->setFlags(item->flags() & ~Qt::ItemIsEditable);
 
-            item = new QTableWidgetItem(QString(). sprintf("%02i:%02i:%02i", h,m,s));
+            item = new QTableWidgetItem( QString::asprintf("%02i:%02i:%02i", h,m,s));
             ui->tableOxiSessions->setItem(i, 1, item);
             item->setFlags(item->flags() & ~Qt::ItemIsEditable);
 
@@ -407,7 +412,7 @@ void OximeterImport::on_fileImportButton_clicked()
     if (documentsFolder.isEmpty())
         documentsFolder = QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
 
-	qDebug() << "oximod - File Import button clicked";
+    qDebug() << "oximod - File Import button clicked";
 	
     QString filename = QFileDialog::getOpenFileName(nullptr , tr("Select a valid oximetry data file"), documentsFolder, tr("Oximetry Files (*.spo *.spor *.spo2 *.SpO2 *.dat)"));
 
@@ -417,14 +422,14 @@ void OximeterImport::on_fileImportButton_clicked()
     // Make sure filename dialog had time to close properly..
     QApplication::processEvents();
 
-	qDebug() << "oximod - Chosen filename is " << filename;
+    qDebug() << "oximod - Chosen filename is " << filename;
     QList<SerialOximeter *> loaders = GetOxiLoaders();
 
     bool success = false;
 
     oximodule = nullptr;
     Q_FOREACH(SerialOximeter * loader, loaders) {
-        if (loader->Open(filename)) {
+        if (loader && loader->Open(filename)) { // there is a null entry in loaders. so ths check is necessary
             success = true;
             oximodule = loader;
             break;
@@ -595,7 +600,7 @@ void OximeterImport::on_calendarWidget_clicked(const QDate &date)
         sessbar->update();
     } else if (ui->radioSyncOximeter) {
         qDebug() << "oximod - Using oximeter date and time";
-        ui->sessbarLabel->setText(tr("%1").arg(date.toString(Qt::SystemLocaleLongDate)));
+        ui->sessbarLabel->setText(QString("%1").arg(date.toString(Qt::SystemLocaleLongDate)));
         ui->dateTimeEdit->setDateTime(QDateTime(date, ui->dateTimeEdit->dateTime().time()));
     }
 }
@@ -739,17 +744,17 @@ void OximeterImport::updateLiveDisplay()
         pulse = (*(oximodule->oxirec))[size].pulse;
         spo2 = (*(oximodule->oxirec))[size].spo2;
         if (pulse > 0) {
-            ui->pulseDisplay->display(QString().sprintf("%3i", pulse));
+            ui->pulseDisplay->display(QString::asprintf("%3i", pulse));
         } else {
             ui->pulseDisplay->display("---");
         }
         if (spo2 > 0) {
-            ui->spo2Display->display(QString().sprintf("%2i", spo2));
+            ui->spo2Display->display(QString::asprintf("%2i", spo2));
         } else {
             ui->spo2Display->display("--");
         }
 
-        ui->lcdDuration->display(QString().sprintf("%02i:%02i:%02i",hours, minutes, seconds));
+        ui->lcdDuration->display(QString::asprintf("%02i:%02i:%02i",hours, minutes, seconds));
 
     }
 }
@@ -1004,7 +1009,7 @@ void OximeterImport::on_saveButton_clicked()
     calcSPO2Drop(session);
     calcPulseChange(session);
 
-    qDebug() << "oximod - Setting up machine and session";
+    qDebug() << "oximod - Setting up device and session";
 
     mach->setModel(oximodule->getModel());
     mach->setBrand(oximodule->getVendor());
@@ -1039,9 +1044,9 @@ void OximeterImport::on_saveButton_clicked()
 
     session->setOpened(true);
 
-    qDebug() << "oximod - Adding session to machine";
+    qDebug() << "oximod - Adding session to device";
     mach->AddSession(session);
-    qDebug() << "oximod - Saving machine";
+    qDebug() << "oximod - Saving device";
     mach->Save();
     mach->SaveSummaryCache();
     p_profile->StoreMachines();
@@ -1088,7 +1093,7 @@ void OximeterImport::chooseSession()
         int  m = (duration / 60) % 60;
         int  s = duration % 60;
  
-        item = new QTableWidgetItem(QString(). sprintf("%02i:%02i:%02i", h,m,s));
+        item = new QTableWidgetItem( QString::asprintf("%02i:%02i:%02i", h,m,s));
         ui->tableOxiSessions->setItem(row, 1, item);
         item->setFlags(item->flags() & ~Qt::ItemIsEditable);
 
@@ -1151,7 +1156,7 @@ void OximeterImport::setInformation()
             +tr("If you are trying to sync oximetry and CPAP data, please make sure you imported your CPAP sessions first before proceeding!")+"</span></p>"
     "<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">"
             "<span style=\" font-weight:600;\">"+tr("Important Notes:")+" </span>"
-            +tr("For OSCAR to be able to locate and read directly from your Oximeter device, you need to ensure the correct device drivers (eg. USB to Serial UART) have been installed on your computer. For more information about this, %1click here%2.").arg("<a href=\"http://www.silabs.com/products/mcu/pages/usbtouartbridgevcpdrivers.aspx\"><span style=\" text-decoration: underline; color:#0000ff;\">").arg("</span></a>")+"</p>"
+            +tr("For OSCAR to be able to locate and read directly from your Oximeter device, you need to ensure the correct device drivers (eg. USB to Serial UART) have been installed on your computer. For more information about this, %1click here%2.").arg("<a href=\"https://www.silabs.com/developers/usb-to-uart-bridge-vcp-drivers\"><span style=\" text-decoration: underline; color:#0000ff;\">").arg("</span></a>")+"</p>"
     "<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">"
             +tr("Contec CMS50D+ devices do not have an internal clock, and do not record a starting time. If you do not have a CPAP session to link a recording to, you will have to enter the start time manually after the import process is completed.")+"</p>"
     "<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">"
