@@ -1,12 +1,15 @@
 /* OSCAR Preferences Dialog Implementation
  *
- * Copyright (c) 2019 The OSCAR Team
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
  * for more details. */
 
+#define TEST_MACROS_ENABLEDoff
+#include <test_macros.h>
+
 #include <QLabel>
 #include <QColorDialog>
 #include <QMessageBox>
@@ -28,6 +31,8 @@
 #include <mainwindow.h>
 #include "ui_preferencesdialog.h"
 #include "SleepLib/machine_common.h"
+#include "highresolution.h"
+#include "daily.h"
 
 extern QFont *defaultfont;
 extern QFont *mediumfont;
@@ -39,6 +44,23 @@ typedef QMessageBox::StandardButtons StandardButtons;
 
 QHash<schema::ChanType, QString> channeltype;
 
+
+QString PreferencesDialog::clinicalHelp() {
+    QStringList str; str
+    <<tr("Clinical Mode:")
+    <<tr("Reports what is on the data card, all of it including any and all data deselected in the Permissive mode.")
+    <<tr("Basically replicates the reports and data stored on the devices data card.")
+    <<tr("This includes pap devices, oximeters, etc. Compliance reports fall under this mode.")
+    <<tr("Compliance reports always include all data within the chosen Compliance period, even if otherwise deselected.")
+    <<""
+    <<tr("Permissive Mode:")
+    <<tr("Allows user to select which data sets/ sessions to be used for calculations and display.")
+    <<tr("Additional charts and calculations may be available that are not available from the vendor data.")
+    <<tr("Enables Custom UserFlags displayed in the statistics Therapy Efficacy section")
+    <<"";
+    return str.join("\n");
+}
+
 PreferencesDialog::PreferencesDialog(QWidget *parent, Profile *_profile) :
     QDialog(parent, Qt::WindowTitleHint | Qt::WindowCloseButtonHint),
     ui(new Ui::PreferencesDialog),
@@ -63,15 +85,15 @@ PreferencesDialog::PreferencesDialog(QWidget *parent, Profile *_profile) :
             }
         }
     } else {
-        if (QMessageBox::question(this, tr("No CPAP machines detected"),
-                tr("Will you be using a ResMed brand machine?"),
+        if (QMessageBox::question(this, tr("No CPAP devices detected"),
+                tr("Will you be using a ResMed brand device?"),
                 QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes )
             haveResMed = true;
     }
 
 #ifdef LOCK_RESMED_SESSIONS
     // Remove access to session splitting options and show ResMed users a notice instead
-    ui->ResMedWarning->setText(tr("<p><b>Please Note:</b> OSCAR's advanced session splitting capabilities are not possible with <b>ResMed</b> machines due to a limitation in the way their settings and summary data is stored, and therefore they have been disabled for this profile.</p><p>On ResMed machines, days will <b>split at noon</b> like in ResMed's commercial software.</p>"));
+    ui->ResMedWarning->setText(tr("<p><b>Please Note:</b> OSCAR's advanced session splitting capabilities are not possible with <b>ResMed</b> devices due to a limitation in the way their settings and summary data is stored, and therefore they have been disabled for this profile.</p><p>On ResMed devices, days will <b>split at noon</b> like in ResMed's commercial software.</p>"));
     ui->ResMedWarning->setVisible(haveResMed);
 
     if (haveResMed) {
@@ -130,14 +152,15 @@ PreferencesDialog::PreferencesDialog(QWidget *parent, Profile *_profile) :
     ui->showLeakRedline->setChecked(profile->cpap->showLeakRedline());
     ui->leakRedlineSpinbox->setValue(profile->cpap->leakRedline());
 
-    ui->oxiDesaturationThreshold->setValue(schema::channel[OXI_SPO2].lowerThreshold());
-    ui->flagPulseAbove->setValue(schema::channel[OXI_Pulse].upperThreshold());
-    ui->flagPulseBelow->setValue(schema::channel[OXI_Pulse].lowerThreshold());
+    //change initialization from hard coded in schema to profile data.
+    ui->oxiDesaturationThreshold->setValue(profile->oxi->oxiDesaturationThreshold());
+    ui->flagPulseAbove->setValue(profile->oxi->flagPulseAbove());
+    ui->flagPulseBelow->setValue(profile->oxi->flagPulseBelow());
 
     ui->spo2Drop->setValue(profile->oxi->spO2DropPercentage());
-    ui->spo2DropTime->setValue(profile->oxi->spO2DropDuration());
+    ui->spo2DropDuration->setValue(profile->oxi->spO2DropDuration());
     ui->pulseChange->setValue(profile->oxi->pulseChangeBPM());
-    ui->pulseChangeTime->setValue(profile->oxi->pulseChangeDuration());
+    ui->pulseChangeDuration->setValue(profile->oxi->pulseChangeDuration());
     ui->oxiDiscardThreshold->setValue(profile->oxi->oxiDiscardThreshold());
 
     ui->eventIndexCombo->setCurrentIndex(profile->general->calculateRDI() ? 1 : 0);
@@ -218,6 +241,15 @@ PreferencesDialog::PreferencesDialog(QWidget *parent, Profile *_profile) :
     ui->allowYAxisScaling->setChecked(AppSetting->allowYAxisScaling());
     ui->includeSerial->setChecked(AppSetting->includeSerial());
     ui->monochromePrinting->setChecked(AppSetting->monochromePrinting());
+    ui->eventFlagSessionBar->setChecked(profile->appearance->eventFlagSessionBar());
+    ui->complianceHours->setValue(profile->cpap->complianceHours());
+    ui->clinicalMode->setChecked(profile->cpap->clinicalMode());
+    ui->clinicalTextEdit->setPlainText(clinicalHelp());
+    // clinicalMode and permissiveMode are radio buttons and must be set to opposite values. Once clinicalMode is used.
+    // Radio Buttons illustrate the operating mode.
+    ui->permissiveMode->setChecked(!profile->cpap->clinicalMode());
+    HighResolution::checkBox(false,ui->highResolution);
+    ui->alternatingColorsCombo->setCurrentIndex(AppSetting->alternatingColorsCombo());
 
     ui->autoLaunchImporter->setChecked(AppSetting->autoLaunchImport());
 #ifndef NO_CHECKUPDATES
@@ -260,8 +292,6 @@ PreferencesDialog::PreferencesDialog(QWidget *parent, Profile *_profile) :
     ui->cacheSessionData->setChecked(AppSetting->cacheSessions());
     ui->preloadSummaries->setChecked(profile->session->preloadSummaries());
     ui->animationsAndTransitionsCheckbox->setChecked(AppSetting->animations());
-    ui->complianceCheckBox->setChecked(profile->cpap->showComplianceInfo());
-    ui->complianceHours->setValue(profile->cpap->complianceHours());
 
     ui->prefCalcMiddle->setCurrentIndex(profile->general->prefCalcMiddle());
     ui->prefCalcMax->setCurrentIndex(profile->general->prefCalcMax());
@@ -290,13 +320,18 @@ PreferencesDialog::PreferencesDialog(QWidget *parent, Profile *_profile) :
     ui->updateCheckEvery->setValue(AppSetting->updateCheckFrequency());
     if (AppSetting->updatesLastChecked().isValid()) {
         RefreshLastChecked();
-    } else { 
+    } else {
         ui->updateLastChecked->setText(tr("Never"));
     }
 #endif
 
     ui->overlayFlagsCombo->setCurrentIndex(AppSetting->overlayType());
+    #ifndef REMOVE_FITNESS
     ui->overviewLinecharts->setCurrentIndex(AppSetting->overviewLinechartMode());
+    #else
+    ui->overviewLinecharts->hide();
+    ui->overviewLinechartsLabel->hide();
+    #endif
 
     ui->ahiGraphWindowSize->setEnabled(false);
     ui->ahiGraphWindowSize->setValue(profile->cpap->AHIWindow());
@@ -318,8 +353,8 @@ PreferencesDialog::PreferencesDialog(QWidget *parent, Profile *_profile) :
     ui->maskLeaks4Slider->setValue(profile->cpap->custom4cmH2OLeaks()*10.0);
     ui->maskLeaks20Slider->setValue(profile->cpap->custom20cmH2OLeaks()*10.0);
 
-    ui->maskLeaks4Label->setText(tr("%1 %2").arg(profile->cpap->custom4cmH2OLeaks(), 5, 'f', 1).arg(STR_UNIT_LPM));
-    ui->maskLeaks20Label->setText(tr("%1 %2").arg(profile->cpap->custom20cmH2OLeaks(), 5, 'f', 1).arg(STR_UNIT_LPM));
+    ui->maskLeaks4Label->setText(QString("%1 %2").arg(profile->cpap->custom4cmH2OLeaks(), 5, 'f', 1).arg(STR_UNIT_LPM));
+    ui->maskLeaks20Label->setText(QString("%1 %2").arg(profile->cpap->custom20cmH2OLeaks(), 5, 'f', 1).arg(STR_UNIT_LPM));
 
     /*    QLocale locale=QLocale::system();
         QString shortformat=locale.dateFormat(QLocale::ShortFormat);
@@ -814,11 +849,6 @@ bool PreferencesDialog::Save()
         }
     }
 
-    schema::channel[OXI_SPO2].setLowerThreshold(ui->oxiDesaturationThreshold->value());
-    schema::channel[OXI_Pulse].setLowerThreshold(ui->flagPulseBelow->value());
-    schema::channel[OXI_Pulse].setUpperThreshold(ui->flagPulseAbove->value());
-
-
     AppSetting->setUserEventPieChart(ui->showUserFlagsInPie->isChecked());
     profile->session->setLockSummarySessions(ui->LockSummarySessionSplitting->isChecked());
     profile->session->setWarnOnUntestedMachine(ui->warnOnUntestedMachine->isChecked());
@@ -830,6 +860,18 @@ bool PreferencesDialog::Save()
     AppSetting->setAllowYAxisScaling(ui->allowYAxisScaling->isChecked());
     AppSetting->setIncludeSerial(ui->includeSerial->isChecked());
     AppSetting->setMonochromePrinting(ui->monochromePrinting->isChecked());
+    p_profile->appearance->setEventFlagSessionBar(ui->eventFlagSessionBar->isChecked());
+
+    bool clicicalModeChanged = profile->cpap->clinicalMode() != ui->clinicalMode->isChecked() ;
+    p_profile->cpap->setClinicalMode(ui->clinicalMode->isChecked());
+
+    needs_restart |= HighResolution::checkBox(true,ui->highResolution);
+    
+    if (ui->alternatingColorsCombo->currentIndex() != AppSetting->alternatingColorsCombo()) {
+        AppSetting->setAlternatingColorsCombo(ui->alternatingColorsCombo->currentIndex());
+        mainwin->GenerateStatistics();
+    }
+
     AppSetting->setGraphTooltips(ui->graphTooltips->isChecked());
 
     AppSetting->setAntiAliasing(ui->useAntiAliasing->isChecked());
@@ -855,7 +897,6 @@ bool PreferencesDialog::Save()
     profile->cpap->setShowLeakRedline(ui->showLeakRedline->isChecked());
     profile->cpap->setLeakRedline(ui->leakRedlineSpinbox->value());
 
-    profile->cpap->setShowComplianceInfo(ui->complianceCheckBox->isChecked());
     profile->cpap->setComplianceHours(ui->complianceHours->value());
 
     if (ui->graphHeight->value() != AppSetting->graphHeight()) {
@@ -883,14 +924,20 @@ bool PreferencesDialog::Save()
     profile->cpap->setClockDrift(s);
 
     AppSetting->setOverlayType((OverlayDisplayType)ui->overlayFlagsCombo->currentIndex());
+    #ifndef REMOVE_FITNESS
     AppSetting->setOverviewLinechartMode((OverviewLinechartModes)ui->overviewLinecharts->currentIndex());
+    #endif
 
     profile->oxi->setSpO2DropPercentage(ui->spo2Drop->value());
-    profile->oxi->setSpO2DropDuration(ui->spo2DropTime->value());
+    profile->oxi->setSpO2DropDuration(ui->spo2DropDuration->value());
     profile->oxi->setPulseChangeBPM(ui->pulseChange->value());
-    profile->oxi->setPulseChangeDuration(ui->pulseChangeTime->value());
+    profile->oxi->setPulseChangeDuration(ui->pulseChangeDuration->value());
     profile->oxi->setOxiDiscardThreshold(ui->oxiDiscardThreshold->value());
 
+    profile->oxi->setOxiDesaturationThreshold(ui->oxiDesaturationThreshold->value());
+    profile->oxi->setFlagPulseAbove(ui->flagPulseAbove->value());
+    profile->oxi->setFlagPulseBelow(ui->flagPulseBelow->value());
+
     profile->cpap->setAHIWindow(ui->ahiGraphWindowSize->value());
     profile->cpap->setAHIReset(ui->ahiGraphZeroReset->isChecked());
 
@@ -974,7 +1021,14 @@ bool PreferencesDialog::Save()
 
     p_pref->Save();
     profile->Save();
+    profile->resetOxiChannelPref();
 
+    if (clicicalModeChanged) {
+        // this fails - causing duplicate reload.
+        //QTimer::singleShot(0, mainwin, SLOT(reloadProfile()));
+        // while this one works.
+        mainwin->reloadProfile();
+    } else
     if (recompress_events) {
         mainwin->recompressEvents();
     } else if (recalc_events) {
@@ -983,7 +1037,7 @@ bool PreferencesDialog::Save()
     } else if (needs_reload) {
         QTimer::singleShot(0, mainwin, SLOT(reloadProfile()));
     } else if (needs_restart) {
-        mainwin->RestartApplication();
+        mainwin->RestartApplication(true,"-l");
     } else {
         mainwin->getDaily()->LoadDate(mainwin->getDaily()->getDate());
         // Save early.. just in case..
@@ -1162,7 +1216,7 @@ void PreferencesDialog::on_createSDBackups_toggled(bool checked)
 
         if (haveS9 && QMessageBox::question(this,
                 tr("This may not be a good idea"),
-                tr("ResMed S9 machines routinely delete certain data from your SD card older than 7 and 30 days (depending on resolution).") +
+                tr("ResMed S9 devices routinely delete certain data from your SD card older than 7 and 30 days (depending on resolution).") +
                 tr(" If you ever need to reimport this data again (whether in OSCAR or ResScan) this data won't come back.") +
                 tr(" If you need to conserve disk space, please remember to carry out manual backups.") +
                 tr(" Are you sure you want to disable these backups?"),
@@ -1205,6 +1259,42 @@ void PreferencesDialog::on_resetChannelDefaults_clicked()
     }
 }
 
+void PreferencesDialog::on_resetOxiMetryDefaults_clicked()
+{
+
+    if (QMessageBox::question(this, STR_MessageBox_Warning, QObject::tr("Are you sure you want to reset all your oximetry settings to defaults?"), QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::Yes) {
+
+            // reset ui with defaul values
+            ui->spo2Drop->setValue(           profile->oxi->defaultValue_OS_SPO2DropPercentage);
+            ui->spo2DropDuration->setValue(   profile->oxi->defaultValue_OS_SPO2DropDuration);
+            ui->pulseChange->setValue(        profile->oxi->defaultValue_OS_PulseChangeBPM);
+            ui->pulseChangeDuration->setValue(profile->oxi->defaultValue_OS_PulseChangeDuration);
+
+            ui->oxiDiscardThreshold->setValue(profile->oxi->defaultValue_OS_OxiDiscardThreshold);
+            ui->oxiDesaturationThreshold->setValue(  profile->oxi->defaultValue_OS_oxiDesaturationThreshold);
+            ui->flagPulseAbove->setValue(     profile->oxi->defaultValue_OS_flagPulseAbove );
+            ui->flagPulseBelow->setValue(     profile->oxi->defaultValue_OS_flagPulseBelow );
+
+            if (Save() ) {
+                // comment accept out to return to the preference tab
+                // other wise the preference tab will close and return
+                accept();
+            }
+        } else {
+            // restore values changed
+            ui->spo2Drop->setValue(profile->oxi->spO2DropPercentage());
+            ui->spo2DropDuration->setValue(profile->oxi->spO2DropDuration());
+            ui->pulseChange->setValue(profile->oxi->pulseChangeBPM());
+            ui->pulseChangeDuration->setValue(profile->oxi->pulseChangeDuration());
+
+            ui->oxiDiscardThreshold->setValue(profile->oxi->oxiDiscardThreshold());
+            ui->oxiDesaturationThreshold->setValue(profile->oxi->defaultValue_OS_oxiDesaturationThreshold);
+            ui->flagPulseAbove->setValue( profile->oxi->defaultValue_OS_flagPulseAbove );
+            ui->flagPulseBelow->setValue(  profile->oxi->defaultValue_OS_flagPulseBelow );
+        }
+
+}
+
 void PreferencesDialog::on_createSDBackups_clicked(bool checked)
 {
     if (!checked && p_profile->session->backupCardData()) {
@@ -1281,12 +1371,12 @@ void PreferencesDialog::on_waveView_doubleClicked(const QModelIndex &index)
 
 void PreferencesDialog::on_maskLeaks4Slider_valueChanged(int value)
 {
-    ui->maskLeaks4Label->setText(tr("%1 %2").arg(value/10.0f, 5,'f',1).arg(STR_UNIT_LPM));
+    ui->maskLeaks4Label->setText(QString("%1 %2").arg(value/10.0f, 5,'f',1).arg(STR_UNIT_LPM));
 }
 
 void PreferencesDialog::on_maskLeaks20Slider_valueChanged(int value)
 {
-    ui->maskLeaks20Label->setText(tr("%1 %2").arg(value/10.0f, 5,'f',1).arg(STR_UNIT_LPM));
+    ui->maskLeaks20Label->setText(QString("%1 %2").arg(value/10.0f, 5,'f',1).arg(STR_UNIT_LPM));
 }
 
 void PreferencesDialog::on_calculateUnintentionalLeaks_toggled(bool)
