@@ -1,12 +1,15 @@
 /* Profile Selector Implementation
  *
- * Copyright (c) 2019 The OSCAR Team
- * Copyright (c) 2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
  * for more details. */
 
+#define TEST_MACROS_ENABLEDoff
+#include <test_macros.h>
+
 #include <QMessageBox>
 
 #include "profileselector.h"
@@ -60,7 +63,7 @@ ProfileSelector::ProfileSelector(QWidget *parent) :
     ui->diskSpaceInfo->setVisible(false);
 
     QItemSelectionModel * sm = ui->profileView->selectionModel();
-    if (sm) 
+    if (sm)
         connect(sm, SIGNAL(currentRowChanged(QModelIndex,QModelIndex)), this, SLOT(on_selectionChanged(QModelIndex,QModelIndex)));
     ui->buttonEditProfile->setEnabled(false);
     ui->buttonOpenProfile->setEnabled(false);
@@ -125,14 +128,14 @@ void ProfileSelector::updateProfileList()
 
         Machine * mach = prof->GetMachine(MT_CPAP);  // only interested in last cpap machine...
         if (!mach) {
-            qDebug() << "Couldn't find machine info for" << name;
+            qDebug() << "Couldn't find device info for" << name;
         }
 
         model->insertRows(row, 1, QModelIndex());
         // Problem: Can't access profile details until it's loaded.
         QString usersname;
         if (!prof->user->lastName().isEmpty()) {
-            usersname = tr("%1, %2").arg(prof->user->lastName()).arg(prof->user->firstName());
+            usersname = QString("%1, %2").arg(prof->user->lastName()).arg(prof->user->firstName());
         }
 
         model->setData(model->index(row, 0, QModelIndex()), name);
@@ -302,14 +305,14 @@ void ProfileSelector::on_buttonEditProfile_clicked()
         Profile * prof = Profiles::profiles[name];
         //SelectProfile(name); // may not be necessary...
 
-        NewProfile *newprof = new NewProfile(this);
+        NewProfile *newprof = new NewProfile(this,&name);
         newprof->edit(name);
         newprof->setWindowModality(Qt::ApplicationModal);
         newprof->setModal(true);
         if (newprof->exec() != NewProfile::Rejected) {
             QString usersname;
             if (!prof->user->lastName().isEmpty()) {
-                usersname = tr("%1, %2").arg(prof->user->lastName()).arg(prof->user->firstName());
+                usersname = QString("%1, %2").arg(prof->user->lastName()).arg(prof->user->firstName());
             }
 
             proxy->setData(proxy->index(ui->profileView->currentIndex().row(), 5, QModelIndex()), usersname);
