@@ -1,18 +1,22 @@
 /* Reports/Printing Module
  *
- * Copyright (c) 2019-2020 The OSCAR Team
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
  * for more details. */
 
+#define TEST_MACROS_ENABLEDoff
+#include <test_macros.h>
+
 #include <QMessageBox>
 #include <QtPrintSupport/qprinter.h>
 #include <QtPrintSupport/qprintdialog.h>
 #include <QTextDocument>
 #include <QApplication>
 #include <cmath>
+#include <QPageLayout>
 
 #include "reports.h"
 #include "mainwindow.h"
@@ -31,6 +35,7 @@ void Report::PrintReport(gGraphView *gv, QString name, QDate date)
 {
     if (!gv) { return; }
 
+
     Session *journal = nullptr;
     //QDate d=QDate::currentDate();
 
@@ -78,10 +83,10 @@ void Report::PrintReport(gGraphView *gv, QString name, QDate date)
     printer->setOutputFileName(filename);
 #endif
     printer->setPrintRange(QPrinter::AllPages);
-    printer->setOrientation(QPrinter::Portrait);
+    printer->setPageOrientation(QPageLayout::Portrait);
     printer->setFullPage(false); // This has nothing to do with scaling
-    printer->setNumCopies(1);
-    printer->setPageMargins(10, 10, 10, 10, QPrinter::Millimeter);
+    printer->setCopyCount(1);
+    printer->setPageMargins(QMarginsF(10, 10, 10, 10), QPageLayout::Millimeter);
     QPrintDialog dialog(printer);
 #ifdef Q_OS_MAC
     // QTBUG-17913
@@ -104,9 +109,8 @@ void Report::PrintReport(gGraphView *gv, QString name, QDate date)
 
     GLint gw;
     gw = 2048; // Rough guess.. No GL_MAX_RENDERBUFFER_SIZE in mingw.. :(
-
     //QSizeF pxres=printer->paperSize(QPrinter::DevicePixel);
-    QRect prect = printer->pageRect();
+    QRect prect = printer->pageLayout().paintRectPixels( printer->resolution() ) ;
     float ratio = float(prect.height()) / float(prect.width());
     float virt_width = gw;
     float virt_height = virt_width * ratio;
@@ -208,8 +212,7 @@ void Report::PrintReport(gGraphView *gv, QString name, QDate date)
                 cpapinfo += /*QObject::tr("Pressure Relief")+": "+ */day->getPressureRelief() + "\n";
             }
 
-            float ahi = (day->count(CPAP_Obstructive) + day->count(CPAP_Hypopnea) +
-                         day->count(CPAP_ClearAirway) + day->count(CPAP_Apnea));
+            float ahi = day->count(AllAhiChannels);
 
             if (p_profile->general->calculateRDI()) { ahi += day->count(CPAP_RERA); }
 
@@ -219,6 +222,7 @@ void Report::PrintReport(gGraphView *gv, QString name, QDate date)
             //float pb = (100.0 / hours) * (day->sum(CPAP_PB) / 3600.0);
             float uai = day->count(CPAP_Apnea) / hours;
             float oai = day->count(CPAP_Obstructive) / hours;
+            float ai = day->count(CPAP_AllApnea) / hours;
             float hi = (day->count(CPAP_ExP) + day->count(CPAP_Hypopnea)) / hours;
             float cai = day->count(CPAP_ClearAirway) / hours;
             float rei = day->count(CPAP_RERA) / hours;
@@ -290,6 +294,8 @@ void Report::PrintReport(gGraphView *gv, QString name, QDate date)
             } else if (cpap->loaderName() == STR_MACH_Intellipap) {
                 stats += QObject::tr("NRI=%1 LKI=%2 EPI=%3")
                         .arg(nri, 0, 'f', 2).arg(lki, 0, 'f', 2).arg(exp, 0,'f', 2);
+            } else if (cpap->loaderName() == STR_MACH_SleepStyle) {
+                stats += QObject::tr("AI=%1 ").arg(ai, 0, 'f', 2);
             }
 
             bounds = painter.boundingRect(QRectF(0, top + ttop, virt_width, 0), stats,
@@ -550,7 +556,7 @@ void Report::PrintReport(gGraphView *gv, QString name, QDate date)
 
         if (first) {
             QDateTime timestamp = QDateTime::currentDateTime();
-            QString footer = QObject::tr("%1 %2 %3").arg(timestamp.toString(MedDateFormat+" hh:mm"))
+            QString footer = QString("%1 %2 %3").arg(timestamp.toString(MedDateFormat+" hh:mm"))
                                                            .arg(STR_TR_OSCAR)
                                                            .arg(getVersion());
 
