@@ -1,12 +1,15 @@
 /* Welcome page Implementation
  *
- * Copyright (c) 2019 The OSCAR Team
- * Copyright (c) 2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2024 The OSCAR Team
+ * Copyright (c) 2018 Mark Watkins 
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of Source Code. */
 
 #include <cmath>
+// remember to turn test_macros off for release.
+#define TEST_MACROS_ENABLEDoff
+#include <test_macros.h>
 
 #include "welcome.h"
 #include "ui_welcome.h"
@@ -54,7 +57,7 @@ void Welcome::refreshPage()
     ui->S9Warning->setVisible(showCardWarning);
 
     if (!b) {
-        qDebug() << "No machines in Profile";
+        qDebug() << "No devices in Profile";
 //        sleep(3);
         ui->cpapIcon->setPixmap(pixmap);
     }
@@ -151,15 +154,16 @@ QString Welcome::GenerateCPAPHTML()
     "p,a,td,body { font-family: '"+QApplication::font().family()+"'; }"
     "p,a,td,body { font-size: "+QString::number(QApplication::font().pointSize() + 2)+"px; }"
     "</style>"
-
     "</head>"
+
     "<body leftmargin=5 topmargin=10 rightmargin=5 bottommargin=5 vertical-align=center align=center>";
+    html += "<font size='+0'>" ;
 
     Machine * cpap = nullptr;
     if (!havecpapdata && !haveoximeterdata) {
         html += "<p>" + tr("It would be a good idea to check File->Preferences first,") + "<br />" +
                         tr("as there are some options that affect import.")+"</p>" +
-        "<p>" + tr("Note that some preferences are forced when a ResMed machine is detected") + "</p>" +
+        "<p>" + tr("Note that some preferences are forced when a ResMed device is detected") + "</p>" +
         "<p>" + tr("First import can take a few minutes.") + "</p>";
     } else {
         QDate date = p_profile->LastDay(MT_CPAP);
@@ -170,7 +174,6 @@ QString Welcome::GenerateCPAPHTML()
         }
         if (day && (cpap != nullptr)) {
             QString cpapimage = cpap->getPixmapPath();
-
             ui->cpapIcon->setPixmap(QPixmap(cpapimage));
 
             html+= "<b>"+tr("The last time you used your %1...").arg(cpap->brand()+" "+cpap->model())+"</b><br/>";
@@ -179,6 +182,7 @@ QString Welcome::GenerateCPAPHTML()
             QString daystring;
             if (daysto == 1) daystring += tr("last night");
             else if (daysto == 2) daystring += tr("1 day ago");
+            else if (daysto == 0) daystring += tr("today");
             else daystring += tr("%2 days ago").arg(daysto-1);
 
             html += tr("was %1 (on %2)").arg(daystring).arg(date.toString(Qt::SystemLocaleLongDate)) + "<br/>";
@@ -192,7 +196,7 @@ QString Welcome::GenerateCPAPHTML()
             QString timestr = tr("%1 hours, %2 minutes and %3 seconds").arg(hour).arg(minutes).arg(seconds);
 
             const EventDataType compliance_min = p_profile->cpap->m_complianceHours; // 4.0;
-            if (hours > compliance_min) html += tr("Your machine was on for %1.").arg(timestr)+"<br/>";
+            if (hours > compliance_min) html += tr("Your device was on for %1.").arg(timestr)+"<br/>";
             else html += tr("<font color = red>You only had the mask on for %1.</font>").arg(timestr)+"<br/>";
 
 
@@ -202,7 +206,8 @@ QString Welcome::GenerateCPAPHTML()
             QDate endtime = date.addDays(-1);
 
 
-            EventDataType ahi = (day->count(CPAP_Obstructive) + day->count(CPAP_Hypopnea) + day->count(CPAP_ClearAirway) + day->count(CPAP_Apnea)) / hours;
+//            EventDataType ahi = (day->count(CPAP_AllApnea) + day->count(CPAP_Obstructive) + day->count(CPAP_Hypopnea) + day->count(CPAP_ClearAirway) + day->count(CPAP_Apnea)) / hours;
+            EventDataType ahi = day->count(AllAhiChannels) / hours;
             EventDataType ahidays = calcAHI(starttime, endtime);
 
             const QString under = tr("under");
@@ -245,6 +250,9 @@ QString Welcome::GenerateCPAPHTML()
             if (day->channelHasData(CPAP_EPAPSet)) {
                 epapDataChanID = CPAP_EPAPSet;
             }
+            if (day->channelHasData(CPAP_EEPAP)) {
+                epapDataChanID = CPAP_EEPAP;
+            }
 
             if (pressChanID == NoChannel) {
                 qWarning() << "Unable to find pressure channel for welcome summary!";
@@ -253,7 +261,7 @@ QString Welcome::GenerateCPAPHTML()
                 pressSettingChanID = CPAP_Pressure;  // DreamStation ventilators report EPAP/IPAP data, but the setting is Pressure
                 EventDataType pressure = day->settings_max(pressSettingChanID);
                 qDebug() << pressSettingChanID << pressure;
-                html += tr("Your CPAP machine used a constant %1 %2 of air")
+                html += tr("Your CPAP device used a constant %1 %2 of air")
                         .arg(pressure)
                         .arg(schema::channel[pressChanID].units());
             } else if (cpapmode == MODE_APAP) {
@@ -266,14 +274,14 @@ QString Welcome::GenerateCPAPHTML()
 //                pressSettingChanID = CPAP_IPAP;
 //                EventDataType ipap = day->settings_max(pressSettingChanID);
 //                EventDataType epap = day->settings_min(CPAP_EPAP);
-                html += tr("Your machine used a constant %1-%2 %3 of air.")
+                html += tr("Your device used a constant %1-%2 %3 of air.")
                         .arg(day->validPressure(day->settings_min(CPAP_EPAP)))
                         .arg(day->validPressure(day->settings_max(CPAP_IPAP)))
                         .arg(schema::channel[CPAP_IPAP].units());
             } else if (cpapmode == MODE_BILEVEL_AUTO_FIXED_PS) {
                 EventDataType ipap = day->percentile(pressChanID, perc/100.0);
                 EventDataType epap = day->percentile(epapDataChanID, perc/100.0);
-                html += tr("Your machine was under %1-%2 %3 for %4% of the time.")
+                html += tr("Your device was under %1-%2 %3 for %4% of the time.")
                         .arg(epap)
                         .arg(ipap)
                         .arg(schema::channel[pressChanID].units())
@@ -294,20 +302,26 @@ QString Welcome::GenerateCPAPHTML()
 
                 html += tr("Your EPAP pressure was under %1 %2 for %3% of the time.").arg(epap).arg(schema::channel[epapDataChanID].units()).arg(perc)+"<br/>";
                 html += tr("Your IPAP pressure was under %1 %2 for %3% of the time.").arg(ipap).arg(schema::channel[pressChanID].units()).arg(perc);
+            } else if (cpapmode == MODE_TRILEVEL_AUTO_VARIABLE_PDIFF){
+                EventDataType ipap = day->percentile(pressChanID, perc/100.0);
+                EventDataType eepap = day->percentile(epapDataChanID, perc/100.0);
+
+                html += tr("Your EEPAP pressure was under %1 %2 for %3% of the time.").arg(eepap).arg(schema::channel[epapDataChanID].units()).arg(perc)+"<br/>";
+                html += tr("Your IPAP pressure was under %1 %2 for %3% of the time.").arg(ipap).arg(schema::channel[pressChanID].units()).arg(perc);
             }
             html += "<br/>";
 
             //EventDataType lat = day->timeAboveThreshold(CPAP_Leak, p_profile->cpap->leakRedline())/ 60.0;
             //EventDataType leaks = 1.0/hours * lat;
 
-            EventDataType leak = day->avg(CPAP_Leak);
-            EventDataType leakdays = p_profile->calcAvg(CPAP_Leak, MT_CPAP, starttime, endtime);
+            EventDataType leak = day->wavg(CPAP_Leak);
+            EventDataType leakdays = p_profile->calcWavg(CPAP_Leak, MT_CPAP, starttime, endtime);
 
             if ((leak < leakdays) && ((leakdays - leak) >= 0.1)) {
                 comp = under;
             } else if ((leak > leakdays) && ((leak - leakdays) >= 0.1)) {
                 comp = over;
-            } else if ((fabs(ahi > ahidays) >= 0.01) ) {
+            } else if ((fabs(leak - leakdays) >= 0.01) ) {
                 comp = close;
             } else {
                 comp = equal;
@@ -349,7 +363,9 @@ QString Welcome::GenerateOxiHTML()
     "p,a,td,body { font-size: "+QString::number(QApplication::font().pointSize() + 2)+"px; }"
     "</style>"
     "</head>"
+
     "<body leftmargin=5 topmargin=5 rightmargin=5 bottommargin=5 valign=center align=center>";
+    html += "<font size='+0'>" ;
 
     if (haveoximeterdata) {
         QDate oxidate=p_profile->LastDay(MT_OXIMETER);
