@@ -1,7 +1,7 @@
 /* OSCAR ZIP archive creation
  * Provides a Qt-convenient wrapper around miniz, see https://github.com/richgel999/miniz
  *
- * Copyright (c) 2020 The OSCAR Team
+ * Copyright (c) 2020-2024 The OSCAR Team
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
@@ -168,6 +168,14 @@ bool FileQueue::AddDirectory(const QString & path, const QString & prefix)
     QDir dir(path);
     if (!dir.exists() || !dir.isReadable()) {
         qWarning() << dir.canonicalPath() << "can't read directory";
+#if defined(Q_OS_MACOS)
+        // If this is a directory known to be protected by macOS "Full Disk Access" permissions,
+        // skip it but don't consider it an error.
+        static const QSet<QString> s_macProtectedDirs = { ".fseventsd", ".Spotlight-V100", ".Trashes" };
+        if (s_macProtectedDirs.contains(dir.dirName())) {
+            return true;
+        }
+#endif
         return false;
     }
     QString base = prefix;
@@ -190,14 +198,12 @@ bool FileQueue::AddDirectory(const QString & path, const QString & prefix)
             qWarning() << "skipping symlink" << canonicalPath << fi.symLinkTarget();
         } else if (fi.isDir()) {
             // Descend and recurse
-            ok = AddDirectory(canonicalPath, relative_path);
+            ok &= AddDirectory(canonicalPath, relative_path);
         } else {
             // Add the file to the zip
-            ok = AddFile(canonicalPath, relative_path);
-        }
-        if (!ok) {
-            break;
+            ok &= AddFile(canonicalPath, relative_path);
         }
+        // Don't stop in our tracks when we hit an error.
     }
 
     return ok;
@@ -269,7 +275,7 @@ const QString FileQueue::toString() const
 // ==================================================================================================
 // Static functions to abstract the details of miniz from the primary logic.
 
-#include "miniz.h"
+#include "SleepLib/thirdparty/miniz.h"
 
 // Callback for miniz to write compressed data
 static size_t zip_write(void *pOpaque, mz_uint64 /*file_ofs*/, const void *pBuf, size_t n)
